using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using Cursed.gob;
using Cursed.gob.mob;
using Cursed.input;
using Cursed.gob.tile;
using Cursed.gob.mob.kob;
using Cursed.sound;
using Cursed.editor;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using Cursed.input.ai; 
using Cursed.view.renders;
using System.Windows;
using Cursed.col;
using Cursed.geom;
using System.Xml;
using Cursed.ui;


namespace Cursed
{

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CursedGame : Microsoft.Xna.Framework.Game
    {

        GraphicsDeviceManager graphics;

        #region gamevars

        SpriteBatch spriteBatch;

        OrthoCamera camera;
        //CameraDetector cameraDetector;

        HUD hud;

        readonly object gobLock = new object();

        Dictionary<Render.RenderType, Render> renders;

        int ticksUntilActiveCheck;

        ICollection<Gob> gobsToAdd;
        ICollection<Gob> gobsToRemove;
        ICollection<Gob> gobs;

        ICollection<Mob> mobs;
        ICollection<Tile> tiles;
        Dictionary<string, List<Gob>> labeledGobs;

        ICollection<GobInfo> incomingLoadGobInfos;
        string loadLevelName;

        Druid druidPlayer;
        string spawnLabel;
        string currentLevel;

        ICollection<Controller> controllersToAdd;
        ICollection<Controller> controllersToRemove;
        ICollection<Controller> controllers;

        ICollider collider;

        Cursed.sound.SoundBank soundBank;
        SoundManager soundManager;

        string music;

        SaveUtil saveUtil;
        Cursed.view.Console console;
        Editor editor = null;

        #region temporary
        public static Color FogColor = new Color(.2f, .22f, .2f);
        #endregion

        Effect effect;
        Effect lifeEffect;
        public static TextureLib textureLib;

        bool paused;
        static Random random;

        Profiler profiler;
        bool f1down = false;
        bool pausable = false;

        public Menu currentMenu = null;
        
        //Post Processing
        LifeMap lifeMap;
        LifePostProcessor lifePP;
        RenderTarget2D PPTarget;
        Texture2D PPTexture;

        List<Label> labels;
        Label fpsLabel;

        float fadeCountDown;
        Texture2D fadeTexture;

        public Render LastRender;

        public static CursedGame TheGame;


        #endregion

        public CursedGame()
            : base()
        {
            TheGame = this;

            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            Components.Add(new GamerServicesComponent(this));

            profiler = new Profiler(true);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;

            //this.IsFixedTimeStep = true;
            //this.TargetElapsedTime = TimeSpan.FromMilliseconds(1000.0f / 65.0f);
            //graphics.SynchronizeWithVerticalRetrace = true;

            if (Const.FullScreen)
            {
                foreach (DisplayMode displayMode in GraphicsAdapter.DefaultAdapter.SupportedDisplayModes)
                {
                    
                        graphics.PreferredBackBufferWidth = displayMode.Width;
                        graphics.PreferredBackBufferHeight = displayMode.Height;
                }
                graphics.ToggleFullScreen();
                graphics.IsFullScreen = true;
            }
            graphics.ApplyChanges();
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
            Window.Title = "Cursed!";

            renders = new Dictionary<Render.RenderType, Render>();
            gobs = new HashSet<Gob>();
            mobs = new HashSet<Mob>();
            tiles = new HashSet<Tile>();
            labeledGobs = new Dictionary<string, List<Gob>>();
            controllers = new HashSet<Controller>();

            gobsToAdd = new HashSet<Gob>();
            gobsToRemove = new HashSet<Gob>();
            controllersToAdd = new HashSet<Controller>();
            controllersToRemove = new HashSet<Controller>();

            incomingLoadGobInfos = null;

            paused = false;
            random = new Random();

            druidPlayer = null;
            spawnLabel = "";

            saveUtil = new SaveUtil(this);

            //post processing
            lifeMap = new LifeMap(this);

            fadeCountDown = 0;
            loadLevelName = "";

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            ticksUntilActiveCheck = 0;
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            MediaPlayer.IsRepeating = true;

            camera = new OrthoCamera(Const.ViewSize, Window);

            effect = Content.Load<Effect>("effects/BasicEffect");
            lifeEffect = Content.Load<Effect>("effects/LifeEffect");

            labels = new List<Label>();
            labels.Add(fpsLabel = new Label("hi", new Vector2(600, 10), true, Color.White, Content.Load<SpriteFont>("fonts/FPS")));

            #region Texture and Animation Libs
            textureLib = new TextureLib(Content);
            #endregion

            #region Renders
            SpriteRender sr = new SpriteRender(GraphicsDevice, effect);
            renders[Render.RenderType.SpriteRender] = sr;

            TrailRender tr = new TrailRender(effect, GraphicsDevice);
            renders[Render.RenderType.TrailRender] = tr;

            DialogRender dr = new DialogRender(this);
            renders[Render.RenderType.DialogRender] = dr;

            LifebarRender lr = new LifebarRender(GraphicsDevice, effect);
            renders[Render.RenderType.LifebarRender] = lr;
            #endregion

            //post processing
            lifePP = new LifePostProcessor(10,10, camera, lifeEffect);
            PPTarget = new RenderTarget2D(GraphicsDevice, GraphicsDevice.Viewport.Width,
                GraphicsDevice.Viewport.Height,1, GraphicsDevice.DisplayMode.Format);

            fadeTexture = Content.Load<Texture2D>("Images/square");

            //collider = new QTCollider(this);
            collider = new Box2DCollider(this);
		            
            //incomingLoadGobInfos = Levels.StartScreen();
            MainMenu();
            //editor = new Editor(this, Vector2.Zero);

            soundBank = new Cursed.sound.SoundBank(Content);
            soundManager = new SoundManager();

            console = new Cursed.view.Console(this);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (!Const.UpdateWithDraw)
                doUpdate(gameTime);
        }
        protected void doUpdate(GameTime gameTime)
        {
            if (loadLevelName != "")
                loadReadyLevel();

            if (fadeCountDown > 0)
            {
                gameTime = new GameTime();
            }

            if (Window.ClientBounds.Width == 0)
                return;

            if (pausable && (Keyboard.GetState().IsKeyDown(Keys.P) || Keyboard.GetState().IsKeyDown(Keys.Escape)
                || GamePad.GetState(PlayerIndex.One).IsButtonDown(Buttons.Start))
                &&(editor == null || !Editor.active))
            {
                PauseMenu();
            }

            if (currentMenu != null)
            {
                currentMenu.Update(this, Mouse.GetState());
            }

            if (!f1down && Keyboard.GetState().IsKeyDown(Keys.F1))
            {
                profiler.Print();
            }
            f1down = Keyboard.GetState().IsKeyDown(Keys.F1);
            if (Keyboard.GetState().IsKeyDown(Keys.F2))
                profiler.Reset();
            profiler.Tick();

            fpsLabel.Text = "FPS: " + profiler.FPS;

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            if (gameTime.ElapsedGameTime.Milliseconds == 0)
                return;

            profiler.Start("doUpdate");

            checkLoadGobs();

            updateGobList();

            if (Paused)
                gameTime = new GameTime(TimeSpan.Zero, gameTime.ElapsedRealTime, TimeSpan.Zero, TimeSpan.Zero);

            foreach (Render render in renders.Values)
            {
                render.Reposition(camera);
            }

            profiler.Start("collider.Update");
            collider.Update(gameTime);
            profiler.Stop("collider.Update");

            Bounds activeRegion = ActiveRegion;
            Bounds inactiveRegion = InactiveRegion;

            profiler.Start("gob updates");
            foreach (Gob gob in gobs)
            {
                if ((gob.Active && gob.CollisionInfo == null) || ticksUntilActiveCheck == 0)
                {
                    bool active = gob == DruidPlayer || (gob is Wand && ((Wand)gob).Owner == DruidPlayer) || activeRegion.Intersects(gob.Bounds);
                    bool inactive = !inactiveRegion.Intersects(gob.Bounds);
                    bool justAdded = false;
                    if (active && gob.CollisionInfo == null)
                    {
                        //System.Console.WriteLine("adding active gob");
                        collider.AddGob(gob);
                        justAdded = true;
                    }
                    else if (inactive && gob.CollisionInfo != null)
                    {
                        //System.Console.WriteLine("removing inactive gob");
                        collider.RemoveGob(gob);
                    }

                    if (!active || (gob.Inert && !justAdded))
                    {
                        gob.Active = false;
                        continue;
                    }
                    else
                    {
                        gob.Active = true;
                    }
                }

                if (gob.Active)
                {
                    //profiler.Start("gob.Update");
                    gob.PreUpdate();
                    gob.Update(gameTime);
                    gob.PostUpdate();
                    //profiler.Stop("gob.Update");
                }
            }
            profiler.Stop("gob updates");

            if (ticksUntilActiveCheck == 0)
                ticksUntilActiveCheck = Const.TicksBetweenActiveChecks;
            else
                ticksUntilActiveCheck--;

            //profiler.Start("Controllers");
            foreach (Controller controller in controllers)
            {
                controller.act(gameTime);
            }
            //profiler.Stop("Controllers");

            camera.trackFocus();
            
            profiler.Start("lifeMap.UpdateDensities");
            lifeMap.UpdateDensities();
            profiler.Stop("lifeMap.UpdateDensities");

            base.Update(gameTime);
            profiler.Stop("doUpdate");
        }

        #region draw
        class GobDepthComparer : IComparer<Gob>
        {
            public int Compare(Gob g1, Gob g2)
            {
                float d = g1.GobInfo.Depth - g2.GobInfo.Depth;
                float y = g1.GobInfo.Loc.Y - g2.GobInfo.Loc.Y;

                //if (y > 0)
                //    d -= g1.DepthSpan;
                //else
                //    d -= g2.DepthSpan;

                if (d < 0)
                    return 1;
                if (d > 0)
                    return -1;
                if (y < 0)
                    return 1;
                if (y > 0)
                    return -1;
                return g1.GetHashCode() - g2.GetHashCode();
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected  override void Draw(GameTime gameTime)
        {

            if (Window.ClientBounds.Width == 0)
                return;

            if (Const.UpdateWithDraw)
                doUpdate(gameTime);


            profiler.Start("Draw");

            //===========================================
            //POST PROCESSING
            GraphicsDevice.SetRenderTarget(0, PPTarget);
            //===========================================

            GraphicsDevice.Clear(Const.PrimaryGreen);
            GraphicsDevice.RenderState.AlphaBlendEnable = true;
            GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
            GraphicsDevice.RenderState.DepthBufferEnable = false;


            SortedList<Gob, Gob> sortedGobs = new SortedList<Gob, Gob>(new GobDepthComparer());

            Bounds bounds = camera.Bounds;
            profiler.Start("gobs grab");
            ICollection<Gob> gobsInView = collider.GobsInBounds(bounds);
            foreach (Gob gob in gobsInView)
                sortedGobs.Add(gob, gob);
            profiler.Stop("gobs grab");

            profiler.Start("gobs draw");
            LastRender = null;
            foreach (KeyValuePair<Gob,Gob> gobPair in sortedGobs)
            {
                profiler.Start("gob.Draw");
                gobPair.Value.Draw(gameTime);
                profiler.Stop("gob.Draw");
            }
            if (LastRender != null)
                LastRender.End();
            profiler.Stop("gobs draw");


            profiler.Start("lifePP");
            //============================
            //POST PROCESSING
            GraphicsDevice.SetRenderTarget(0, null);
            PPTexture = PPTarget.GetTexture();
            lifePP.draw(GraphicsDevice, lifeMap, PPTexture);
            //============================
            profiler.Stop("lifePP");

            profiler.Start("misc draw");


            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Texture, SaveStateMode.SaveState);
            spriteBatch.Draw(textureLib.GetTexture("fog"),
                new Rectangle(
                    0,
                    0,
                    GraphicsDevice.Viewport.Width,
                    GraphicsDevice.Viewport.Height),
                    FogColor);
            spriteBatch.End();
            if (FogColor.R > 51.2)
            {
                FogColor = new Color((byte)(FogColor.R - 5), FogColor.G, FogColor.B);
            }

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            
            //if (hud != null && DruidPlayer != null && DruidPlayer.Loaded)
            //    hud.Draw(SpriteBatch, gameTime);

            console.Draw(spriteBatch);
           

            foreach (Label label in labels)
            {
                label.Draw(spriteBatch);
            }
            
            if (fadeCountDown > 0)
            {
                fadeCountDown -= gameTime.ElapsedGameTime.Milliseconds * 0.001f;

                float t = 1 - fadeCountDown / Const.LoadFadeTime;
                t = Math.Min(1, Math.Max(0, t));
                Color mask = new Color(0, 0, 0, t);
                Rectangle cb = new Rectangle(0, 0, Window.ClientBounds.Width, Window.ClientBounds.Height);
                spriteBatch.Draw(fadeTexture, cb, mask);
            }
            if (currentMenu != null)
            {
                currentMenu.Draw(spriteBatch);
                Vector2 pos = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);
                spriteBatch.Draw(textureLib.GetTexture("circle"), new Rectangle((int)pos.X - 5, (int)pos.Y - 5, 10, 10), Color.Blue);
                //System.Console.WriteLine(pos.ToString());
            }
            if (editor != null)
                editor.Draw(spriteBatch);
            spriteBatch.End();
           
            profiler.Stop("misc draw");

            base.Draw(gameTime);
            profiler.Stop("Draw");
        }
        #endregion

        #region insert

        public void MainMenu()
        {
            DruidPlayer = null;
           
            LoadLevel("StartScreen");
        }

        public void loadReadyLevel()
        {
            currentMenu = null;
            if (loadLevelName == "new")
            {
                incomingLoadGobInfos = new List<GobInfo>();
                loadLevelName = "";
                return;
            }
            if (loadLevelName == "StartScreen")
            {
                incomingLoadGobInfos = Levels.StartScreen();
                loadLevelName = "";
                currentMenu = new Menu(MenuType.MAIN_MENU, this);
                pausable = false;
                return;
            }

            XmlDocument saveDoc = new XmlDocument();
            try
            {
                saveDoc.Load("Content/levels/" + loadLevelName + ".xml");
            }
            catch (FileNotFoundException)
            {
                System.Console.WriteLine("Couldn't find level " + loadLevelName);
                incomingLoadGobInfos = Levels.StartScreen();
                loadLevelName = "";
                return;
            }
            XmlNode infoNodeList = saveDoc.FirstChild;
            while (infoNodeList.Name != "CursedSave")
                infoNodeList = infoNodeList.NextSibling;

            incomingLoadGobInfos = new List<GobInfo>();
            foreach (XmlNode infoNode in infoNodeList)
            {
                GobInfo info = GobInfo.CreateGobInfoFromXML(infoNode);
                incomingLoadGobInfos.Add(info);
            }

            if (DruidPlayer != null)
                incomingLoadGobInfos.Add(DruidPlayer.DruidInfo);

            currentLevel = loadLevelName;
            hud = new HUD();
            hud.AddComponent(new ManaBar(new Rectangle(0, 0, 200, 25), this.druidPlayer));
            ticksUntilActiveCheck = 0;
            loadLevelName = "";
            LifeMap.allGreen = false;
            
        }

        public void PauseMenu()
        {
            currentMenu = new Menu(MenuType.PAUSE, this);
            Paused = true;
        }
        public void LoadLevel(string identifier)
        {
            fadeCountDown = Const.LoadFadeTime;
            loadLevelName = identifier;
            pausable = true;
        }

        public void ReloadLevel()
        {
            LoadLevel(currentLevel);
        }

        public void SaveLevel(string identifier)
        {

            TextWriter outFile = new StreamWriter("../../../Content/levels/"+identifier + ".xml");
            outFile.Write("<CursedSave>\n");
            foreach (Gob gob in gobs)
            {
                if (gob.Transient)
                    continue;
                if (gob == DruidPlayer)
                    continue;
                GobInfo info = gob.GobInfo;
                string xml = info.XML;
                outFile.Write(xml);
            }
            outFile.Write("</CursedSave>\n");
            outFile.Close();
        }

        public ICollection<GobInfo> IncomingLoadGobInfos
        {
            set { incomingLoadGobInfos = value; }
        }

        void checkLoadGobs()
        {
            if (incomingLoadGobInfos != null)
            {
                clearAllObjects();
                Druid player = null;
                foreach (GobInfo info in incomingLoadGobInfos)
                {
                    Gob gob = info.Create(this);

                    if (gob is Druid)
                    {
                        Druid druid = (Druid)gob;
                        if (druid.DruidInfo.IsPlayer)
                            player = druid;
                    }

                    AddGob(gob);
                }
                incomingLoadGobInfos = null;
                updateGobList();

                Vector2 startLoc = Vector2.Zero;

                if (SpawnLabel != "" && DruidPlayer != null)
                {
                    ICollection<Gob> spawnGobs = GetLabeledGobs(SpawnLabel);
                    foreach (Gob gob in spawnGobs)
                    {
                        if (gob is Entrance)
                        {
                            startLoc = gob.GobInfo.Loc;
                        }
                    }
                }
                if (player == null && DruidPlayer != null)
                {
                    AddGob(DruidPlayer);
                }
                else
                {
                    DruidPlayer = player;
                }
                if (DruidPlayer != null)
                {
                    DruidPlayer.GobInfo.Loc = startLoc;
                    Camera.PushFocus(DruidPlayer);
                }

                Camera.SnapFocus();

                editor = new Editor(this, Vector2.Zero);
            }
        }

        void clearAllObjects()
        {
            foreach (Gob gob in gobs)
            {
                gob.Unload();
                gob.Loaded = false;
            }

            gobs.Clear();
            mobs.Clear();
            tiles.Clear();
            labeledGobs.Clear();
            gobsToAdd.Clear();
            gobsToRemove.Clear();

            controllersToAdd.Clear();
            controllersToRemove.Clear();
            foreach (Controller controller in controllers)
            {
                controller.cleanup();
            }
            controllers.Clear();

            collider.Clear();
        }
        void updateGobList()
        {
            lock (gobLock)
            {
                foreach (Controller controller in controllersToRemove)
                {
                    controllers.Remove(controller);
                }
                controllersToRemove.Clear();

                foreach (Controller controller in controllersToAdd)
                {
                    controllers.Add(controller);
                }
                controllersToAdd.Clear();


                foreach (Gob gob in gobsToRemove)
                {
                    if (gob is Tile)
                    {
                        tiles.Remove((Tile)gob);
                        collider.RemoveTile((Tile)gob);
                    }

                    if (gob is Mob)
                    {
                        mobs.Remove((Mob)gob);
                        collider.RemoveMob((Mob)gob);
                    }

                    if (gob.GobInfo.Label != null)
                    {
                        if (labeledGobs.ContainsKey(gob.GobInfo.Label))
                        {
                            labeledGobs[gob.GobInfo.Label].Remove(gob);
                        }
                    }

                    gobs.Remove(gob);

                    gob.Loaded = false;
                }
                gobsToRemove.Clear();

                foreach (Gob gob in gobsToAdd)
                {
                    if (gob is Tile)
                    {
                        tiles.Add((Tile)gob);
                        //collider.AddTile((Tile)gob);
                    }

                    if (gob is Mob)
                    {
                        mobs.Add((Mob)gob);
                        //collider.AddMob((Mob)gob);
                    }
                    if (gob.GobInfo.Label != "")
                    {
                        if (!labeledGobs.ContainsKey(gob.GobInfo.Label))
                        {
                            labeledGobs[gob.GobInfo.Label] = new List<Gob>();
                        }
                        labeledGobs[gob.GobInfo.Label].Add(gob);
                    }

                    gobs.Add(gob);
                    //gob.Update(new GameTime());
                }
                gobsToAdd.Clear();

            }
        }

        public void AddController(Controller controller)
        {
            if (controller == null)
            {
                return;
            }
            controllersToAdd.Add(controller);

        }

        public void RemoveController(Controller controller)
        {
            if (controller == null)
                return;
            controllersToRemove.Add(controller);
            controller.cleanup();
        }

        public void AddGob(Gob gob)
        {
            if (gob == null)
                return;
            gobsToAdd.Add(gob);
            gob.Loaded = true;
            gob.Load();
            gob.Update(new GameTime());
        }

        public void RemoveGob(Gob gob)
        {
            if (gob == null)
                return;
            if (gob == DruidPlayer)
                DruidPlayer = null;
            gobsToRemove.Add(gob);
            gob.Unload();
        }

        public ICollection<Gob> GetLabeledGobs(string label)
        {
            if (!labeledGobs.ContainsKey(label))
            {
                labeledGobs[label] = new List<Gob>();
            }
            return labeledGobs[label];
        }

        public Gob GetFirstLabeledGob(string label)
        {
            ICollection<Gob> gobs = GetLabeledGobs(label);
            if (gobs.Count == 0)
                return null;
            IEnumerator<Gob> genum = gobs.GetEnumerator();
            genum.MoveNext();
            return genum.Current;
        }

        public ICollection<Mob> MobsInBounds(Bounds bounds)
        {
            return collider.MobsInBounds(bounds);
        }

        public ICollection<Gob> GobsInBounds(Bounds bounds)
        {
            return collider.GobsInBounds(bounds);
        }

        public ICollection<Tile> TilesAt(int x, int y)
        {
            return collider.TilesAt(x, y);
        }

        public void CollectMobsInBounds(Bounds bounds, ICollection<Mob> pool)
        {
            collider.CollectMobsInBounds(bounds, pool);
        }

        public void CollectGobsInBounds(Bounds bounds, ICollection<Gob> pool)
        {
            collider.CollectGobsInBounds(bounds, pool);
        }

        #endregion

        #region accessors

        public Profiler Profiler
        {
            get { return profiler; }
        }

        public LifeMap LifeMap
        {
            get { return lifeMap; }
        }

        public string Music
        {
            get { return music; }
            set
            {
                return;
                if (music == value)
                    return;
                music = value;
                MediaPlayer.Stop();
                MediaPlayer.Play(Content.Load<Song>("music\\" + music));
            }
        }

        public Bounds ActiveRegion
        {
            get
            {
                //return Bounds.Everything;

                float w = Camera.ViewWidth + Const.ActiveRegionBuffer;
                float h = Camera.ViewHeight + Const.ActiveRegionBuffer;
                Bounds bounds = new Bounds(Camera.Position, w, h);

                //bounds.MinX = bounds.MinX + bounds.MinX % (Const.ActiveRegionBuffer / 2);
                //bounds.MinY = bounds.MinY + bounds.MinY % (Const.ActiveRegionBuffer / 2);
                //bounds.MaxX = bounds.MaxX - bounds.MaxX % (Const.ActiveRegionBuffer / 2);
                //bounds.MaxY = bounds.MaxY - bounds.MaxY % (Const.ActiveRegionBuffer / 2);

                return bounds;
            }
        }

        public Bounds InactiveRegion
        {
            get
            {
                //return Bounds.Everything;

                float w = Camera.ViewWidth + 3f * Const.ActiveRegionBuffer;
                float h = Camera.ViewHeight + 3f * Const.ActiveRegionBuffer;
                Bounds bounds = new Bounds(Camera.Position, w, h);

                //bounds.MinX = bounds.MinX + bounds.MinX % (Const.ActiveRegionBuffer / 2);
                //bounds.MinY = bounds.MinY + bounds.MinY % (Const.ActiveRegionBuffer / 2);
                //bounds.MaxX = bounds.MaxX - bounds.MaxX % (Const.ActiveRegionBuffer / 2);
                //bounds.MaxY = bounds.MaxY - bounds.MaxY % (Const.ActiveRegionBuffer / 2);

                return bounds;
            }
        }

        public Render getRender(Render.RenderType type)
        {
            return renders[type];
        }

        public bool Paused
        {
            get { return paused; }
            set { paused = value; }
        }
        public Cursed.sound.SoundBank SoundBank
        {
            get { return soundBank; }
        }
        public SoundManager SoundManager
        {
            get { return soundManager; }
        }
        public ICamera Camera
        {
            get { return camera; }
        }

        public static Random Random
        {
            get { return random; }
        }

        public Vector2 RandomVector2
        {
            get
            {
                Vector2 v = new Vector2(0, 1);
                float theta = (float)(Random.NextDouble() * MathHelper.Pi * 2);
                return Vector2.Transform(v, Matrix.CreateRotationZ(theta));
            }
        }
        public float RandomFloat
        {
            get
            {
                return (float)Random.NextDouble();
            }
        }
        public float RandomInt(int max)
        {
            return random.Next(max);
        }
        public Color RandomColor
        {
            get
            {
                return new Color(RandomFloat, RandomFloat, RandomFloat);
            }
        }

        public ICollection<Gob> Gobs
        {
            get { return gobs; }
        }

        public ICollection<Mob> Mobs
        {
            get { return mobs; }
        }

        public Druid DruidPlayer
        {
            get { return druidPlayer; }
            set { 
                druidPlayer = value;
                hud = new HUD();
                hud.AddComponent(new HealthBar(new Rectangle(0, 0, 125, 20), druidPlayer));
                hud.AddComponent(new ManaBar(new Rectangle(0, 20, 125, 20), druidPlayer));
                camera.PushFocus(druidPlayer);
            }
        }

        public string SpawnLabel
        {
            get { return spawnLabel; }
            set { spawnLabel = value; }
        }

        public ICollection<Tile> Tiles
        {
            get { return tiles; }
        }
        public SpriteBatch SpriteBatch
        {
            get { return spriteBatch; }
        }
        #endregion
    }
}
