﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Cursed
{
    public class Profiler
    {
        bool enabled;

        Dictionary<string, DateTime> startTimes;
        Dictionary<string, TimeSpan> totalTimes;
        Dictionary<string, TimeSpan> maxTimes;
        Dictionary<string, long> counts;
        long ticks;

        long lastFPSTick;
        DateTime lastFPSTickTime;
        float fps;

        public Profiler(bool enabled)
        {
            this.enabled = enabled;
            Reset();
        }

        public void Reset()
        {
            startTimes = new Dictionary<string, DateTime>();
            totalTimes = new Dictionary<string, TimeSpan>();
            maxTimes = new Dictionary<string, TimeSpan>();
            counts = new Dictionary<string, long>();
            ticks = 0;
            lastFPSTick = 0;
        }

        public void Tick()
        {
            ticks++;

            if (ticks - lastFPSTick > 30)
            {
                DateTime now = DateTime.Now; ;
                TimeSpan duration = now.Subtract(lastFPSTickTime);
                lastFPSTickTime = now;
                long durationMillis = duration.Milliseconds;
                durationMillis += 1000 * duration.Seconds;
                durationMillis += 60000 * duration.Minutes;
                fps = 30 / (durationMillis * 0.001f);
                lastFPSTick = ticks;
            }
        }

        public float FPS
        {
            get { return fps; }
        }

        public void Start(string label)
        {
            if (!enabled) return;
            startTimes[label] = DateTime.Now;
        }
        public long Stop(string label)
        {
            if (!enabled) return 0;
            if (!startTimes.ContainsKey(label))
                return 0;
            TimeSpan duration = DateTime.Now.Subtract(startTimes[label]);
            
            if (!totalTimes.ContainsKey(label))
            {
                totalTimes[label] = new TimeSpan();
                maxTimes[label] = new TimeSpan();
                counts[label] = 0;
            }
            totalTimes[label] = totalTimes[label].Add(duration);
            if (duration > maxTimes[label])
                maxTimes[label] = duration;
            counts[label]++;
            return duration.Milliseconds;
        }
        public void Print()
        {
            if (!enabled)
                System.Console.WriteLine("Profiler not enabled");
            System.Console.WriteLine("Profiler:");
            foreach (string label in totalTimes.Keys)
            {
                TimeSpan totalDuration = totalTimes[label];
                long totalDurationMillis = totalDuration.Milliseconds;
                totalDurationMillis += 1000 * totalDuration.Seconds;
                totalDurationMillis += 60000 * totalDuration.Minutes;
                long count = counts[label];
                float countPerTick = 1.0f * count / ticks;
                float time = 1.0f * totalDurationMillis / count;
                float perTick = 1.0f * totalDurationMillis / ticks;
                float sec = totalDurationMillis * .001f; 
                System.Console.WriteLine(" " + label + ": " + countPerTick+" calls/tick, " + perTick + "ms/tick, " + /*sec + "s/" + count + ", "*/ + time + "ms avg, " + maxTimes[label].Milliseconds + "ms max");
            }
        }

        public Boolean Enabled
        {
            get { return enabled; }
            set { enabled = value; }
        }
    }
}
