﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Storage;
using System.IO;
using System.Xml.Serialization;
using Cursed.gob;

namespace Cursed
{
    public class SaveUtil
    {

        public bool operationPending = false;
        CursedGame game;

        public SaveUtil(CursedGame game)
        {
            this.game = game;
        }

        public void Save()
        {
            operationPending = true;
            Guide.BeginShowStorageDeviceSelector(FindStorageDevice, "saveRequest");
        }

        public void Load()
        {
            operationPending = true;
            Guide.BeginShowStorageDeviceSelector(FindStorageDevice, "loadRequest");
        }

        private void FindStorageDevice(IAsyncResult result)
        {
            StorageDevice storageDevice = Guide.EndShowStorageDeviceSelector(result);
            if (storageDevice != null)
            {
                if (((string)result.AsyncState) == "saveRequest")
                    SaveGame(storageDevice);
                else if (((string)result.AsyncState) == "loadRequest")
                    LoadGame(storageDevice);
            }
        }



        private void LoadGame(StorageDevice storageDevice)
        {
            StorageContainer container = storageDevice.OpenContainer("Cursed!");
            string fileName = Path.Combine(container.Path, "save0001.sav");
            if (File.Exists(fileName))
            {
                FileStream saveFile = File.Open(fileName, FileMode.Open);
                XmlSerializer xmlSerializer = new XmlSerializer(typeof(ICollection<GobInfo>));

                ICollection<GobInfo> infos = (ICollection<GobInfo>)xmlSerializer.Deserialize(saveFile);
                saveFile.Close();
                operationPending = false;
            }
            container.Dispose();
        }

        private void SaveGame(StorageDevice storageDevice)
        {
            StorageContainer container = storageDevice.OpenContainer("Cursed!");
            string fileName = Path.Combine(container.Path, "save0001");

            FileStream saveFile = File.Open(fileName, FileMode.Create);

            List<GobInfo> savedGobInfos = new List<GobInfo>();
            foreach (Gob gob in game.Gobs)
            {
                if (gob.Transient)
                    continue;
                GobInfo info = gob.GobInfo;
                savedGobInfos.Add(info);
            }

            XmlSerializer xmlSerializer = new XmlSerializer(typeof(List<GobInfo>));
            xmlSerializer.Serialize(saveFile, savedGobInfos);
            saveFile.Close();
            container.Dispose();
            operationPending = false;
        }
    }
}
