﻿
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Cursed
{
    public delegate void TaskQueueEventHandler<TaskType>(TaskType task);
    public delegate void TaskQueueErrorHandler(Exception ex);

    public class TaskQueue<TaskType>
    {
        string m_name;
        List<TaskType> m_taskQueue;
        AutoResetEvent m_taskEvent;
        Thread m_taskThread;

        public event TaskQueueEventHandler<TaskType> ProcessTask;
        public event TaskQueueErrorHandler HandleError;

        public int MaxTasks;

        volatile bool IsActive = false;

        public TaskQueue()
        {
            m_name = "<no name>";
            m_taskQueue = new List<TaskType>();
            m_taskEvent = new AutoResetEvent(false);
            MaxTasks = Int32.MaxValue;
        }

        public TaskQueue(string taskQueueName)
        {
            m_name = taskQueueName;
            m_taskQueue = new List<TaskType>();
            m_taskEvent = new AutoResetEvent(false);
            MaxTasks = Int32.MaxValue;
        }

        public void PostTask(TaskType task)
        {
            lock (m_taskQueue)
            {
                m_taskQueue.Add(task);
                while (m_taskQueue.Count > MaxTasks)
                    m_taskQueue.RemoveAt(0);
                m_taskEvent.Set();
            }
        }

        public void Start()
        {
            if (m_taskThread == null)
            {
                m_taskThread = new Thread(TaskProcessor);
                IsActive = true;
                //m_taskThread.SetApartmentState(ApartmentState.STA);
                m_taskThread.IsBackground = true;
                m_taskThread.Name = m_name + "_Thread";
                m_taskThread.Start();
            }
        }

        public void Stop()
        {
            if (m_taskThread != null)
            {
                IsActive = false;
                m_taskThread.Join();
                m_taskThread = null;
            }
        }

        void TaskProcessor()
        {
            while (true)
            {
                if (!IsActive) break;

                if (!m_taskEvent.WaitOne(500, false)) continue;

                // Transfer pending events to eventArray
                TaskType[] taskArray = null;
                lock (m_taskQueue)
                {
                    taskArray = m_taskQueue.ToArray();
                    m_taskQueue.Clear();
                }

                if (ProcessTask != null)
                {
                    foreach (TaskType task in taskArray)
                    {
                        // Leave loop if inactive.
                        if (!IsActive) break;
                        try
                        {
                            //ProcessTask(task);
                            ProcessTask.Invoke(task);
                        }
                        catch (Exception ex)
                        {
                            if (HandleError != null)
                            {
                                HandleError.Invoke(ex);
                            }
                        }
                    }
                }

            }
        }

    }
}

