﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Cursed.gob;
using Cursed.gob.mob;
using Cursed.gob.tile;
using Microsoft.Xna.Framework;
using Box2D.XNA;
using Cursed.gob.mob.kob;

namespace Cursed.col
{
    public class ViewUserData
    {
        public Gob gob;
        public ViewUserData(Gob gob)
        {
            this.gob = gob;
        }
    }

    public class B2DCollisionInfo
    {
        public Body body;
        public HashSet<Contact> contacts;
        public B2DCollisionInfo(Body body)
        {
            this.body = body;
            this.contacts = new HashSet<Contact>();
        }
    }

    public class B2DContactListener : IContactListener
    {
        #region IContactListener Members

        public void BeginContact(Contact contact)
        {
            Fixture fixtureA = contact.GetFixtureA();
            Fixture fixtureB = contact.GetFixtureB();
            bool viewA = fixtureA.GetUserData() is ViewUserData;
            bool viewB = fixtureB.GetUserData() is ViewUserData;

            if (viewA != viewB)
            {
                if (viewA && fixtureB.GetUserData() is Mob)
                {
                    ((ViewUserData)fixtureA.GetUserData()).gob.SeenMobs.Add((Mob)fixtureB.GetUserData());
                }
                if (viewB && fixtureA.GetUserData() is Mob)
                {
                    ((ViewUserData)fixtureB.GetUserData()).gob.SeenMobs.Add((Mob)fixtureA.GetUserData());
                }
                return;
            }

            Gob gob1 = (Gob)fixtureA.GetUserData();
            Gob gob2 = (Gob)fixtureB.GetUserData();
            gob1.Touch(gob2);
            gob2.Touch(gob1);
            ((B2DCollisionInfo)gob1.CollisionInfo).contacts.Add(contact);
            ((B2DCollisionInfo)gob2.CollisionInfo).contacts.Add(contact);
        }

        public void EndContact(Contact contact)
        {
            Fixture fixtureA = contact.GetFixtureA();
            Fixture fixtureB = contact.GetFixtureB();
            bool viewA = fixtureA.GetUserData() is ViewUserData;
            bool viewB = fixtureB.GetUserData() is ViewUserData;

            if (viewA != viewB)
            {
                if (viewA && fixtureB.GetUserData() is Mob)
                {
                    ((ViewUserData)fixtureA.GetUserData()).gob.SeenMobs.Remove((Mob)fixtureB.GetUserData());
                }
                if (viewB && fixtureA.GetUserData() is Mob)
                {
                    ((ViewUserData)fixtureB.GetUserData()).gob.SeenMobs.Remove((Mob)fixtureA.GetUserData());
                }
                return;
            }

            Gob gob1 = (Gob)fixtureA.GetUserData();
            Gob gob2 = (Gob)fixtureB.GetUserData();
            gob1.Untouch(gob2);
            gob2.Untouch(gob1);
            ((B2DCollisionInfo)gob1.CollisionInfo).contacts.Remove(contact);
            ((B2DCollisionInfo)gob2.CollisionInfo).contacts.Remove(contact);
        }

        public void PostSolve(Contact contact, ref ContactImpulse impulse)
        {

        }

        public void PreSolve(Contact contact, ref Manifold oldManifold)
        {

        }

        #endregion
    }
    public class B2DContactFilter : IContactFilter
    {

        #region IContactFilter Members

        public bool RayCollide(object userData, Fixture fixture)
        {
            return true;
        }

        public bool ShouldCollide(Fixture fixtureA, Fixture fixtureB)
        {
            if ((fixtureA.GetUserData() is ViewUserData) != (fixtureB.GetUserData() is ViewUserData))
            {
                if (fixtureA.GetUserData() is Familiar || fixtureB.GetUserData() is Familiar)
                {
                    if (fixtureA.GetUserData() is LifeBlob || fixtureB.GetUserData() is LifeBlob)
                    {
                        System.Console.WriteLine("familiar and life blob");
                    }
                }
                return fixtureA.GetUserData() is Mob || fixtureB.GetUserData() is Mob;
            }

            Gob gob1 = (Gob)fixtureA.GetUserData();
            Gob gob2 = (Gob)fixtureB.GetUserData();
            if (gob1 == null || gob2 == null)
                return false;

            if (gob1 is Tile && gob2 is Tile)
            {
                return false;
            }

            if (gob1.IsEffect || gob2.IsEffect)
                return false;

            if (fixtureA.IsSensor() != fixtureB.IsSensor())
            {
                return true;
            }

            if (gob1.Transcendent || gob2.Transcendent)
                return false;

            return gob1.CanTouch(gob2) && gob2.CanTouch(gob1);
        }

        #endregion
    }

    public class Box2DCollider : QTCollider
    {
        World world;

        public Box2DCollider(CursedGame game)
            : base(game)
        {
            world = new World(Vector2.Zero, true);
            world.ContactFilter = new B2DContactFilter();
            world.ContactListener = new B2DContactListener();
        }

        void b2dAddGob(Gob gob, bool isFixed)
        {
            if (world == null)
                return;

            IShape shape = gob.Shape;
            shape.setTransform(Vector2.Zero, 0, 1);

            FixtureDef fixDef = new FixtureDef();

            float scale = 1;
            if (gob is Mob)
                scale = ((Mob)gob).MobInfo.Scale;

            if (shape is Rect)
            {
                Rect rect = (Rect)shape;
                PolygonShape poly = new PolygonShape();
                poly.SetAsBox(rect.Width / 2, scale * rect.Height / 2, scale * rect.Center, 0);
                poly._centroid = Vector2.Zero;
                fixDef.shape = poly;
                fixDef.friction = 0;
            }
            else if (shape is Circle)
            {
                Circle circle = (Circle)shape;
                CircleShape cshape = new CircleShape();
                cshape._radius = scale * circle.Radius;
                cshape._p = circle.Center;
                fixDef.shape = cshape;
                fixDef.friction = 0;
            }
            else
            {
                //complain/impossible
                return;
            }

            Body b2dBody;

            BodyDef b2dBodyDef = new BodyDef();
            if (gob is Tile)
                b2dBodyDef.type = BodyType.Static;
            else if (gob.IsEffect)
                b2dBodyDef.type = BodyType.Kinematic;
            else
                b2dBodyDef.type = BodyType.Dynamic;

            b2dBody = world.CreateBody(b2dBodyDef);

            b2dBody.SetUserData(gob);

            b2dBody.CreateFixture(fixDef).SetUserData(gob);
            fixDef.isSensor = true;
            b2dBody.CreateFixture(fixDef).SetUserData(gob);

            MassData md = new MassData();
            md = new MassData();
            if (!isFixed)
            {
                md.mass = ((Mob)gob).MobInfo.Mass;
                md.i = Single.MaxValue;
            }
            else
            {
                md.mass = Single.MaxValue;
                md.i = Single.MaxValue;
            }
            b2dBody.SetMassData(ref md);

            b2dBody.Position = gob.GobInfo.Loc;

            gob.CollisionInfo = new B2DCollisionInfo(b2dBody);

        }

        #region ICollider Members

        public override void Clear()
        {
            
            world = new World(Vector2.Zero, true);
            world.ContactFilter = new B2DContactFilter();
            world.ContactListener = new B2DContactListener();

            /*
            if (world != null)
            {
                for (Body body = world.GetBodyList(); body != null; body = body.GetNext())
                {
                    world.DestroyBody(body);
                }
            }
             */
            /*
            if (mobs != null)
            {
                foreach (Mob mob in mobs)
                {
                    world.DestroyBody(((B2DCollisionInfo)mob.CollisionInfo).body);
                }
            }
            if (tiles != null)
            {
                foreach (Tile tile in tiles)
                {
                    world.DestroyBody(((B2DCollisionInfo)tile.CollisionInfo).body);
                }
            }
             */
            base.Clear();
        }

        public override void HandleCollisions()
        {

        }

        public override void Update(GameTime gameTime)
        {
            //CursedGame.TheGame.Profiler.Start("buildQT");
            //buildQT();
            //CursedGame.TheGame.Profiler.Stop("buildQT");
            /*
            foreach (Mob mob in mobs)
            {
                mob.clearTouch();
            }
            foreach (Tile tile in tiles)
            {
                tile.clearTouch();
            }
            */
            world.ClearForces();

            float seconds = gameTime.ElapsedGameTime.Milliseconds * .001f;

            //CursedGame.TheGame.Profiler.Start("prepare mobs");
            foreach (Mob mob in mobs)
            {
                if (mob.CollisionInfo == null)
                    continue;
                Body b2dBody = ((B2DCollisionInfo)mob.CollisionInfo).body;
                b2dBody.Position = mob.GobInfo.Loc;
                b2dBody.SetLinearVelocity(mob.MobInfo.Vel);
                b2dBody.Rotation = mob.MobInfo.Theta;
                b2dBody.SetAngularVelocity(mob.MobInfo.Omega);
                b2dBody.ApplyForce(mob.Force, mob.GobInfo.Loc);
                mob.UpdatePosition(seconds);
            }
            //CursedGame.TheGame.Profiler.Stop("prepare mobs");

            //CursedGame.TheGame.Profiler.Start("update contacts");
            for (Contact contact = world.GetContactList(); contact != null; contact = contact.GetNext())
            {
                Gob gob1 = (Gob)contact.GetFixtureA().GetBody().GetUserData();
                Gob gob2 = (Gob)contact.GetFixtureB().GetBody().GetUserData();
                B2DCollisionInfo cinfo1 = (B2DCollisionInfo)gob1.CollisionInfo;
                B2DCollisionInfo cinfo2 = (B2DCollisionInfo)gob2.CollisionInfo;
                cinfo1.contacts.Add(contact);
                cinfo2.contacts.Add(contact);
            }
            //CursedGame.TheGame.Profiler.Stop("update contacts");

            //CursedGame.TheGame.Profiler.Start("world.Step");
            world.Step(seconds, 5, 5);
            //CursedGame.TheGame.Profiler.Stop("world.Step");

            //CursedGame.TheGame.Profiler.Start("debrief mobs");
            foreach (Mob mob in mobs)
            {
                if (mob.CollisionInfo == null)
                    continue;
                Body b2dBody = ((B2DCollisionInfo)mob.CollisionInfo).body;
                if (!mob.MobInfo.Stuck)
                {
                    mob.GobInfo.Loc = b2dBody.Position;
                    mob.MobInfo.Vel = b2dBody.GetLinearVelocity();
                    mob.MobInfo.Theta = b2dBody.GetAngle();
                    mob.MobInfo.Omega = b2dBody.GetAngularVelocity();
                }
            }
            //CursedGame.TheGame.Profiler.Stop("debrief mobs");
        }

        public override void AddGob(Gob gob)
        {
            if (gob is Mob)
            {
                AddMob((Mob)gob);
            }
            if (gob is Tile)
            {
                AddTile((Tile)gob);
            }
        }

        public override void RemoveGob(Gob gob)
        {
            if (gob is Mob)
            {
                RemoveMob((Mob)gob);
            }
            if (gob is Tile)
            {
                RemoveTile((Tile)gob);
            }
        }

        public override void AddMob(Cursed.gob.mob.Mob mob)
        {
            base.AddMob(mob);
            b2dAddGob(mob, mob.MobInfo.Stuck);
        }

        public override void RemoveMob(Cursed.gob.mob.Mob mob)
        {
            if (mob.CollisionInfo == null)
                return;
            Body body = ((B2DCollisionInfo)mob.CollisionInfo).body;
            world.DestroyBody(body);
            mob.CollisionInfo = null;
            base.RemoveMob(mob);
        }

        public override void AddTile(Cursed.gob.tile.Tile tile)
        {
            base.AddTile(tile);
            b2dAddGob(tile, true);
        }

        public override void RemoveTile(Cursed.gob.tile.Tile tile)
        {
            if (tile.CollisionInfo == null)
                return;
            Body body = ((B2DCollisionInfo)tile.CollisionInfo).body;
            world.DestroyBody(body);
            tile.CollisionInfo = null;
            base.RemoveTile(tile);
        }

        Bounds collectionBounds;
        ICollection<Gob> gobCollection;
        ICollection<Mob> mobCollection;
        bool onlyMobs;
        bool AddToCollection(Fixture fixture)
        {
            Gob gob = (Gob)fixture.GetUserData();
            if (!gob.Bounds.Intersects(collectionBounds))
                return true;
            if (onlyMobs)
            {
                if (gob is Mob)
                    mobCollection.Add((Mob)gob);
            }
            else
            {
                gobCollection.Add(gob);
            }
            return true;
        }

        public override ICollection<Gob> GobsInBounds(Bounds bounds)
        {
            collectionBounds = bounds;
            gobCollection = new HashSet<Gob>();
            onlyMobs = false;

            AABB aabb = new AABB();
            aabb.lowerBound = new Vector2(bounds.MinX, bounds.MinY);
            aabb.upperBound = new Vector2(bounds.MaxX, bounds.MaxY);
            world.QueryAABB(AddToCollection, ref aabb);
            return gobCollection;
        }

        public override ICollection<Mob> MobsInBounds(Bounds bounds)
        {
            collectionBounds = bounds;
            mobCollection = new HashSet<Mob>();
            onlyMobs = true;

            AABB aabb = new AABB();
            aabb.lowerBound = new Vector2(bounds.MinX, bounds.MinY);
            aabb.upperBound = new Vector2(bounds.MaxX, bounds.MaxY);
            world.QueryAABB(AddToCollection, ref aabb);
            return mobCollection;
        }

        public override void CollectMobsInBounds(Bounds bounds, ICollection<Mob> pool)
        {
            collectionBounds = bounds;
            mobCollection = pool;
            onlyMobs = true;

            AABB aabb = new AABB();
            aabb.lowerBound = new Vector2(bounds.MinX, bounds.MinY);
            aabb.upperBound = new Vector2(bounds.MaxX, bounds.MaxY);
            world.QueryAABB(AddToCollection, ref aabb);
        }

        public override void CollectGobsInBounds(Bounds bounds, ICollection<Gob> pool)
        {
            collectionBounds = bounds;
            gobCollection = pool;
            onlyMobs = false;

            AABB aabb = new AABB();
            aabb.lowerBound = new Vector2(bounds.MinX, bounds.MinY);
            aabb.upperBound = new Vector2(bounds.MaxX, bounds.MaxY);
            world.QueryAABB(AddToCollection, ref aabb);
        }

        #endregion
    }
}
