﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Cursed.geom
{
    public struct Bounds
    {
        float minX;
        float maxX;
        float minY;
        float maxY;

        public static Bounds Everything
        {
            get
            {
                return new Bounds(Single.NegativeInfinity, Single.NegativeInfinity, Single.PositiveInfinity, Single.PositiveInfinity);
            }
        }
        public static Bounds Nothing
        {
            get
            {
                return new Bounds(Single.PositiveInfinity, Single.PositiveInfinity, Single.NegativeInfinity, Single.NegativeInfinity);
            }
        }

        public Bounds(System.Windows.Rect swr)
        {
            minX = (float)swr.Left;
            maxX = (float)swr.Right;
            minY = (float)swr.Top;
            maxY = (float)swr.Bottom;
        }

        public Bounds(float minX, float minY, float maxX, float maxY)
        {
            this.minX = minX;
            this.maxX = maxX;
            this.minY = minY;
            this.maxY = maxY;
        }

        public Bounds(Vector2 center, float width, float height)
        {
            minX = center.X - width / 2;
            maxX = minX + width;
            minY = center.Y - height / 2;
            maxY = minY + height;
        }

        public bool Equals(Bounds other)
        {
            return minX == other.minX && maxX == other.maxX && minY == other.minY && maxY == other.maxY;
        }

        public override string ToString()
        {
            return "[(" + minX + "," + minY + "), (" + maxX + "," + maxY + ")]";
        }

        public bool Contains(Bounds other)
        {
            return other.minX >= minX && other.minY >= minY && other.maxX <= maxX && other.maxY <= maxY;
        }

        public bool Intersects(Bounds other)
        {
            bool intX = maxX > other.minX && other.MaxX > minX;
            bool intY = maxY > other.minY && other.MaxY > minY;
            return intX && intY;
        }

        public static Bounds Union(Bounds first, Bounds second)
        {
            return new Bounds(
                Math.Min(first.minX, second.MinX),
                Math.Min(first.minY, second.MinY),
                Math.Max(first.maxX, second.MaxX),
                Math.Max(first.maxY, second.MaxY));
        }

        public bool Contains(Vector2 p)
        {
            return minX <= p.X && maxX >= p.X && minY <= p.Y && maxY >= p.Y;
        }

        public Vector2 Center
        {
            get { return new Vector2((minX + maxX) * .5f, (minY + maxY) * .5f); }
        }

        public float Width
        {
            get { return maxX - minX; }
        }
        public float Height
        {
            get { return maxY - minY; }
        }

        public float MinX
        {
            get { return minX; }
            set { minX = value; }
        }
        public float MaxX
        {
            get { return maxX; }
            set { maxX = value; }
        }
        public float MinY
        {
            get { return minY; }
            set { minY = value; }
        }
        public float MaxY
        {
            get { return maxY; }
            set { maxY = value; }
        }
    }
}
