﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using System.Windows;
using Cursed.geom;
using System.Reflection;
using System.Runtime.Serialization;
using Cursed.view.renders;
using Cursed.gob.mob;
using Cursed.gob.tile;
using System.Xml;
using Cursed.gob.mob.kob;
using Cursed.ui;
using Box2D.XNA;
using Cursed.col;
using Cursed.gob.mob.effects;
using Cursed.input.ai;

namespace Cursed.gob
{
    public class GobInfo
    {
        string label;
        Vector2 loc;
        float viewRange;
        // the lower the depth, the closer it is to the camera
        float depth;
        bool triggered;
        string texture;
        float transparency;

        //temporaries to allow larger images than the bounds
        Vector2 drawCenter;
        float drawWidth;
        float drawHeight;

        Color mask;

        string preset;

        public GobInfo()
        {
            label = "";
            loc = Vector2.Zero;
            this.depth = 0;
            viewRange = 0;
            triggered = false;
            texture = "";
            mask = new Color(1f, 1f, 1f, 1f);
            Transparency = 1;
            drawCenter = Vector2.Zero;
            drawWidth = 1;
            drawHeight = 1;
            preset = "";
        }
        public static GobInfo getPresetInfo(string infoType)
        {
            if (infoType == "tree")
            {
                WallInfo info = new WallInfo(0, 0);
                info.Preset = "Tree";
                info.Depth = Const.TreeDepth - (float)CursedGame.Random.NextDouble() * .1f;
                return info;
            }
            if (infoType == "shrub")
            {
                WallInfo info = new WallInfo(0, 0);
                info.Preset = "Shrub";
                info.IsShort = true;
                info.Depth = Const.ShrubDepth - (float)CursedGame.Random.NextDouble() * .1f;
                return info;
            }
            if (infoType == "swater")
            {
                WaterInfo info = new WaterInfo(0, 0);
                info.Preset = "ShallowWater";
                info.Deep = false;
                return info;
            }
            if (infoType == "dwater")
            {
                WaterInfo info = new WaterInfo(0, 0);
                info.Preset = "DeepWater";
                info.Deep = true;
                return info;
            }
            if (infoType == "bridgens")
            {
                BridgeInfo info = new BridgeInfo(0, 0);
                info.Preset = "BridgeNS";
                return info;
            }
            if (infoType == "bridgeew")
            {
                BridgeInfo info = new BridgeInfo(0, 0);
                info.Preset = "BridgeEW";
                return info;
            }
            if (infoType == "rock")
            {
                WallInfo info = new WallInfo(0, 0);
                info.Preset = "Rock";
                return info;
            }
            if (infoType == "boss")
            {
                DemonInfo dmi = new DemonInfo();
                dmi.MaxHealth = 5;
                dmi.CurrentHealth = 5;
                dmi.Scale = 1.8f;
                dmi.Thrust = 8;
                dmi.StrengthFactor = 2;
                dmi.ViewRange = 12;
                dmi.LungeDuration = 1;
                dmi.LungeFactor = 4;
                return dmi;
            }
            if (infoType == "assassin")
            {

                DemonInfo dmi = new DemonInfo();
                dmi.Transparency = .2f;
                dmi.Scale = .45f;
                return dmi;
            }
            if (infoType == "bookcase")
            {
                WallInfo info = new WallInfo(0, 0);
                info.Preset = "Bookcase";
                return info;
            }
            if (infoType == "DemonInfo" || infoType == "minion")
            {
                DemonInfo info = new DemonInfo();
                return info;
            }
            return null;
        }

        public static Type GetInfoType(string name)
        {
            switch (name)
            {
                case "DemonInfo":
                    return typeof(DemonInfo);
                case "DruidInfo":
                    return typeof(DruidInfo);
                case "FamiliarInfo":
                    return typeof(FamiliarInfo);
                case "DoorInfo":
                    return typeof(DoorInfo);
                case "FootSwitchInfo":
                    return typeof(FootSwitchInfo);
                case "GroundInfo":
                    return typeof(GroundInfo);
                case "WallInfo":
                    return typeof(WallInfo);
                case "TriggerZoneInfo":
                    return typeof(TriggerZoneInfo);
                case "EntranceInfo":
                    return typeof(EntranceInfo);
                case "ExitInfo":
                    return typeof(ExitInfo);
                case "CraniumRatInfo":
                    return typeof(CraniumRatInfo);
                case "UINewGameTileInfo":
                    return typeof(UINewGameTileInfo);
                case "UIFamiliarInfo":
                    return typeof(UIFamiliarInfo);
                case "ObstructionInfo":
                    return typeof(ObstructionInfo);
                case "DialogTriggerZoneInfo":
                    return typeof(DialogTriggerZoneInfo);
                case "FogInfo":
                    return typeof(FogInfo);
                case "StickerInfo":
                    return typeof(StickerInfo);
                case "WarrenInfo":
                    return typeof(WarrenInfo);
                case "WaterInfo":
                    return typeof(WaterInfo);
                case "BridgeInfo":
                    return typeof(BridgeInfo);
                case "SpawnInfo":
                    return typeof(SpawnInfo);
                case "CrystalInfo":
                    return typeof(CrystalInfo);
            }
            return null;
        }

        public static GobInfo Instantiate(string typeName)
        {
            Type infoType = GobInfo.GetInfoType(typeName);
            if (infoType == null)
                return null;
            Type[] paramTypes = { };
            object[] paramValues = { };
            if (infoType.IsSubclassOf(typeof(TileInfo)))
            {
                paramTypes = new Type[2];
                paramTypes[0] = typeof(Int32);
                paramTypes[1] = typeof(Int32);
                paramValues = new object[2];
                paramValues[0] = 0;
                paramValues[1] = 0;
            }
            return (GobInfo)infoType.GetConstructor(paramTypes).Invoke(paramValues);
        }

        public void SetField(string fieldName, string fieldValue)
        {
            FieldInfo fieldInfo = null;
            Type type = GetType();
            while (fieldInfo == null && type != typeof(object))
            {
                fieldInfo = type.GetField(fieldName,
                    BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                type = type.BaseType;
            }
            if (fieldInfo == null)
                return;
            Type fieldType = fieldInfo.FieldType;
            try
            {
                if (fieldType == typeof(int))
                {
                    int intValue = Int32.Parse(fieldValue);

                    fieldInfo.SetValue(this, intValue);
                }
                if (fieldType == typeof(float))
                {
                    float floatValue = Single.Parse(fieldValue);
                    fieldInfo.SetValue(this, floatValue);
                }
                if (fieldType == typeof(bool))
                {
                    bool boolValue = Boolean.Parse(fieldValue);
                    fieldInfo.SetValue(this, boolValue);
                }
                if (fieldType == typeof(Vector2))
                {
                    string[] coords = fieldValue.Split(',');
                    float x = Single.Parse(coords[0].Trim());
                    float y = Single.Parse(coords[1].Trim());
                    Vector2 v = new Vector2(x, y);
                    fieldInfo.SetValue(this, v);
                }
                if (fieldType == typeof(string))
                {
                    fieldInfo.SetValue(this, fieldValue);
                }
                if (fieldType == typeof(Color))
                {
                    string[] values = fieldValue.Split(',');
                    int R = int.Parse(values[0].Trim());
                    int G = int.Parse(values[1].Trim());
                    int B = int.Parse(values[2].Trim());
                    int A = 255;
                    if (values.Length == 4)
                        A = int.Parse(values[3].Trim());
                    fieldInfo.SetValue(this, new Color(R,G,B,A));
                }
                if (fieldType == typeof(Line))
                {
                    string[] values = fieldValue.Split('\\');
                    Line l = new Line();
                    l.Label = values[0];
                    l.Length = Single.Parse(values[1]);
                    l.StartTime = Single.Parse(values[2]);
                    l.Text = values[3];
                }
            }
            catch (FormatException)
            {

            }
        }

        public static GobInfo CreateGobInfoFromXML(XmlNode node)
        {
            GobInfo info = Instantiate(node.Name);

            //float d = info.Depth;

            foreach (XmlNode attrNode in node)
            {
                string fieldName = attrNode.Name;
                string fieldValue = attrNode.InnerText;
                info.SetField(fieldName, fieldValue);
            }

            //info.Depth = d;

            return info;
        }

        public string XML
        {
            get
            {
                StringBuilder sb = new StringBuilder();

                sb.Append(" <" + GetType().Name + ">\n");


                Type type = GetType();
                while (type != typeof(object))
                {
                    FieldInfo[] fields = type.GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                    foreach (FieldInfo field in fields)
                    {
                        Type fieldType = field.FieldType;
                        string name = field.Name;
                        object value = field.GetValue(this);
                        if (fieldType == typeof(Vector2))
                        {
                            Vector2 v2 = (Vector2)value;
                            value = "" + v2.X + "," + v2.Y;
                        }
                        if (fieldType == typeof(Color))
                        {
                            Color c = (Color)value;
                            value = "" + c.R+","+c.G+","+c.B+","+c.A;
                        }
                        if (fieldType == typeof(Line))
                        {
                            Line l = (Line)value;
                            value = l.Label + "\\" + l.Length + "\\" + l.StartTime + "\\" + l.Text;
                        }
                        sb.Append("  <" + name + ">" + value + "</" + name + ">\n");
                    }
                    type = type.BaseType;
                }

                sb.Append(" </" + GetType().Name + ">\n");

                return sb.ToString();
            }
        }

        public GobInfo Clone()
        {
            Type type = GetType();

            GobInfo clone;

            if (this is TileInfo)
            {
                TileInfo me = (TileInfo)this;
                Type[] constructorParamTypes = { typeof(int), typeof(int) };
                ConstructorInfo constructor = type.GetConstructor(constructorParamTypes);
                object[] constructorParams = { me.X, me.Y };
                clone = (GobInfo)constructor.Invoke(constructorParams);
            }
            else
            {
                Type[] constructorParamTypes = { };
                ConstructorInfo constructor = type.GetConstructor(constructorParamTypes);
                object[] constructorParams = { };
                clone = (GobInfo)constructor.Invoke(constructorParams);
            }

            while (type != typeof(object))
            {
                FieldInfo[] fields = type.GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                foreach (FieldInfo field in fields)
                {
                    object value = field.GetValue(this);
                    field.SetValue(clone, value);
                }
                type = type.BaseType;
            }
            return clone;
        }

        public virtual Gob Create(CursedGame game)
        {
            throw new NotImplementedException();
        }

        public float ViewRange
        {
            get { return viewRange; }
            set { viewRange = value; }
        }
        public string Label
        {
            get { return label; }
            set { label = value; }
        }
        public Color Mask
        {
            get { return mask; }
            set 
            { 
                mask = value;
                transparency = mask.A / 255f;
            }
        }
        public virtual Vector2 Loc
        {
            get { return loc; }
            set { loc = value; }
        }

        public Vector3 Loc3
        {
            get { return new Vector3(loc.X, loc.Y, Depth); }
            set
            {
                loc.X = value.X;
                loc.Y = value.Y;
                Depth = value.Z;
            }
        }

        public float Depth
        {
            get { return depth; }
            set { depth = value; }
        }

        public bool Triggered
        {
            get { return triggered; }
            set { triggered = value; }
        }

        public string Texture
        {
            get { return texture; }
            set { texture = value; }
        }
        public float Transparency
        {
            get { return transparency; }
            set 
            { 
                transparency = value;
                mask.A = (byte)(int)(255 * value);
            }
        }
        public Vector2 DrawCenter
        {
            get { return drawCenter; }
            set { drawCenter = value; }
        }
        public float DrawWidth
        {
            get { return drawWidth; }
            set { drawWidth = value; }
        }
        public float DrawHeight
        {
            get { return drawHeight; }
            set { drawHeight = value; }
        }
        public string Preset
        {
            get { return preset; }
            set { preset = value; }
        }
    }

    public class RenderPair
    {
        public Render render;
        public RenderInfo info;
        public RenderPair(Render render, RenderInfo info)
        {
            this.render = render;
            this.info = info;
        }
    }

    public class Gob : DrawableGameComponent
    {
        //used by box2d to tie this Gob to a Box2D object
        protected object collisionInfo;

        protected GobInfo info;
        protected List<RenderPair> renders;

        //the render info for GobInfo.Preset
        protected RenderInfo renderInfo;
        
        // if something is transcendant, then it does not participate in physics collisions
        bool transcendant;
        //if transient, don't save/send 
        bool transient;
        //inert gobs don't have Update called
        bool inert;
        //gobs that are effects don't interact with anything else, they just update/draw
        bool isEffect;

        //this gob is in the game currently
        bool loaded;
        //this gob is in the active region
        bool active;

        float depthSpan;

        FindMe findMe;
        FindMe findMe2;
        FindMe findMeFlying;
        FindMe findMe2Flying;

        FindMe findMeWet;
        FindMe findMe2Wet;
        FindMe findMeFlyingWet;
        FindMe findMe2FlyingWet;

        /*
         * the zero-offset, zero-rotation shape used for collision detection.
         */
        protected IShape shape;
        protected HashSet<Gob> touchedGobs;

        HashSet<Mob> seenMobs;
        HashSet<Mob> unseenMobs;

        float timeToNextScan = 0;

        protected float coolDown;

        public Gob(Game game, GobInfo info)
            : base(game)
        {
            collisionInfo = null;
            this.info = info;
            renders = new List<RenderPair>();
            touchedGobs = new HashSet<Gob>();
            seenMobs = new HashSet<Mob>();
            unseenMobs = new HashSet<Mob>();
            transcendant = true;
            transient = false;
            inert = false;
            isEffect = false;

            coolDown = 0;
            loaded = false;
            active = true;

            depthSpan = 0;

            findMe = new FindMe(this, 1, false, false);
            findMe2 = new FindMe(this, 2, false, false);
            findMeFlying = new FindMe(this, 1, true, false);
            findMe2Flying = new FindMe(this, 2, true, false);
            findMeWet = new FindMe(this, 1, false, true);
            findMe2Wet = new FindMe(this, 2, false, true);
            findMeFlyingWet = new FindMe(this, 1, true, true);
            findMe2FlyingWet = new FindMe(this, 2, true, true);
        }

        public FindMe FindMe(Mob you)
        {
            if (!you.MobInfo.Aquaphobic && you.MobInfo.Flying && you.MobInfo.Scale < 1)
                return findMeFlying;
            if (!you.MobInfo.Aquaphobic && !you.MobInfo.Flying && you.MobInfo.Scale < 1)
                return findMe;
            if (!you.MobInfo.Aquaphobic && you.MobInfo.Flying && you.MobInfo.Scale >= 1)
                return findMe2Flying;
            if (!you.MobInfo.Aquaphobic && !you.MobInfo.Flying && you.MobInfo.Scale >= 1)
                return findMe2;
            if (you.MobInfo.Aquaphobic && you.MobInfo.Flying && you.MobInfo.Scale < 1)
                return findMeFlyingWet;
            if (you.MobInfo.Aquaphobic && !you.MobInfo.Flying && you.MobInfo.Scale < 1)
                return findMeWet;
            if (you.MobInfo.Aquaphobic && you.MobInfo.Flying && you.MobInfo.Scale >= 1)
                return findMe2FlyingWet;
            if (you.MobInfo.Aquaphobic && !you.MobInfo.Flying && you.MobInfo.Scale >= 1)
                return findMe2Wet;
            return null;
        }

        public float DepthSpan
        {
            get { return depthSpan; }
            set { depthSpan = value; }
        }

        public bool Loaded
        {
            get { return loaded; }
            set { loaded = value; }
        }
        public bool Active
        {
            get { return active; }
            set { active = value; }
        }

        public virtual void Load()
        {
            if (GobInfo.Preset != "")
            {
                renderInfo = RenderPresets.GetPreset(GobInfo.Preset);
                addRender(renderInfo.Render, renderInfo);
            }
        }

        public virtual bool touches(Gob other)
        {
            if (Shape == null || other.Shape == null) return false;
            return Shape.intersects(other.Shape);
        }

        public virtual bool CanTouch(Gob other)
        {
            return true;
        }

        public void Touch(Gob gob)
        {
            touchedGobs.Add(gob);
        }

        public void Untouch(Gob gob)
        {
            touchedGobs.Remove(gob);
        }

        public void clearTouch()
        {
            touchedGobs.Clear();
        }
        protected virtual void updateSeenMobs()
        {
            seenMobs.Clear();
            unseenMobs.Clear();

            Bounds viewBounds = new Bounds(GobInfo.Loc, GobInfo.ViewRange * 2, GobInfo.ViewRange * 2);

            CursedGame.CollectMobsInBounds(viewBounds, seenMobs);

            float vr2 = GobInfo.ViewRange * GobInfo.ViewRange;
            foreach (Mob mob in seenMobs)
            {
                float distance2 = (mob.GobInfo.Loc - GobInfo.Loc).LengthSquared();
                if (mob == this || distance2 > vr2)
                {
                    unseenMobs.Add(mob);
                }
            }
            foreach (Mob mob in unseenMobs)
            {
                seenMobs.Remove(mob);
            }
             
            timeToNextScan = Const.RescanTime * (float)CursedGame.Random.NextDouble();
        }
        public HashSet<Mob> SeenMobs
        {
            get { return seenMobs; }
        }

        public virtual void Unload()
        {
            foreach (Gob touchedGob in TouchedGobs)
            {
                touchedGob.Untouch(this);
            }
            renders.Clear();
        }

        public override void Draw(GameTime gameTime)
        {
            foreach (RenderPair pair in renders)
            {
                if (CursedGame.LastRender != pair.render)
                {
                    if (CursedGame.LastRender != null)
                        CursedGame.LastRender.End();
                    CursedGame.LastRender = pair.render;
                    CursedGame.LastRender.Begin(pair.info);
                }
                pair.render.Draw(gameTime, pair.info);
                //pair.render.End();
            }
            base.Draw(gameTime);
        }

        public virtual void PreUpdate()
        {

        }

        public override void Update(GameTime gameTime)
        {
            coolDown -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            timeToNextScan -= gameTime.ElapsedGameTime.Milliseconds * .001f;

            if (coolDown < 0)
                coolDown = 0;
            if (GobInfo.ViewRange > 0 && timeToNextScan <= 0)
                updateSeenMobs();

            base.Update(gameTime);
        }

        public virtual void PostUpdate()
        {
            GobInfo.Triggered = false;
        }

        public float CoolDown
        {
            get { return coolDown; }
            set { coolDown = value; }
        }

        public bool Ready
        {
            get { return coolDown <= 0; }
        }

        protected void addRender(Render render, RenderInfo info)
        {
            renders.Add(new RenderPair(render, info));
        }

        protected void clearRenders()
        {
            renders.Clear();
        }

        public ICollection<Gob> TouchedGobs
        {
            get { return touchedGobs; }
        }

        public CursedGame CursedGame
        {
            get { return (CursedGame)Game; }
        }

        public IShape Shape
        {
            get { return shape; }
        }

        public virtual bool Transcendent
        {
            get { return transcendant; }
            set {
                
                if (transcendant != value)
                {
                    B2DCollisionInfo cinfo = (B2DCollisionInfo)CollisionInfo;
                    if (cinfo != null)
                    {
                        foreach (Contact contact in cinfo.contacts)
                        {
                            contact.FlagForFiltering();
                        }
                    }
                }
                
                transcendant = value; 
            }
        }

        public bool Transient
        {
            get { return transient; }
            set { transient = value; }
        }

        public bool Inert
        {
            get { return inert; }
            set { inert = value; }
        }

        public bool IsEffect
        {
            get { return isEffect; }
            set { isEffect = value; }
        }

        public virtual Bounds Bounds
        {
            get {
                if (Shape == null)
                    return Bounds.Nothing;
                return Shape.Bounds; 
            }
        }

        public object CollisionInfo
        {
            get { return collisionInfo; }
            set { collisionInfo = value; }
        }

        public GobInfo GobInfo
        {
            get { return info; }
        }
        
    }
}
