﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob;
using Cursed.gob.tile;
using System.Windows;

namespace Cursed.gob
{
    public class SquaredCollider : ICollider
    {
        ICollection<Mob> mobs;
        ICollection<Tile> tiles;

        ICollection<Tile>[,] tileGrid;
        ICollection<Tile> empty;

        int minX;
        int maxX;
        int spanX;
        int minY;
        int maxY;
        int spanY;

        public SquaredCollider()
        {
            mobs = new List<Mob>();
            tiles = new List<Tile>();
            empty = new List<Tile>();
            repackTiles();
        }

        void repackTiles()
        {
            minX = int.MaxValue;
            maxX = -1 * int.MaxValue;
            minY = int.MaxValue;
            maxY = -1 * int.MaxValue;
            foreach (Tile tile in tiles)
            {
                if (tile.X < minX) minX = tile.X;
                if (tile.Y < minY) minY = tile.Y;
                if (tile.X > maxX) maxX = tile.X;
                if (tile.Y > maxY) maxY = tile.Y;
            }
            spanX = maxX - minX + 1;
            spanY = maxY - minY + 1;
            tileGrid = new ICollection<Tile>[spanX, spanY];
            foreach (Tile tile in tiles)
            {
                if (tileGrid[tile.X - minX, tile.Y - minY] == null)
                    tileGrid[tile.X - minX, tile.Y - minY] = new List<Tile>();
                tileGrid[tile.X - minX, tile.Y - minY].Add(tile);
            }
        }

        public void AddMob(Mob mob)
        {
            mobs.Add(mob);
        }

        public void RemoveMob(Mob mob)
        {
            mobs.Remove(mob);
        }

        public void AddTile(Tile tile)
        {
            tiles.Add(tile);
            if (tile.TileInfo.X > maxX || tile.TileInfo.Y > maxY ||
                tile.TileInfo.X < minX || tile.TileInfo.Y < minY)
            {
                repackTiles();
            }
            else
            {
                if (tileGrid[tile.X - minX, tile.Y - minY] == null)
                    tileGrid[tile.X - minX, tile.Y - minY] = new List<Tile>();
                tileGrid[tile.X - minX, tile.Y - minY].Add(tile);
            }
        }

        public void RemoveTile(Tile tile)
        {
            tiles.Remove(tile);
            tilesAt(tile.TileInfo.X, tile.TileInfo.Y).Remove(tile);
        }

        ICollection<Tile> tilesAt(int x, int y)
        {
            if (x < minX || x > maxX) return empty;
            if (y < minY || y > maxY) return empty;
            ICollection<Tile> tiles = tileGrid[x - minX, y - minY];
            if (tiles == null) return empty;
            return tiles;
        }

        public void handleCollisions()
        {

            foreach (Mob mob in mobs)
                mob.clearTouch();
            foreach (Tile tile in tiles)
                tile.clearTouch();

            foreach (Mob mob1 in mobs)
            {
                int minX = (int)(mob1.Bounds.X - .5f);
                int minY = (int)(mob1.Bounds.Y - .5f);
                int maxX = (int)(mob1.Bounds.X + mob1.Bounds.Width + .5f);
                int maxY = (int)(mob1.Bounds.Y + mob1.Bounds.Height + .5f);


                for (int x = minX; x <= maxX; x++)
                {
                    for (int y = minY; y <= maxY; y++)
                    {
                        ICollection<Tile> tilesHere = tilesAt(x, y);
                        foreach (Tile tile in tilesHere)
                        {
                            if (mob1.touches(tile))
                                mob1.touch(tile);
                            if (tile.touches(mob1))
                                tile.touch(mob1);
                        }
                    }
                }
            }

            foreach (Mob mob1 in mobs)
            {

                foreach (Mob mob2 in mobs)
                {
                    if (mob1 == mob2)
                    {
                        break;
                    }

                    if (mob1.Bounds.IntersectsWith(mob2.Bounds))
                    {
                        if (mob1.touches(mob2))
                        {
                            mob1.touch(mob2);
                        }

                        if (mob2.touches(mob1))
                        {
                            mob2.touch(mob1);
                        }
                    }
                }
            }
        }
    }
}
