﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob.kob;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Microsoft.Xna.Framework.Graphics;
using Cursed.sound;

namespace Cursed.gob.mob
{
    public class AttackInfo : MobInfo
    {
        float strength;
        float energy;
        float duration;

        public AttackInfo()
            : base()
        {
            strength = 0;
            energy = 0;
            duration = 0;
            Depth = Const.AttackDepth;
        }

        public float Strength
        {
            get { return strength; }
            set { strength = value; }
        }

        public float Energy
        {
            get { return energy; }
            set { energy = value; }
        }

        public float Duration
        {
            get { return duration; }
            set { duration = value; }
        }
    }

    public class ClawsAttack : Attack
    {
        static String[] sounds = { "Spirit\\SpiritAttack1", "Spirit\\SpiritAttack1", "Spirit\\SpiritAttack1" };
        public ClawsAttack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info, source, shape)
        {

        }

        public override void Load()
        {
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            ((SpriteRenderInfo)renderInfo).Texture = CursedGame.textureLib.GetTexture("hit");
            ((SpriteRenderInfo)renderInfo).Mask = Color.White;
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }
    }
    public class SpinAttack : Attack
    {
        float timePassed;
        public SpinAttack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info, source, shape)
        {
            timePassed = 0;
        }
        public override void Load()
        {
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            ((SpriteRenderInfo)renderInfo).Texture = CursedGame.textureLib.GetTexture("spin");
            ((SpriteRenderInfo)renderInfo).Mask = Color.White;
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }
        public override void Update(GameTime gameTime)
        {
            timePassed += gameTime.ElapsedGameTime.Milliseconds * .001f;
            this.AttackInfo.Theta += (float)Math.PI * 2 * (1 - gameTime.ElapsedGameTime.Milliseconds * .001f / AttackInfo.Duration);

            if (Source is Demon)
            {
                ((Demon)Source).SpinHands(timePassed / AttackInfo.Duration);
            }

            base.Update(gameTime);
        }
    }
    public class Attack : Mob
    {
        protected float timeLeft;
        ICollection<Kob> targets;
        Gob source;
        Vector2 offset;

        public Attack(CursedGame game, AttackInfo info, Gob source, IShape shape)
            : base(game, info)
        {
            this.source = source;
            this.shape = shape;
            renderInfo = null;
            targets = new HashSet<Kob>();
            timeLeft = AttackInfo.Duration;
            Transcendent = true;
            Transient = true;

            offset = GobInfo.Loc - source.GobInfo.Loc;
        }


        public override void Unload()
        {
            if (Source is Demon)
            {
                ((Demon)Source).RelaxHands();
            }
            base.Unload();
        }

        public Gob Source
        {
            get { return source; }
            set { source = value; }
        }

        public AttackInfo AttackInfo
        {
            get { return (AttackInfo)info; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            timeLeft -= gameTime.ElapsedGameTime.Milliseconds * .001f;

            GobInfo.Loc = source.GobInfo.Loc + offset;

            float alpha = timeLeft / AttackInfo.Duration;
            if (alpha < 0) alpha = 0;
            Color mask = ((SpriteRenderInfo)renderInfo).Mask;
            mask.A = (byte)(alpha * 255);
            ((SpriteRenderInfo)renderInfo).Mask = mask;

            foreach (Gob gob in TouchedGobs)
            {
                if (gob is Kob && gob != source && source.CanTouch(gob))
                {
                    targets.Add((Kob)gob);
                }
            }

            if (timeLeft <= 0)
            {
                foreach (Kob target in targets)
                {
                    if (source is Kob && target.Mean == ((Kob)source).Mean)
                        continue;
                    float damage = AttackInfo.Strength;
                    if (source is Mob && ((Mob)source).Lunging)
                        damage *= Const.LungeDamageFactor;
                    target.takeDamage(damage, source);
                }
                CursedGame.RemoveGob(this);
            }

            base.Update(gameTime);
        }
    }
}
