﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.input.ai;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.geom;
using Cursed.input;
using Cursed.view.renders;
using Cursed.sound;


namespace Cursed.gob.mob
{
    public class DialogMobInfo : MobInfo
    {
        public DialogMobInfo()
            :base()
        {
            Depth = Const.DialogDepth;
        }
        
        Line line;

        public override Gob Create(CursedGame game)
        {
            return new DialogMob(game, this);
        }

        public Line Line
        {
            get { return line; }
            set { line = value; }
        }

    }
    public class DialogMob: Mob
    {
        float ellapsedTime = 0;
        Mob target;
        string textLeft;

        public DialogMob(CursedGame game, DialogMobInfo info)
            : base(game, info)
        {
            this.IsEffect = true;
            this.Transcendent = true;
            this.Transient = true;
        }

        string nextText()
        {
            if (textLeft == null)
                return null;
            int stop = textLeft.IndexOf(';');
            string message = textLeft;
            if (stop == -1)
            {
                textLeft = null;
                return message;
            }
            message = textLeft.Substring(0, stop);
            textLeft = textLeft.Substring(stop + 1);
            return message;
        }

        public override void Load()
        {
            textLeft = DialogMobInfo.Line.Text;

            this.clearRenders();
            DialogRenderInfo info = new DialogRenderInfo();
            info.Texture = CursedGame.textureLib.GetTexture("square");
            info.Mask = new Color(.25f, .25f, .25f, .7f);
            info.Line = nextText();
            this.renderInfo = info;
            this.addRender(CursedGame.getRender(Render.RenderType.DialogRender),info);

            target = (Mob)CursedGame.GetFirstLabeledGob(DialogMobInfo.Label);
        }
        public override void Update(GameTime gameTime)
        {
            ((DialogRenderInfo)renderInfo).Pos = target.GobInfo.Loc + new Vector2(0, target.MobInfo.Scale / .9f);
            ellapsedTime += gameTime.ElapsedGameTime.Milliseconds*.001f;
            if (ellapsedTime > DialogMobInfo.Line.Length)
            {
                string nextLine = nextText();
                if (nextLine == null)
                    CursedGame.RemoveGob(this);
                else
                {
                    ((DialogRenderInfo)renderInfo).Line = nextLine;
                    ellapsedTime = 0;
                }
            }
            this.shape = new Rect(CursedGame.Camera.Position, CursedGame.Camera.ScreenSize, CursedGame.Camera.ScreenSize);
        }
        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);
        }

        public DialogMobInfo DialogMobInfo
        {
            get { return (DialogMobInfo)info; }
        }
        public override void Unload()
        {
            base.Unload();
        }
    }
}
