﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.view;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Cursed.geom;
using Cursed.input;

namespace Cursed.gob.mob
{
    public class FamiliarChooserInfo : MobInfo
    {
        int whichFamiliar;
        Vector2 offset;

        public FamiliarChooserInfo()
            : base()
        {
            whichFamiliar = 0;
            offset = new Vector2(0, 1.5f);
            Preset = "SummonFrame";
            Depth = Const.ChooserDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new FamiliarChooser(game, this);
        }

        public int WhichFamiliar
        {
            get { return whichFamiliar; }
            set { whichFamiliar = value; }
        }

        public Vector2 Offset
        {
            get { return offset; }
            set { offset = value; }
        }
    }

    public class FamiliarChooser : Mob
    {
        RenderInfo stickerInfo;
        MouseController mouseController;

        public FamiliarChooser(CursedGame game, FamiliarChooserInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Inert = false;
            Transient = true;
            IsEffect = false;
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, 0.5f);
            base.Load();
            stickerInfo = null;
            switch (FamiliarChooserInfo.WhichFamiliar)
            {
                case -2:
                    stickerInfo = RenderPresets.GetPreset("FamiliarTeleport");
                    break;
                case -1:
                    stickerInfo = RenderPresets.GetPreset("FamiliarDismiss");
                    break;
                case 0:
                    stickerInfo = RenderPresets.GetPreset("Familiar0");
                    break;
                case 1:
                    stickerInfo = RenderPresets.GetPreset("Familiar1");
                    break;
            }
            if (stickerInfo != null)
                renders.Add(new RenderPair(stickerInfo.Render, stickerInfo));

            mouseController = new MouseController(CursedGame.Camera, CursedGame.Window);
            mouseController.Push(this);
            CursedGame.AddController(mouseController);
        }

        public override void Unload()
        {
            CursedGame.RemoveController(mouseController);
            base.Unload();
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {

            if (CursedGame.DruidPlayer != null)
                GobInfo.Loc = CursedGame.DruidPlayer.GobInfo.Loc + FamiliarChooserInfo.Offset;
            updateRenderTransform(stickerInfo);

            base.Update(gameTime);
        }

        public override void control(ControlInfo cInfo)
        {
            if (cInfo is MouseCursorInfo)
            {
                MouseCursorInfo mci = (MouseCursorInfo)cInfo;
                if (mci.Clicked)
                {
                    if (Shape.intersects(new Circle(mci.Mouse, 0.1f)))
                    {
                        if (FamiliarChooserInfo.WhichFamiliar >= 0)
                        {
                            CursedGame.DruidPlayer.SummonFamiliar(FamiliarChooserInfo.WhichFamiliar);
                        }
                        else if (FamiliarChooserInfo.WhichFamiliar == -1)
                        {
                            CursedGame.DruidPlayer.DismissFamiliar();
                        }
                        else if (FamiliarChooserInfo.WhichFamiliar == -2)
                        {
                            CursedGame.DruidPlayer.TeleportFamiliar();
                        }
                    }
                }
            }
            base.control(cInfo);
        }

        public FamiliarChooserInfo FamiliarChooserInfo
        {
            get { return (FamiliarChooserInfo)info; }
        }
    }
}
