﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.mob.kob;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.gob.mob
{
    public class LifeBlobInfo : MobInfo
    {
        float amount;
        float maxAmount;
        float sizeRatio;
        float decay;
        public LifeBlobInfo()
        {
            amount = .5f;
            maxAmount = .5f;
            decay = Const.LifeDecay;
            sizeRatio = Const.LifeAspect;
            Depth = Const.LifeBlobDepth;
            Scale = SizeRatio * MaxAmount;
        }

        public override Gob Create(CursedGame game)
        {
            return new LifeBlob(game, this);
        }
        public float Amount
        {
            get { return amount; }
            set { amount = value; }
        }
        public float MaxAmount
        {
            get { return maxAmount; }
            set { maxAmount = value; }
        }
        public float SizeRatio
        {
            get{return sizeRatio;}
            set{ sizeRatio = value;}
        }
        public float Decay
        {
            get { return decay; }
            set { decay = value; }
        }
    }

    class LifeBlob : Mob
    {
        Druid source;
        

        public LifeBlob(CursedGame game, LifeBlobInfo info)
            : base(game, info)
        {
            Transient = true;
            Transcendent = true;
            source = null;
        }

        public Druid Source
        {
            get { return source; }
            set { source = value; }
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .5f);
            //renderInfo = new SpriteRenderInfo(shape.Bounds);
            //renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.Green;
            //renderInfo.Texture = CursedGame.textureLib.GetTexture("circle");
            //addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            
            LifeBlobInfo.Amount -= LifeBlobInfo.Decay * gameTime.ElapsedGameTime.Milliseconds / 1000f;


            int count = 0;
            foreach (Gob gob in TouchedGobs)
            {
                if (gob == Source)
                    continue;
                if (gob is Kob)
                {
                    count++;
                }
            }
            foreach (Gob gob in TouchedGobs)
            {
                if (gob == Source)
                    continue;
                if (gob is Kob)
                {
                    Kob kob = (Kob)gob;
                    if (Source != null && kob.Mean == Source.Mean)
                        continue;
                    float transfer = Const.LifeBlobDrainRate * LifeBlobInfo.Amount / count;
                    transfer *= gameTime.ElapsedGameTime.Milliseconds * .001f;
                    if (!(gob is Familiar))
                        kob.KobInfo.Energy += transfer;
                    LifeBlobInfo.Amount -= transfer;
                }
            }

            if (LifeBlobInfo.Amount <= 0)
            {
                CursedGame.RemoveGob(this);
            }

            float t = LifeBlobInfo.Amount / LifeBlobInfo.MaxAmount;
            if (t < 0) t = 0;
            if (t > 1) t = 1;
            //renderInfo.Mask = new Color(0, 255, 0, (byte)(t * 255));

            base.Update(gameTime);
        }
        public LifeBlobInfo LifeBlobInfo
        {
            get { return (LifeBlobInfo)info; }
        }

    }
}
