﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob.kob;

namespace Cursed.gob.mob
{
    public class TriggerZoneInfo : MobInfo
    {
        float width;
        float height;
        bool byPlayer;
        bool byDruid;
        bool byFamiliar;
        bool byMonster;

        public TriggerZoneInfo()
            : base()
        {
            width = 1;
            height = 1;
            Label = "door";
            Depth = Const.TriggerZoneDepth;
            byPlayer = true;
            byDruid = true;
            byFamiliar = false;
            byMonster = false;
        }

        public override Gob Create(CursedGame game)
        {
            return new TriggerZone(game, this);
        }

        public bool ByPlayer
        {
            get { return byPlayer; }
            set { byPlayer = value; }
        }

        public bool ByDruid
        {
            get { return byDruid; }
            set { byDruid = value; }
        }
        public bool ByFamiliar
        {
            get { return byFamiliar; }
            set { byFamiliar = value; }
        }
        public bool ByMonster
        {
            get { return byMonster; }
            set { byMonster = value; }
        }

        public float Width
        {
            get { return width; }
            set { width = value; }
        }

        public float Height
        {
            get { return height; }
            set { height = value; }
        }
    }

    public class TriggerZone : Mob
    {
        public TriggerZone(CursedGame game, TriggerZoneInfo info)
            : base(game, info)
        {
            Transcendent = true;
        }

        public override void Load()
        {
            shape = new Rect(Vector2.Zero, TriggerZoneInfo.Width, TriggerZoneInfo.Height);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Texture = CursedGame.textureLib.GetTexture("square");
            renderInfo.Mask = new Color(.8f, .8f, 0f,.4f);
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }

        protected virtual bool canTrigger(Gob gob)
        {
            if (TriggerZoneInfo.ByPlayer && gob == CursedGame.DruidPlayer)
                return true;
            if (TriggerZoneInfo.ByDruid && gob is Druid)
                return true;
            if (TriggerZoneInfo.ByFamiliar && gob is Familiar)
                return true;
            if (TriggerZoneInfo.ByMonster && gob is Kob && ((Kob)gob).Mean)
                return true;
            return false;
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            GobInfo.Triggered = false;
            foreach (Gob gob in TouchedGobs)
            {
                if (canTrigger(gob))
                {
                    
                    GobInfo.Triggered = true;
                    break;
                }
            }
            if (GobInfo.Triggered)
            {
                ICollection<Gob> labelGobs = CursedGame.GetLabeledGobs(GobInfo.Label);
                foreach (Gob gob in labelGobs)
                {
                    gob.GobInfo.Triggered = true;
                }
            }

            base.Update(gameTime);
        }

        public override void PostUpdate()
        {
            //not setting triggered to false
        }

        public TriggerZoneInfo TriggerZoneInfo
        {
            get { return (TriggerZoneInfo)info; }
        }
        public override void Draw(GameTime gameTime)
        {
            if(editor.Editor.active)
                base.Draw(gameTime);
        }
    }
}
