﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Cursed.input;
using Cursed.gob.mob.effects;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob.kob;

namespace Cursed.gob.mob
{
    public class WandInfo : MobInfo
    {
        public WandInfo()
            : base()
        {
            Depth = Const.WandDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Wand(game, this);
        }
        public override Vector2 Loc
        {
            get
            {
                return base.Loc;
            }
            set
            {
              
                base.Loc = value;
            }
        }
    }

    public class Wand : Mob
    {
        MouseType mouseType;
        Druid owner;
        bool strong;

        public Wand(CursedGame game, WandInfo info)
            : base(game, info)
        {
            Transient = true;
            Transcendent = true;
            //IsEffect = true;

            mouseType = MouseType.None;
            strong = false;
        }

        public Druid Owner
        {
            get { return owner; }
            set { owner = value; }
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .05f);    
        }

        public override void Update(GameTime gameTime)
        {
            if (gameTime.ElapsedGameTime.Milliseconds > 0)
            {
                SparkInfo sparkInfo = new SparkInfo();
                sparkInfo.Loc = GobInfo.Loc;
                sparkInfo.Vel = CursedGame.RandomVector2;
                sparkInfo.Omega = 25 * (float)(CursedGame.Random.NextDouble() - .5);
                sparkInfo.Color = new Color((float)CursedGame.Random.NextDouble(), 
                    1f, (float)CursedGame.Random.NextDouble());
                if (strong)
                {
                    sparkInfo.Duration = 1;
                    sparkInfo.Vel *= 0.25f;
                }
                sparkInfo.Scale *= (GobInfo.Transparency + 0.1f);

                CursedGame.AddGob(sparkInfo.Create(CursedGame));
            }
            base.Update(gameTime);
        }

        public WandInfo WandInfo
        {
            get { return (WandInfo)info; }
        }

        public MouseType MouseType
        {
            get { return mouseType; }
            set { mouseType = value; }
        }

        public bool Strong
        {
            get { return strong; }
            set { strong = value; }
        }
    }
}
