﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob.kob;

namespace Cursed.gob.mob
{
    public class WoodInfo : KobInfo
    {
        float duration;

        public WoodInfo()
            : base()
        {
            duration = 3f;
            Stuck = true;
            Depth = Const.WoodDepth;
            this.Theta = (float)(CursedGame.Random.NextDouble() * Math.PI * 2);
            CurrentHealth = Const.WoodHealth;
            MaxHealth = Const.WoodHealth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Wood(game, this);
        }

        public float Duration
        {
            get { return duration; }
            set { duration = value; }
        }
    }

    public class Wood : Kob
    {
        Druid source;

        float timeLeft;

        public Wood(CursedGame game, WoodInfo info)
            : base(game, info)
        {
            Transient = true;
            Transcendent = false;
            source = null;
           
            timeLeft = WoodInfo.Duration;
        }

        public Druid Source
        {
            get { return source; }
            set { source = value; }
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, Const.WoodSize);
            renderInfo = new SpriteRenderInfo(shape.Bounds);
            renderInfo.Texture = CursedGame.textureLib.GetTexture("wood");
            renderInfo.Mask = new Color(92, 51, 23);
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);
        }

        public override bool CanTouch(Gob other)
        {
            if (other is Wood || !(other is Mob))
                return false;
            return base.CanTouch(other);
        }

        public override void Update(GameTime gameTime)
        {

            timeLeft -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeLeft <= 0)
            {
                CursedGame.RemoveGob(this);
            }


            float t = timeLeft / WoodInfo.Duration;
            if (t < 0) t = 0;
            if (t > 1) t = 1;
            renderInfo.Mask = new Color(92, 51, 23, (byte)(t * 255));

            base.Update(gameTime);
        }

        public WoodInfo WoodInfo
        {
            get { return (WoodInfo)info; }
        }
    }
}
