﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.view.renders;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.view;

namespace Cursed.gob.mob.effects
{
    public class BloodStainInfo : MobInfo
    {
        Color color;
        float duration;

        public BloodStainInfo()
            : base()
        {
            color = Color.White;
            Depth = Const.BloodStainDepth;
            duration = 3;
            Texture = "BloodSheet";
            Preset = "BloodStain";
        }

        public override Gob Create(CursedGame game)
        {
            return new BloodStain(game, this);
        }

        public Color Color
        {
            get { return color; }
            set { color = value; }
        }

        public float Duration
        {
            get { return duration; }
            set { duration = value; }
        }
    }

    public class BloodStain : Mob
    {
        float timeLeft;

        public BloodStain(CursedGame game, BloodStainInfo info)
            : base(game, info)
        {
            IsEffect = true;
            Transcendent = true;
            Transient = true;
        }

        public override void Load()
        {
            base.Load();
            timeLeft = BloodStainInfo.Duration;
            shape = new Rect(Vector2.Zero, 1, 1);
        }

        public BloodStainInfo BloodStainInfo
        {
            get { return (BloodStainInfo)info; }
        }

        public override void Update(GameTime gameTime)
        {
            timeLeft -= gameTime.ElapsedGameTime.Milliseconds * .001f;

            if (timeLeft <= 0)
            {
                CursedGame.RemoveGob(this);
            }

            float t = timeLeft / BloodStainInfo.Duration;
            if (t < 0) t = 0;
            if (t > 1) t = 1;

            Color c = BloodStainInfo.Color;
            c.A = (byte)(t * 255);
            renderInfo.Mask = c;

            base.Update(gameTime);
        }
    }
}
