﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Cursed.gob.mob.kob;

namespace Cursed.gob.mob.effects
{
    public class FogInfo : MobInfo
    {
        public FogInfo()
            : base()
        {
            Texture = "fogPatch";
            Depth = Const.FogDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Fog(game, this);
        }
    }

    public class Fog : Mob
    {
        Vector2 offset;

        public Fog(CursedGame game, FogInfo info)
            : base(game, info)
        {
            IsEffect = true;
            Transcendent = true;
            Transient = true;
            offset = Vector2.Zero;
        }

        public override void Load()
        {
            shape = new Rect(Vector2.Zero, 2, 2);
            renderInfo = new PatchRenderInfo(Shape.Bounds, 2, 2);
            renderInfo.Texture = CursedGame.textureLib.GetTexture(GobInfo.Texture);
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender), renderInfo);

            base.Load();
        }

        protected override void updateRenderTransform(RenderInfo rInfo)
        {
            rInfo.Transform = Microsoft.Xna.Framework.Matrix.CreateScale(MobInfo.Scale);
            rInfo.Transform *= Microsoft.Xna.Framework.Matrix.CreateRotationZ(MobInfo.Theta);

            Vector3 pos = MobInfo.Loc3;
            pos.X += offset.X;
            pos.Y += offset.Y;

            rInfo.Transform *= Microsoft.Xna.Framework.Matrix.CreateTranslation(pos);
        }

        public override void Update(GameTime gameTime)
        {
            Druid druid = CursedGame.DruidPlayer;
            if (druid != null)
            {
                offset = GobInfo.Loc - druid.GobInfo.Loc;
                float ll = offset.LengthSquared();
                float factor = Const.FogFactor * (float)Math.Exp(-ll / Const.FogWidth);
                offset.Normalize();
                offset *= factor;
            }
            else
            {
                offset = Vector2.Zero;
            }

            base.Update(gameTime);
        }
    }
}
