﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.view.renders;
using Cursed.view;

namespace Cursed.gob.mob.effects
{
    public class SparkInfo : MobInfo
    {
        Color color;
        float duration;

        public SparkInfo()
            : base()
        {
            color = Color.White;
            duration = 0.2f;
            Scale = 0.4f;
            Depth = Const.SparkDepth;
            Preset = "Spark";
            Texture = "crystal";
        }

        public override Gob Create(CursedGame game)
        {
            return new Spark(game, this);
        }

        public Color Color
        {
            get { return color; }
            set { color = value; }
        }

        public float Duration
        {
            get { return duration; }
            set { duration = value; }
        }
    }

    public class Spark : Mob
    {
        float timeLeft;

        public Spark(CursedGame game, SparkInfo info)
            : base(game, info)
        {
            IsEffect = true;
            Transient = true;
            Transcendent = true;
        }

        public override void Load()
        {
            base.Load();
            timeLeft = SparkInfo.Duration;
            shape = new Rect(Vector2.Zero, 1, 1);
            Color c = SparkInfo.Color;
            c.A = (byte)(SparkInfo.Transparency * 255);
            renderInfo.Mask = c;
            
        }

        public override void Update(GameTime gameTime)
        {
            timeLeft -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeLeft < 0)
                CursedGame.RemoveGob(this);

            float t = SparkInfo.Transparency * timeLeft / SparkInfo.Duration;
            if (t < 0) t = 0;
            if (t > 1) t = 1;

            Color c = SparkInfo.Color;
            c.A = (byte)(t * 255);
            renderInfo.Mask = c;

            base.Update(gameTime);
        }

        public SparkInfo SparkInfo
        {
            get { return (SparkInfo)info; }
        }
    }
}
