﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;

namespace Cursed.gob.mob.kob
{
    public class CrystalInfo : MobInfo
    {
        bool mean;
        float givenEnergy;
        float width;

        public CrystalInfo()
            : base()
        {
            mean = false;
            width = 3;
            givenEnergy = 1;
            ViewRange = 3;
            Stuck = true;
            Preset = "LifeCrystal";
            Depth = Const.CrystalDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Crystal(game, this);
        }

        public bool Mean
        {
            get { return mean; }
            set { mean = value; }
        }
        public float GivenEnergy
        {
            get { return givenEnergy; }
            set { givenEnergy = value; }
        }
        public float Width
        {
            get { return width; }
            set { width = value; }
        }
    }
    public class Crystal : Mob
    {

        public Crystal(CursedGame game, CrystalInfo info)
            : base(game, info)
        {
            Transcendent = false;
            Transient = false;
            IsEffect = false;
            Inert = false;
        }
        public override void Load()
        {
            shape = new Circle(Vector2.Zero, 0.5f);
            base.Load();
        }

        public float EnergyAt(Vector2 loc)
        {
            float d2 = (GobInfo.Loc - loc).LengthSquared();
            return CrystalInfo.GivenEnergy * (float)Math.Exp(-d2 / CrystalInfo.Width);
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            foreach (Mob mob in SeenMobs)
            {
                if (mob is Kob)
                {
                    Kob kob = (Kob)mob;
                    kob.KobInfo.Energy = Math.Max(kob.KobInfo.Energy, CrystalInfo.GivenEnergy);
                }
            }

            base.Update(gameTime);
        }

        public CrystalInfo CrystalInfo
        {
            get { return (CrystalInfo)info; }
        }
    }
}
