﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.input.ai;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Cursed.geom;
using Cursed.input;
using Cursed.view.renders;
using Cursed.sound;
using Cursed.view;
using Cursed.input.ai.rl;

namespace Cursed.gob.mob.kob
{
    public class DemonInfo : KobInfo
    {
        int tailLength;
        bool spinAttack;
        float strength;
        float lungeRange;
        float attackSpeed;

        public DemonInfo()
            : base()
        {
            lungeRange = 4;
            ViewRange = 5;
            Thrust = 5;
            Drag = 2f;
            tailLength = 10;
            Transparency = 1f;
            Energy = -1f;
            Mass = Scale;
            Depth = Const.DemonDepth;
            spinAttack = false;
            strength = 1;
            Preset = "Demon";
            attackSpeed = 1f;
            Aquaphobic = false;
            Flying = true;
            
        }

        public override Gob Create(CursedGame game)
        {
            return new Demon(game, this);
        }

        public float AttackSpeed
        {
            get { return attackSpeed; }
            set { attackSpeed = value; }
        }

        public float StrengthFactor
        {
            get { return strength; }
            set { strength = value; }
        }
        public int TailLength
        {
            get { return tailLength; }
            set { tailLength = value; }

        }
        public bool SpinAttack
        {
            get { return spinAttack; }
            set { spinAttack = value; }
        }

        public float LungeRange
        {
            get { return lungeRange; }
            set { lungeRange = value; }
        }

    }

    public class Demon : Kob
    {  
        protected TrailRenderInfo trailInfo;

        protected Sound basicAttackSound;
        protected Sound spinAttackSound;

        DruidSeeker seekerControl;
        DruidAttacker attackerControl;
        WoodChopper chopperControl;

        public TrailRenderInfo TrailInfo
        {
            get { return trailInfo; }
            set { trailInfo = value; }
        }
        public Demon(CursedGame game, DemonInfo info)
            : base(game, info)
        {
            basicAttackSound = null;
            spinAttackSound = null;
            Transient = false;
            Transcendent = false;

            seekerControl = null;
            attackerControl = null;
            chopperControl = null;

            HasHands = true;
            HandPreset = "DemonArm";
        }
        public override void Load()
        {
            trailInfo = new TrailRenderInfo();
            addRender(CursedGame.getRender(Cursed.view.renders.Render.RenderType.TrailRender), trailInfo);
            base.Load();
            shape = new Circle(Vector2.Zero, .5f);
            DemonInfo.Aquaphobic = false;
            KobInfo.EnergyDecay *= 3;


            hitSound = SoundIDs.DemonHit(CursedGame);
            basicAttackSound = SoundIDs.FamiliarBasicAttack(CursedGame);
            deathSound = SoundIDs.DemonDeath(CursedGame);
            addLifeBar();
            addControllers();
        }

        protected virtual void addControllers()
        {
            
            seekerControl = new DruidSeeker(this);
            CursedGame.AddController(seekerControl);
            seekerControl.Push(this);
            

            //MCTSDaemon sd = MCTSDaemon.Self;
            //sd.AddDemon(this);

            attackerControl = new DruidAttacker(this);
            CursedGame.AddController(attackerControl);
            attackerControl.Push(this);

            chopperControl = new WoodChopper();
            CursedGame.AddController(chopperControl);
            chopperControl.Push(this);
            
        }

        protected virtual void performBasicAttack(Vector2 direction)
        {
            if (Ready)
            {
                basicAttackSound.stop();
                basicAttackSound.play();
                Vector2 loc = GobInfo.Loc + direction * (.75f);

                float theta = (float)Math.Acos(direction.X);
                if (direction.Y < 0) theta *= -1;

                ClapHands(theta);

                MobInfo.Theta = theta;
                theta -= MathHelper.PiOver2;

                AttackInfo ati = new AttackInfo();
                ati.Loc = loc;
                ati.Vel = MobInfo.Vel;
                ati.Theta = theta;
                ati.Energy = KobInfo.Energy;
                ati.Strength = Const.DemonClawStrength*DemonInfo.StrengthFactor;
                ati.Duration = Const.DemonClawAttackDuration;
                Circle c = new Circle(Vector2.Zero, .5f);
                Attack at = new ClawsAttack(CursedGame, ati, this, c);
                CursedGame.AddGob(at);
                //((AnimatedSpriteRenderInfo)renderInfo).ChangeState("attack");
                CoolDown += Const.DemonClawAttackCoolDown / DemonInfo.AttackSpeed;
            }
        }
        protected void performSpinAttack()
        {
            float theta = (float)Math.Acos(direction.X);
                if (direction.Y < 0) theta *= -1;
                theta -= MathHelper.PiOver2;
            if (Ready)
            {
                spinAttackSound.stop();
                spinAttackSound.play();
                AttackInfo ati = new AttackInfo();
                ati.Loc = GobInfo.Loc;
                ati.Vel = MobInfo.Vel;
                ati.Theta = theta;
                ati.Energy = KobInfo.Energy;
                ati.Strength = Const.DemonSpinStrength * DemonInfo.StrengthFactor;
                ati.Duration = Const.DemonSpinAttackDuration;
                Circle c = new Circle(Vector2.Zero, this.KobInfo.Scale * .75f);
                Attack at = new SpinAttack(CursedGame, ati, this, c);
                CursedGame.AddGob(at);
                CoolDown += Const.DemonSpinAttackCoolDown / DemonInfo.AttackSpeed ;

            }
        }
        public override void control(Cursed.input.ControlInfo cInfo)
        {
 	        base.control(cInfo);
            if (cInfo is BasicAttackInfo)
            {
                if(((BasicAttackInfo)cInfo).Type == BasicAttackInfo.ATTACKTYPE.NORMAL)
                    performBasicAttack(((BasicAttackInfo)cInfo).Direction);
                else if (((BasicAttackInfo)cInfo).Type == BasicAttackInfo.ATTACKTYPE.SPIN)
                {
                    performSpinAttack();
                }
            }
        }
        public override void Update(GameTime gameTime)
        {
            Vector2 oldPosition = GobInfo.Loc;

            HashSet<Mob> moreSeenMobs = new HashSet<Mob>();
            foreach (Mob mob in SeenMobs)
            {
                if (mob is Familiar)
                {
                    Familiar familiar = (Familiar)mob;
                    if (familiar.Master != null)
                        moreSeenMobs.Add(familiar.Master);
                }
            }
            SeenMobs.UnionWith(moreSeenMobs);

            if (Lunging)
            {
                foreach (Gob gob in TouchedGobs)
                {
                    if (gob is Kob && ((Kob)gob).Mean != Mean)
                    {
                        Vector2 diff = gob.GobInfo.Loc - GobInfo.Loc;
                        diff.Normalize();
                        performBasicAttack(diff);
                    }
                }
            }

            //updateColor();

            if (renderInfo is AnimatedSpriteRenderInfo)
                ((AnimatedSpriteRenderInfo)renderInfo).Update(GobInfo.Loc, direction, gameTime);
            if (trailInfo != null && renderInfo != null) 
            trailInfo.Update((SpriteRenderInfo)renderInfo);


            base.Update(gameTime);
        }
        public virtual void updateColor()
        {
            float val = (float)Math.Min(1, DemonInfo.Energy);
            renderInfo.Mask = new Color((1 - val) * .3f, (1 - val) * .3f + val*.6f, (1 - val) * .3f, DemonInfo.Transparency); 
        }
        public DemonInfo DemonInfo
        {
            get { return (DemonInfo)info; }
        }

        public override void Unload()
        {
            CursedGame.RemoveController(seekerControl);
            CursedGame.RemoveController(attackerControl);
            CursedGame.RemoveController(chopperControl);
            //MCTSDaemon.Self.RemoveDemon(this);
            base.Unload();
        }
    }
}
