﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using Cursed.input;
using System.Runtime.Serialization;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.tile;
using Cursed.sound;
using Cursed.gob.mob.effects;
using Cursed.input.util;
using Cursed.input.ai;

namespace Cursed.gob.mob.kob
{
    public class DruidInfo : KobInfo
    {
        float healthRegen;
        float currentMana;
        float maxMana;
        float manaRegen;
        bool isPlayer;

        bool hasFamiliar0;
        bool hasFamiliar1;
        int whichFamiliar;

        public DruidInfo()
            : base()
        {
            Thrust = 20;
            Drag = 5f;
            healthRegen = 0.05f;
            currentMana = Const.MaxMana;
            maxMana = Const.MaxMana;
            manaRegen = Const.ManaRegen;
            isPlayer = false;
            Depth = Const.DruidDepth;
            ViewRange = 10;
            Preset = "Druid";
            Energy = 0;
            EnergyDecay = 0.3f;
            CurrentHealth = Const.DruidHealth;
            MaxHealth = Const.DruidHealth;
            hasFamiliar0 = true;
            hasFamiliar1 = false;
            whichFamiliar = 0;
        }

        public override Gob Create(CursedGame game)
        {
            return new Druid(game, this);
        }

        public float HealthRegen
        {
            get { return healthRegen; }
            set { healthRegen = value; }
        }
        public float CurrentMana
        {
            get { return currentMana; }
            set { currentMana = value; }
        }
        public float MaxMana
        {
            get { return maxMana; }
            set { maxMana = value; }
        }
        public float ManaRegen
        {
            get { return manaRegen; }
            set { manaRegen = value; }
        }
        public bool IsPlayer
        {
            get { return isPlayer; }
            set { isPlayer = value; }
        }
        public bool HasFamiliar0
        {
            get { return hasFamiliar0; }
            set { hasFamiliar0 = value; }
        }
        public bool HasFamiliar1
        {
            get { return hasFamiliar1; }
            set { hasFamiliar1 = value; }
        }
        public int WhichFamiliar
        {
            get { return whichFamiliar; }
            set { whichFamiliar = value; }
        }
    }
    
    public class Druid : Kob
    {
        Wand wand;
        float timeToNextPoop;
        Vector2 lastSpell;
        bool casting = false;
        Familiar familiar;

        KeyDirectionController keyController;
        SpaceControl spaceControl;
        MouseController mouseController;
        MouseTrailController mouseTrailController;
        MouseGestureController gestureController;
        EvilDruidControl evilDruidControl;
        GamePadController padController;

        List<WGesture.Direction> familiarGesture;

        List<FamiliarChooser> choosers;

        public Druid(CursedGame game, DruidInfo info)
            : base(game, info)
        {
            familiar = null;
            mean = false;
            Transcendent = false;
            choosers = null;

            timeToNextPoop = 0;

            HandPreset = "DemonArm";
            HasHands = true;
            ArmSpacing = 0.5f;
        }
        public override void Load()
        {

            mean = !DruidInfo.IsPlayer;
            if (mean)
                DruidInfo.Preset = "EvilDruid";
            base.Load();
            shape = new Circle(Vector2.Zero, .5f);

            WandInfo wandInfo = new WandInfo();
            wand = new Wand(CursedGame, wandInfo);
            wand.WandInfo.Loc = GobInfo.Loc;
            wand.Owner = this;
            CursedGame.AddGob(wand);

            hitSound = SoundIDs.DruidHit(CursedGame);

            if (DruidInfo.IsPlayer)
            {
                keyController = new KeyDirectionController();
                CursedGame.AddController(keyController);
                keyController.Push(this);

                mouseController = new MouseController(CursedGame.Camera, CursedGame.Window);
                CursedGame.AddController(mouseController);
                mouseController.Push(this);

                mouseTrailController = new MouseTrailController(CursedGame.Camera, CursedGame.Window);
                CursedGame.AddController(mouseTrailController);
                mouseTrailController.Push(this);

                spaceControl = new SpaceControl();
                CursedGame.AddController(spaceControl);
                spaceControl.Push(this);

                //gestureController = new MouseGestureController(CursedGame.Camera, CursedGame.Window);
                //CursedGame.AddController(gestureController);
                //gestureController.Push(this);

                evilDruidControl = null;

                padController = new GamePadController(CursedGame.Camera,CursedGame.Window);
                CursedGame.AddController(padController);
                padController.Push(this);
            }
            else
            {
                evilDruidControl = new EvilDruidControl(this);
                CursedGame.AddController(evilDruidControl);

                keyController = null;
                mouseController = null;
                mouseTrailController = null;
                gestureController = null;
            }

            familiarGesture = new List<WGesture.Direction>();
            familiarGesture.Add(WGesture.Direction.Right);

            familiar = null;
            DruidInfo.WhichFamiliar = 0;
            if (DruidInfo.WhichFamiliar != -1)
            {
                //SummonFamiliar(DruidInfo.WhichFamiliar);
            }
            mean = !DruidInfo.IsPlayer;

            addLifeBar();
        }
        public bool Casting
        {
            get { return casting; }
        }
        public DruidInfo DruidInfo
        {
            get { return (DruidInfo)info; }
        }
        public Familiar Familiar
        {
            get { return familiar; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (familiar == null && gameTime.ElapsedGameTime.Milliseconds != 0)
            {
                SummonFamiliar(0);
            }

            if (DruidInfo.CurrentHealth < DruidInfo.MaxHealth)
            {
                DruidInfo.CurrentHealth = Math.Min(DruidInfo.MaxHealth,
                    DruidInfo.CurrentHealth + DruidInfo.HealthRegen * gameTime.ElapsedGameTime.Milliseconds/1000f);
                bar.hit();
            }
            if (DruidInfo.CurrentMana < DruidInfo.MaxMana && !casting)
            {
                DruidInfo.CurrentMana = Math.Min(DruidInfo.MaxMana,
                    DruidInfo.CurrentMana + DruidInfo.ManaRegen * gameTime.ElapsedGameTime.Milliseconds / 1000f);
            } 
            wand.GobInfo.Transparency = DruidInfo.CurrentMana / DruidInfo.MaxMana;
            

            timeToNextPoop -= gameTime.ElapsedGameTime.Milliseconds * .001f;
            if (timeToNextPoop < 0)
            {
                LifeBlobInfo info = new LifeBlobInfo();
                Vector2 loc = GobInfo.Loc;
                loc.X += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                loc.Y += ((float)CursedGame.Random.NextDouble() - .5f) * .5f;
                info.Loc = loc;
                info.Decay = Const.DruidPoopDecay;
                info.Amount = Const.DruidPoopAmount;
                LifeBlob blob = new LifeBlob(CursedGame, info);
                blob.Source = this;
                CursedGame.AddGob(blob);
                timeToNextPoop = Const.DruidRepoopTime;
            }

            if (casting)
            {
                Vector2 dir = wand.GobInfo.Loc - GobInfo.Loc;
                if (dir != Vector2.Zero)
                {
                    float theta = (float)Math.Atan2(dir.Y, dir.X);
                    MobInfo.Theta = theta;
                    WaveHands(theta);
                }
            }
            else if (MobInfo.Vel.LengthSquared() > 0)
            {
                WaddleHands();
            }
            else
            {
                RelaxHands();
            }

            DruidInfo.CurrentMana = Math.Max(0, DruidInfo.CurrentMana);

            base.Update(gameTime);
        }

        protected void castLife(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.LifeSpellMana)
            {
                SoundIDs.LifeCast(CursedGame).play();
                if (!casting)
                {
                    casting = true;
                    lastSpell = loc;
                }
                for (int i = 0; i < Const.LifePerCast; i++)
                {
                    LifeBlobInfo info = new LifeBlobInfo();
                    info.Loc = lastSpell + (loc-lastSpell)*(i+1)/Const.LifePerCast+
                        CursedGame.RandomVector2 * CursedGame.RandomFloat * Const.LifeRange; ;
                    info.Amount = Const.LifeSpellAmount;
                    LifeBlob blob = new LifeBlob(CursedGame, info);
                    blob.Source = this;
                    CursedGame.AddGob(blob);
                }
                lastSpell = loc;
                CoolDown += Const.LifeSpellCoolDown;
                DruidInfo.CurrentMana -= Const.LifeSpellMana;
            }
        }

        protected void castWood(Vector2 loc)
        {
            if (Ready && DruidInfo.CurrentMana >= Const.WoodSpellMana)
            {
                bool blocked = false;
                foreach (Gob gob in wand.TouchedGobs)
                {
                    if (!gob.Bounds.Intersects(wand.Bounds))
                        continue;
                    if (gob is Kob && !(gob is Wood) && !gob.Transcendent)
                    {
                        //System.Console.WriteLine(gob.GetType().Name);
                        blocked = true;
                        break;
                    }
                }
                if (!casting)
                {
                    casting = true;
                    lastSpell = loc;
                }
                if (!blocked)
                {
                    SoundIDs.WoodCast(CursedGame).play();
                    for (int i = 0; i < Const.WoodPerCast; i++)
                    {
                        WoodInfo info = new WoodInfo();
                        info.Loc = lastSpell + (loc - lastSpell) * (i + 1) / Const.WoodPerCast +
                            CursedGame.RandomVector2 * CursedGame.RandomFloat * Const.WoodRange;
                        //info.Theta = (float)CursedGame.Random.NextDouble() * MathHelper.Pi * 2;
                        Wood wood = new Wood(CursedGame, info);
                        wood.Source = this;
                        CursedGame.AddGob(wood);

                    }
                }
                lastSpell = loc;
                CoolDown += Const.WoodSpellCoolDown;
                DruidInfo.CurrentMana -= Const.WoodSpellMana;
            }
        }

        public void SummonFamiliar(int type)
        {
            if (familiar != null)
                return;

            //if (DruidInfo.CurrentMana < Const.SummonCost)
            //    return;
            DruidInfo.CurrentMana -= Const.SummonCost;

            FamiliarInfo familiarInfo;
            switch (type)
            {
                case 0:
                    familiarInfo = new FamiliarInfo();
                    familiarInfo.Loc = GobInfo.Loc+CursedGame.RandomVector2*0.25f;
                    if (mean) familiarInfo.Preset = "EvilFamiliar";
                    familiar = (Familiar)familiarInfo.Create(CursedGame);
                    familiar.Master = this;
                    CursedGame.AddGob(familiar);
                    DruidInfo.WhichFamiliar = type;
                    break;
                case 1:
                    familiarInfo = new FamiliarInfo();
                    familiarInfo.Drag *= 3f;
                    familiarInfo.StrengthFactor *= 4f;
                    familiarInfo.AttackSpeed *= .4f;
                    familiarInfo.LungeRange /= 2f;
                    familiarInfo.Scale *= 1.5f;
                    familiarInfo.Loc = GobInfo.Loc + CursedGame.RandomVector2 * .25f;
                    familiar = (Familiar)familiarInfo.Create(CursedGame);
                    familiar.Master = this;
                    CursedGame.AddGob(familiar);
                    DruidInfo.WhichFamiliar = type;
                    break;
            }
            if (choosers != null)
                foreach (FamiliarChooser chooser in choosers)
                {
                    CursedGame.RemoveGob(chooser);
                }
            choosers = null;
        }
        public void DismissFamiliar()
        {
            DruidInfo.WhichFamiliar = -1;
            CursedGame.RemoveGob(familiar);
            familiar = null;
            if (choosers != null)
                foreach (FamiliarChooser chooser in choosers)
                {
                    CursedGame.RemoveGob(chooser);
                }
            choosers = null;
        }

        public void TeleportFamiliar()
        {
            if (familiar != null)
            {
                if (DruidInfo.CurrentMana < Const.TeleportCost)
                    return;
                DruidInfo.CurrentMana -= Const.TeleportCost;
                //familiar.GobInfo.Loc = GobInfo.Loc + CursedGame.RandomVector2 * 0.5f;
                int which = DruidInfo.WhichFamiliar;
                DismissFamiliar();
                SummonFamiliar(which);
            }
        }

        public override void control(ControlInfo cInfo)
        {
            base.control(cInfo);
            if (cInfo is SpaceControlInfo)
            {
                TeleportFamiliar();
            }
            if (cInfo is MouseTrailInfo && !CursedGame.Paused)
            {
                MouseTrailInfo mInfo = (MouseTrailInfo)cInfo;
                wand.MouseType = mInfo.Type;
                //wand.Strong = mInfo.Modifier;
                if (mInfo.Trail.Count > 0)
                {
                    Vector2 last = mInfo.Trail[mInfo.Trail.Count - 1];
                    if (mInfo.Type == MouseType.Offensive && mInfo.Modifier == false)
                        castLife(last);
                    if (mInfo.Type == MouseType.Defensive && mInfo.Modifier == false)
                        castWood(last);
                }
                if (mInfo.Finished)
                    casting = false;
            }
            if (cInfo is GestureInfo)
            {
                GestureInfo gInfo = (GestureInfo)cInfo;
                if (gInfo.Gesture.Matches(familiarGesture))
                {
                    if (gInfo.Type == MouseType.Offensive)
                        SummonFamiliar(0);
                    if (gInfo.Type == MouseType.Defensive)
                        DismissFamiliar();
                }
            }
            if (cInfo is MouseCursorInfo && !CursedGame.Paused)
            {
                MouseCursorInfo mInfo = (MouseCursorInfo)cInfo;
                wand.GobInfo.Loc = mInfo.Mouse;
                wand.MouseType = mInfo.Type;
                //wand.Strong = mInfo.Modifier;

                if (mInfo.Clicked && mInfo.Type == MouseType.Offensive && mInfo.Modifier == false)
                {
                    castLife(mInfo.Mouse);
                }

                if (mInfo.Clicked && mInfo.Type == MouseType.Defensive && mInfo.Modifier == false)
                {
                    castWood(mInfo.Mouse);
                }

                if (false && mInfo.Modifier == true && choosers == null)
                {
                    choosers = new List<FamiliarChooser>();
                    if (DruidInfo.WhichFamiliar == -1)
                    {
                        if (DruidInfo.HasFamiliar0)
                        {
                            FamiliarChooserInfo chooserInfo = new FamiliarChooserInfo();
                            FamiliarChooser chooser = (FamiliarChooser)chooserInfo.Create(CursedGame);
                            choosers.Add(chooser);
                        }

                        if (DruidInfo.HasFamiliar1)
                        {
                            FamiliarChooserInfo chooserInfo = new FamiliarChooserInfo();
                            chooserInfo.WhichFamiliar = 1;
                            chooserInfo.Offset = new Vector2(0, -1.5f);
                            FamiliarChooser chooser = (FamiliarChooser)chooserInfo.Create(CursedGame);
                            choosers.Add(chooser);
                        }
                    }
                    else
                    {

                        FamiliarChooserInfo chooserInfo = new FamiliarChooserInfo();
                        chooserInfo.WhichFamiliar = -1;
                        FamiliarChooser chooser = (FamiliarChooser)chooserInfo.Create(CursedGame);
                        choosers.Add(chooser);

                        chooserInfo = new FamiliarChooserInfo();
                        chooserInfo.WhichFamiliar = -2;
                        chooser = (FamiliarChooser)chooserInfo.Create(CursedGame);
                        choosers.Add(chooser);
                    }
                    for (int i = 0; i < choosers.Count; i++)
                    {
                        float t = i * (float)(2 * MathHelper.Pi / choosers.Count);
                        Vector2 offset = Vector2.Transform(new Vector2(0, 1.5f), Matrix.CreateRotationZ(t));
                        choosers[i].FamiliarChooserInfo.Offset = offset;
                        CursedGame.AddGob(choosers[i]);
                    }
                }

                if (mInfo.Modifier == false && choosers != null)
                {
                    foreach (FamiliarChooser chooser in choosers)
                    {
                        CursedGame.RemoveGob(chooser);
                    }
                    choosers = null;
                }
            }
            if (cInfo is GamePadInfo)
            {
                GamePadInfo gInfo = (GamePadInfo)cInfo;

                if (gInfo.A && familiar != null)
                    TeleportFamiliar();
            }
        }
        public override void takeDamage(float damage, Gob source)
        {
            #region temporary
            if(DruidInfo.IsPlayer)
                CursedGame.FogColor = new Microsoft.Xna.Framework.Graphics.Color((byte)255, CursedGame.FogColor.G, CursedGame.FogColor.B);
            #endregion

            BloodStainInfo bloodStainInfo = new BloodStainInfo();
            bloodStainInfo.Loc = GobInfo.Loc;
            CursedGame.AddGob(bloodStainInfo.Create(CursedGame));

            base.takeDamage(damage, source);
        }

        public override void die()
        {
            bar.hit();
            if (mean)
                LifeMap.allGreen = true;
            if (DruidInfo.IsPlayer)
            {
                DruidInfo.CurrentHealth = DruidInfo.MaxHealth;
                CursedGame.DruidPlayer = (Druid)DruidInfo.Create(CursedGame);
                CursedGame.ReloadLevel();
            }
            else
            {
                base.die();
            }
        }

        public override void Unload()
        {
            CursedGame.RemoveController(keyController);
            CursedGame.RemoveController(mouseController);
            CursedGame.RemoveController(mouseTrailController);
            CursedGame.RemoveController(gestureController);
            CursedGame.RemoveController(evilDruidControl);
            CursedGame.RemoveController(spaceControl);
            CursedGame.RemoveGob(wand);

            DismissFamiliar();
           

            base.Unload();
        }
        public Wand Wand
        {
            get { return wand; }
        }
    }
}
