﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.input.ai;
using Cursed.geom;
using Cursed.view.renders;
using Microsoft.Xna.Framework;
using Cursed.sound;
using Cursed.gob.tile;
namespace Cursed.gob.mob.kob
{
    public class FamiliarInfo : DemonInfo
    {
        public FamiliarInfo()
            : base()
        {
            Thrust = 1;
            Drag = .25f;
            Depth = Const.FamiliarDepth;
            Mass = .1f;
            LungeDuration = 1;
            ViewRange = 10;
            Label = "familiar";
            Preset = "Familiar0";
            Flying = true;
        }

        public override Gob Create(CursedGame game)
        {
            return new Familiar(game, this);
        }
    }

    public class Familiar: Demon
    {
        protected Druid master;
        protected GreedyFamiliarControl familiarControl;
        protected WoodChopper woodChopper;

        public Familiar(CursedGame game, FamiliarInfo info)
            : base(game, info)
        {
            Transient = true;
            info.SpinAttack = true;
        }

        public override bool Mean
        {
            get
            {
                if (Master != null)
                    return Master.Mean;
                return mean;
            }
        }

        public override void Load()
        {
            base.Load();
            //renderInfo.Mask = Microsoft.Xna.Framework.Graphics.Color.White;
            spinAttackSound = SoundIDs.FamiliarSpinAttack(CursedGame);

            SoundIDs.FamiliarSummon(CursedGame).play();
        }
        protected override void addControllers()
        {
            familiarControl = new GreedyFamiliarControl(this);
            CursedGame.AddController(familiarControl);

            woodChopper = new WoodChopper();
            woodChopper.Push(this);
            CursedGame.AddController(woodChopper);
        }

        public override void Unload()
        {
            SoundIDs.FamiliarDismiss(CursedGame).play();
            CursedGame.RemoveController(familiarControl);
            CursedGame.RemoveController(woodChopper);
            base.Unload();
        }

        public override void takeDamage(float damage, Gob source)
        {
            //familiar doesn't get damaged
        }

        public override bool CanTouch(Gob other)
        {
            if (other == Master || (other is Wood && ((Wood)other).Source == Master))
                return false;
            if (Master == CursedGame.DruidPlayer && other is Warren)
                return false;
            return base.CanTouch(other);
        }

        public Druid Master
        {
            get { return master; }
            set { master = value; }
        }

        public FamiliarInfo FamiliarInfo
        {
            get { return (FamiliarInfo)info; }
        }
        public override void updateColor()
        {
            
            return;
        }
    }
}
