﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Cursed.view;
using System.Windows.Media;
using System.Windows;
using Cursed.input;
using System.Runtime.Serialization;
using Cursed.sound;


namespace Cursed.gob.mob.kob
{
    //====================
    //Killable Game Object
    //====================
    public class KobInfo : MobInfo
    {
        float currentHealth;
        float maxHealth;
        float energy;
        float energyDecay;

        public KobInfo()
            : base()
        {
            currentHealth = 1;
            maxHealth = 1;
            energy = 0;
            energyDecay = .1f;
        }

        public float CurrentHealth
        {
            get { return currentHealth; }
            set { currentHealth = value; }
        }
        public float MaxHealth
        {
            get { return maxHealth; }
            set { maxHealth = value; }
        }
        public float Energy
        {
            get { return energy; }
            set { energy = value; }
        }
        public float EnergyDecay
        {
            get { return energyDecay; }
            set { energyDecay = value; }
        }
    }
    public class Kob : Mob
    {
        protected Sound hitSound;
        protected Sound deathSound;
        protected LifeBar bar;

        protected bool mean;

        Gob lastAttacker;

        float timeToDie;

        public Kob(CursedGame game, KobInfo info)
            : base(game, info)
        {
            hitSound = null;
            bar = null;
            mean = true;
            DepthSpan = 1;
            timeToDie = Single.PositiveInfinity;
        }
        public Gob LastAttacker
        {
            get
            {
                if (lastAttacker != null && lastAttacker.Loaded)
                    return lastAttacker;
                return null;
            }
        }
        public virtual bool Mean
        {
            get { return mean; }
        }
        public KobInfo KobInfo
        {
            get { return (KobInfo)info; }
        }

        public override void Update(GameTime gameTime)
        {
            float t = Math.Min(1, timeToDie / Const.DieFadeOutTime);
            if (t < 1)
            {
                foreach (RenderPair pair in renders)
                {
                    Microsoft.Xna.Framework.Graphics.Color c = pair.info.Mask;
                    c.A = (byte)(c.A * t);
                    pair.info.Mask = c;
                }
            }
            if (timeToDie <= 0)
                CursedGame.RemoveGob(this);
            timeToDie -= gameTime.ElapsedGameTime.Milliseconds * .001f;

            KobInfo.Energy -= gameTime.ElapsedGameTime.Milliseconds * .001f * KobInfo.EnergyDecay;
            if (KobInfo.Energy < 0)
            {
                KobInfo.Energy = 0f;
            }

            KobInfo.CurrentHealth = Math.Max(0, KobInfo.CurrentHealth);

            if (bar != null)
                bar.Percent = KobInfo.CurrentHealth / KobInfo.MaxHealth;
            base.Update(gameTime);
        }
        public void addLifeBar()
        {
            bar = new LifeBar(CursedGame, new LifeBarInfo());
            bar.Source = this;
            CursedGame.AddGob(bar);
        }
        public virtual void takeDamage(float damage, Gob source)
        {
            lastAttacker = source;
            KobInfo.CurrentHealth -= damage;
            if (KobInfo.CurrentHealth <= 0)
            {
                die();
            }
            else
            {
                if (hitSound != null)
                {
                    hitSound.stop();
                    hitSound.play();
                }
            }
            if(bar!= null)
                bar.hit();
        }
        public virtual void die()
        {
            if (deathSound != null)
            {
                deathSound.play();
            }

            foreach (Gob gob in CursedGame.GetLabeledGobs(GobInfo.Label))
            {
                gob.GobInfo.Triggered = true;
            }

            if (timeToDie > Const.DieFadeOutTime)
                timeToDie = Const.DieFadeOutTime;
        }
        public override void Unload()
        {
            if(bar != null)
                CursedGame.RemoveGob(bar);
            base.Unload();
        }
       
    }
}
