﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Cursed.input;
using Cursed.input.ai;

namespace Cursed.gob.mob.kob
{
    public class WarrenInfo : KobInfo
    {
        //0 = follow the druid
        int which;

        public WarrenInfo()
            : base()
        {
            Thrust = 15;
            Drag = 3f;
            which = 0;
            ViewRange = 10;
            Label = "Warren";
            Preset = "Warren";
            Depth = Const.DruidDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Warren(game, this);
        }

        public int Which
        {
            get { return which; }
            set { which = value; }
        }
    }

    public class Warren : Kob
    {
        Controller controller;

        public Warren(CursedGame game, WarrenInfo info)
            : base(game, info)
        {
            Inert = false;
            IsEffect = false;
            Transient = false;
            Transcendent = false;
            controller = null;
        }

        public override void Load()
        {
            shape = new Circle(Vector2.Zero, .5f);
            base.Load();

            switch (WarrenInfo.Which)
            {
                case 0:
                    controller = new Warren0(this);
                    CursedGame.AddController(controller);
                    break;
                case 2:
                    controller = new Warren02(this);
                    CursedGame.AddController(controller);
                    break;
                case 3:
                    controller = new Warren03(this);
                    CursedGame.AddController(controller);
                    break;
            }
        }

        public override void Unload()
        {
            CursedGame.RemoveController(controller);
            controller = null;
            base.Unload();
        }

        public override void takeDamage(float damage, Gob source)
        {

        }

        public override void Update(GameTime gameTime)
        {
            WarrenInfo.Energy = 0;
            base.Update(gameTime);
        }

        public WarrenInfo WarrenInfo
        {
            get { return (WarrenInfo)info; }
        }
    }
}
