﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Cursed.gob.mob;
using System.Runtime.Serialization;
using Cursed.view;
using Cursed.col;
using Box2D.XNA;
using Cursed.sound;

namespace Cursed.gob.tile
{
    public class FootSwitchInfo : GroundInfo
    {
        public FootSwitchInfo(int x, int y)
            : base(x, y)
        {
            Label = "door";
            Depth = Const.FootSwitchDepth;
            Preset = "FootSwitch";
        }

        public override Gob Create(CursedGame game)
        {
            return new FootSwitch(game, this);
        }
    }

    public class FootSwitch : Ground
    {
        public FootSwitch(CursedGame game, FootSwitchInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Inert = false;
        }

        public FootSwitchInfo SwitchInfo
        {
            get { return (FootSwitchInfo)info; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            
            foreach (Gob gob in touchedGobs)
            {
                if (gob is Mob && !gob.Transcendent && !(gob is Wood))
                {
                    GobInfo.Triggered = true;
                    break;
                }
            }
            if (GobInfo.Triggered)
            {
                ICollection<Gob> labelGobs = CursedGame.GetLabeledGobs(SwitchInfo.Label);
                foreach (Gob gob in labelGobs)
                {
                    if (gob is Door)
                    {
                        Door door = (Door)gob;
                        door.GobInfo.Triggered = true;
                    }
                }
            }
            base.Update(gameTime);
        }
    }

    public class DoorInfo : WallInfo
    {
        bool open;
        bool canClose;
        bool inverted;

        public DoorInfo(int x, int y)
            : base(x, y)
        {
            open = false;
            canClose = true;
            inverted = false;
            Label = "door";
            Depth = Const.DoorClosedDepth;
            Preset = "StoneDoor";
        }

        public override Gob Create(CursedGame game)
        {
            return new Door(game, this);
        }

        public bool Open
        {
            get { return open; }
            set { open = value; }
        }

        public bool CanClose
        {
            get { return canClose; }
            set { canClose = value; }
        }

        public bool Inverted
        {
            get { return inverted; }
            set { inverted = value; }
        }
    }

    public class Door : Wall
    {
        Sound openSound;
        Sound closeSound;

        public Door(CursedGame game, DoorInfo info)
            : base(game, info)
        {
            Inert = false;
        }

        public override void Load()
        {
            base.Load();
            renderInfo.Mask = new Color(240, 200, 162);
            openSound = SoundIDs.DoorOpen(CursedGame);
            closeSound = SoundIDs.DoorClose(CursedGame);
        }

        public DoorInfo DoorInfo
        {
            get { return (DoorInfo)info; }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            bool occupied = false;
           
            foreach (Gob gob in touchedGobs)
            {
                if (gob is Mob && !gob.Transcendent)
                    occupied = true;
            }

            bool triggered = DoorInfo.Triggered;
            if (DoorInfo.Inverted)
                triggered = !triggered;

            if (DoorInfo.Open && !triggered && !occupied && DoorInfo.CanClose)
            {
                DoorInfo.Open = false;
                closeSound.play();
            }
            else if (!DoorInfo.Open && triggered)
            {
                DoorInfo.Open = true;
                openSound.play();
            }

            if (DoorInfo.Open)
            {
                DoorInfo.Depth = Const.DoorOpenDepth;
            }
            else
            {
                DoorInfo.Depth = Const.DoorClosedDepth;
            }

            Transcendent = DoorInfo.Open;


            B2DCollisionInfo cinfo = (B2DCollisionInfo)CollisionInfo;
            if (cinfo != null)
            {
                foreach (Contact contact in cinfo.contacts)
                {
                    contact.FlagForFiltering();
                }
            }

            base.Update(gameTime);
        }
    }
}
