﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using System.Windows.Media;
using Cursed.view;
using Cursed.gob.mob;
using Cursed.input.ai;
using Cursed.view.renders;
using Cursed.gob.mob.kob;
using Cursed.sound;

namespace Cursed.gob.tile
{
    public class SpawnInfo : TileInfo
    {
        float spawnTime;
        int maxSpawns;
        string infoPreset;

        public SpawnInfo(int x, int y)
            : base(x, y)
        {
            spawnTime = 5;
            maxSpawns = 3;
            infoPreset = "minion";
            Depth = Const.SpawnDepth;

        }

        public override Gob Create(CursedGame game)
        {
            return new Spawn(game, this);
        }

        public float SpawnTime
        {
            get { return spawnTime; }
            set { spawnTime = value; }
        }

        public int MaxSpawns
        {
            get { return maxSpawns; }
            set { maxSpawns = value; }
        }

        public string InfoPreset
        {
            get { return infoPreset; }
            set { infoPreset = value; }
        }
    }

    public class Spawn : Tile
    {
        bool spawned;
        Sound spawnSound;

        public Spawn(CursedGame game, SpawnInfo info)
            : base(game, info)
        {
            Inert = false;
            Transcendent = true;
            spawned = false;
        }

        public SpawnInfo SpawnInfo
        {
            get { return (SpawnInfo)info; }
        }

        public override void Load()
        {
            base.Load();
            spawnSound = SoundIDs.Spawn(CursedGame);
        }

        public override void Update(GameTime gameTime)
        {
            if (!spawned && GobInfo.Triggered)
            {
                spawned = true;
                GobInfo spawnedInfo = GobInfo.getPresetInfo(SpawnInfo.InfoPreset);
                spawnedInfo.Loc = GobInfo.Loc;
                Gob spawnedCreature = spawnedInfo.Create(CursedGame);
                CursedGame.AddGob(spawnedCreature);
                spawnSound.play();
            }
            base.Update(gameTime);
        }
    }

}
