﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Cursed.view.renders;
using Cursed.gob.mob.kob;

namespace Cursed.gob.tile
{
    public class EntranceInfo : TileInfo
    {
        string music = "improv2";

        public EntranceInfo(int x, int y)
            : base(x, y)
        {
            Depth = Const.EntranceDepth;
        }
        public override Gob Create(CursedGame game)
        {
            return new Entrance(game, this);
        }

        public string Music
        {
            get { return music; }
            set { music = value; }
        }
    }

    public class Entrance : Tile
    {
        public Entrance(CursedGame game, EntranceInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Transient = false;
            Inert = true;
        }

        public override void Load()
        {
            base.Load();
            CursedGame.Music = EntranceInfo.Music;
        }

        public EntranceInfo EntranceInfo
        {
            get { return (EntranceInfo)info; }
        }
    }

    public class ExitInfo : TileInfo
    {
        string level;

        public ExitInfo(int x, int y)
            : base(x, y)
        {
            level = "";
            Preset = "Portal";
            Depth = Const.ExitDepth;
        }

        public override Gob Create(CursedGame game)
        {
            return new Exit(game, this);
        }

        public string Level
        {
            get { return level; }
            set { level = value; }
        }
    }

    public class Exit : Tile
    {
        bool touchingPlayer;

        public Exit(CursedGame game, ExitInfo info)
            : base(game, info)
        {
            Transcendent = true;
            Transient = false;
            Inert = false;
        }

        public override void Load()
        {
            shape = new Rect(Vector2.Zero, 1, 1);
            touchingPlayer = true;
            base.Load();
        }

        protected virtual void doWarp()
        {
            if (GobInfo.Label != "" && CursedGame.DruidPlayer.DruidInfo != null)
            {
                DruidInfo dinfo = CursedGame.DruidPlayer.DruidInfo;
                dinfo.Vel = Vector2.Zero;
            }
            CursedGame.SpawnLabel = ExitInfo.Label;
            CursedGame.LoadLevel(ExitInfo.Level);
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            if (gameTime.ElapsedGameTime.Milliseconds != 0)
            {
                if (TouchedGobs.Contains(CursedGame.DruidPlayer))
                {
                    if (!touchingPlayer)
                        doWarp();
                    touchingPlayer = true;
                }
                else
                    touchingPlayer = false;
            }
            base.Update(gameTime);
        }

        public ExitInfo ExitInfo
        {
            get { return (ExitInfo)info; }
        }
    }
}
