﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob;

namespace Cursed.gob.tile
{
    public class WaterInfo : GroundInfo
    {
        bool deep;

        public WaterInfo(int x, int y)
            : base(x, y)
        {
            deep = true;
            Depth = Const.WaterDepth;
            Drag = 5f;
        }

        public override Gob Create(CursedGame game)
        {
            return new Water(game, this);
        }

        public bool Deep
        {
            get { return deep; }
            set { deep = value; }
        }
    }

    public class Water : Ground
    {
        AquaWall aquaWall;

        public Water(CursedGame game, WaterInfo info)
            : base(game, info)
        {

        }

        public override void Load()
        {
            AquaWallInfo awi = new AquaWallInfo(TileInfo.X, TileInfo.Y);
            CursedGame.AddGob(aquaWall = (AquaWall)awi.Create(CursedGame));
            aquaWall.Water = this;
            base.Load();
        }

        public override void Unload()
        {
            CursedGame.RemoveGob(aquaWall);
            base.Unload();
        }

        public bool IsBridged
        {
            get
            {
                foreach (Tile tile in CursedGame.TilesAt(TileInfo.X, TileInfo.Y))
                    if (tile is Bridge)
                    {
                        return true;
                    }
                return false;
            }

        }

        public WaterInfo WaterInfo
        {
            get { return (WaterInfo)info; }
        }
    }

    public class AquaWallInfo : WallInfo
    {
        public AquaWallInfo(int x, int y)
            : base(x, y)
        {
            Preset = "";
        }

        public override Gob Create(CursedGame game)
        {
            return new AquaWall(game, this);
        }
    }

    public class AquaWall : Wall
    {
        Water water;

        public AquaWall(CursedGame game, AquaWallInfo info)
            : base(game, info)
        {
            Transient = true;
        }

        public Water Water
        {
            get { return water; }
            set { water = value; }
        }

        public override bool CanTouch(Gob other)
        {
            if (other is Mob && ((Mob)other).MobInfo.Aquaphobic)
            {
                return true;
            }
            if (other is Mob && ((Mob)other).MobInfo.Flying)
            {
                return false;
            }
            return Water.WaterInfo.Deep && !Water.IsBridged;
        }
    }
}
