﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.view;
using Microsoft.Xna.Framework.Input;

namespace Cursed.input
{
    public class MouseTrailController : MouseController
    {
        protected MouseTrailInfo trailInfo;
        bool mouseButtonPushed = false;
        public MouseTrailController(ICamera camera, GameWindow window)
            : base(camera, window)
        {
            trailInfo = new MouseTrailInfo();
        }

        protected virtual void updateTrail()
        {
            getMousePos();
            if (!Camera.Bounds.Contains(CurrentMousePosition))
                return;

            if (trailInfo.Type == MouseType.None)
            {
                trailInfo.Modifier = Keyboard.GetState()[Keys.Space] == KeyState.Down;
                trailInfo.Trail.Clear();
                if (Mouse.GetState().LeftButton == ButtonState.Pressed)
                {
                    trailInfo.Type = MouseType.Offensive;
                    mouseButtonPushed = true;
                }
                else if (Mouse.GetState().RightButton == ButtonState.Pressed)
                {
                    trailInfo.Type = MouseType.Defensive;
                    mouseButtonPushed = true;
                }
            }

            if (trailInfo.Type == MouseType.Offensive && mouseButtonPushed)
            {
                if (Mouse.GetState().LeftButton == ButtonState.Released)
                {
                    mouseButtonPushed = false;
                    trailInfo.Finished = true;
                }
                else
                {

                    trailInfo.Trail.Add(CurrentMousePosition);
                }
            }

            if (trailInfo.Type == MouseType.Defensive && mouseButtonPushed)
            {
                if (Mouse.GetState().RightButton == ButtonState.Released)
                {
                    mouseButtonPushed = false;
                    trailInfo.Finished = true;
                }
                else
                {
                    trailInfo.Trail.Add(CurrentMousePosition);
                }
            }
        }

        public override void act(GameTime time)
        {
            base.act(time);
            updateTrail();

            if (Target != null && trailInfo.Trail.Count != 0)
                Target.control(trailInfo);

            if (trailInfo.Finished)
            {
                /*
                System.Console.WriteLine("Trail type " + info.Type);
                foreach (Vector2 pos in info.Trail)
                {
                    System.Console.WriteLine(pos);
                }
                 */
                trailInfo = new MouseTrailInfo();
                
            }
            
        }
    }
}
