﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob;
using Microsoft.Xna.Framework;
using Cursed.gob.tile;

namespace Cursed.input.ai
{
    public struct TC
    {
        int x;
        int y;
        public TC(int x, int y)
        {
            this.x = x;
            this.y = y;
        }
        public TC(Vector2 loc)
        {
            x = (int)Math.Floor(loc.X + .5f);
            y = (int)Math.Floor(loc.Y + .5f);
        }
        public Vector2 Vector2
        {
            get { return new Vector2(x, y); }
            set
            {
                x = (int)Math.Floor(value.X + .5f);
                y = (int)Math.Floor(value.Y + .5f);
            }
        }
        public int X
        {
            get { return x; }
            set { x = value; }
        }
        public int Y
        {
            get { return y; }
            set { y = value; }
        }
        public bool Equals(TC second)
        {
            return x == second.x && y == second.y;
        }
        public List<TC> Neighbors
        {
            get
            {
                List<TC> n = new List<TC>();

                n.Add(new TC(x - 1, y + 1));
                n.Add(new TC(x, y + 1));
                n.Add(new TC(x + 1, y + 1));
                n.Add(new TC(x - 1, y - 1));
                n.Add(new TC(x, y - 1));
                n.Add(new TC(x + 1, y - 1));
                n.Add(new TC(x - 1, y));
                n.Add(new TC(x + 1, y));

                return n;
            }
        }
        public override string ToString()
        {
            return "TC(" + x + "," + y + ")";
        }
    }

    public class FindMe : IComparer<TC>
    {


        Gob me;
        int width;
        bool flying;
        bool aquaphobic;

        int minX, minY;
        bool[,] walls;
        bool[,] closedDoors;
        bool[,] visited;
        float[,] costs;
        Vector2[,] directions;
        TC goal;

        List<TC> fringe;

        Vector2 start;

        public FindMe(Gob me, int width, bool flying, bool aquaphobic)
        {
            this.me = me;
            this.width = width;
            this.flying = flying;
            this.aquaphobic = aquaphobic;
            goal = new TC(Int32.MaxValue, Int32.MaxValue);
        }

        void updateWalls()
        {

            CursedGame.TheGame.Profiler.Start("FindMe.updateWalls");
            CursedGame game = me.CursedGame;
            minX = Int32.MaxValue;
            minY = Int32.MaxValue;
            int maxX = -1 * Int32.MaxValue;
            int maxY = -1 * Int32.MaxValue;
            foreach (Tile tile in game.Tiles)
            {
                minX = Math.Min(minX, tile.TileInfo.X);
                minY = Math.Min(minY, tile.TileInfo.Y);
                maxX = Math.Max(maxX, tile.TileInfo.X);
                maxY = Math.Max(maxY, tile.TileInfo.Y);
            }
            int spanX = maxX - minX + 1;
            int spanY = maxY - minY + 1;
            walls = new bool[spanX, spanY];
            closedDoors = new bool[spanX, spanY];
            visited = new bool[spanX, spanY];
            costs = new float[spanX, spanY];
            directions = new Vector2[spanX, spanY];

            foreach (Tile tile in game.Tiles)
            {
                int ox = tile.TileInfo.X - minX;
                int oy = tile.TileInfo.Y - minY;
                if (tile is Water && (aquaphobic || (((Water)tile).WaterInfo.Deep) && !flying))
                    walls[ox, oy] = true;
                if (tile is Wall && !(tile is Door))
                {
                    if (!flying || !((Wall)tile).WallInfo.IsShort)
                        walls[ox, oy] = true;
                }
                if (tile is Door && !((Door)tile).DoorInfo.Open)
                {
                    closedDoors[ox, oy] = true;
                    walls[ox, oy] = true;
                }
            }

            if (width > 1)
            {
                bool[,] walls2 = new bool[spanX, spanY];
                for (int tx = 0; tx < spanX; tx++)
                {
                    for (int ty = 0; ty < spanY; ty++)
                    {
                        TC tc = new TC(tx + minX, ty + minY);
                        //if (Wall(tc))//if (walls[tc.X - minX, tc.Y - minY])
                        walls2[tc.X - minX, tc.Y - minY] = Wall(tc);
                        for (int d = 1; d < width && !walls2[tc.X - minX, tc.Y - minY]; d++)
                        {
                            bool above = false;
                            for (int x = -d; x <= d; x++)
                                above |= Wall(new TC(tx + minX + x, ty + minY + d));// walls[tc.X - minX + x, tc.Y - minY + d];
                            bool below = false;
                            for (int x = -d; x <= d; x++)
                                below |= Wall(new TC(tx + minX + x, ty + minY - d));//walls[tc.X - minX + x, tc.Y - minY - d];
                            bool left = false;
                            for (int y = -d; y <= d; y++)
                                left |= Wall(new TC(tx + minX - d, ty + minY + y));//walls[tc.X - minX - d, tc.Y - minY + y];
                            bool right = false;
                            for (int y = -d; y <= d; y++)
                                right |= Wall(new TC(tx + minX + d, ty + minY + y));//walls[tc.X - minX + d, tc.Y - minY + y];

                            if (above && below || left && right)
                                walls2[tc.X - minX, tc.Y - minY] = true;
                        }
                    }
                }
                walls = walls2;
            }


            goal = new TC(me.GobInfo.Loc);
            this.fringe = new List<TC>();
            fringe.Add(goal);
            Visit(goal);
            SetCost(goal, 0);
            SetDirection(goal, Vector2.Zero);

            CursedGame.TheGame.Profiler.Stop("FindMe.updateWalls");
        }

        bool Visited(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return true;
            return visited[tc.X - minX, tc.Y - minY];
        }

        bool Wall(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return true;
            if (walls[tc.X - minX, tc.Y - minY])
                return true;
            return false;
        }

        bool ClosedDoor(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return false;
            if (closedDoors[tc.X - minX, tc.Y - minY])
                return true;
            return false;
        }

        void Visit(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return;
            visited[tc.X - minX, tc.Y - minY] = true;
        }

        Vector2 Direction(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return goal.Vector2 - tc.Vector2;
            return directions[tc.X - minX, tc.Y - minY];
        }

        float Cost(TC tc)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return Single.PositiveInfinity;
            return costs[tc.X - minX, tc.Y - minY];
        }

        void SetDirection(TC tc, Vector2 dir)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return;
            directions[tc.X - minX, tc.Y - minY] = dir;
        }

        void SetCost(TC tc, float cost)
        {
            if (tc.X < minX || tc.Y < minY || tc.X >= minX + visited.GetLength(0) || tc.Y >= minY + visited.GetLength(1))
                return;
            costs[tc.X - minX, tc.Y - minY] = cost;
        }

        bool Pop()
        {

            CursedGame.TheGame.Profiler.Start("FindMe.Pop");
            TC cur = fringe[0];


            float cost = Cost(cur);

            //if (cost + H(cur.Vector2) > range)
            //    return false;

            fringe.RemoveAt(0);


            //System.Console.WriteLine(cur);

            Visit(cur);

            if (Wall(cur))
            {
                CursedGame.TheGame.Profiler.Stop("FindMe.Pop");
                return true;
            }


            foreach (TC neighbor in cur.Neighbors)
            {
                Vector2 dir = cur.Vector2 - neighbor.Vector2;
                if (dir.X != 0 && dir.Y != 0)
                {
                    if (Wall(new TC(cur.X, neighbor.Y)) || Wall(new TC(neighbor.X, cur.Y)))
                        continue;
                }

                float stepCost = dir.Length();
                if (false && ClosedDoor(neighbor))
                    stepCost += 100;
                dir.Normalize();
                float nc = cost + stepCost;
                if (!Wall(neighbor) && (!Visited(neighbor) || Cost(neighbor) > nc))
                {
                    Visit(neighbor);
                    SetDirection(neighbor, dir);
                    SetCost(neighbor, nc);
                    fringe.Add(neighbor);
                    //System.Console.WriteLine(" " + neighbor + " : " + dir + ", " + nc);
                }
            }
            CursedGame.TheGame.Profiler.Stop("FindMe.Pop");
            return true;
        }

        public Vector2 From(Vector2 start, float range)
        {
            CursedGame.TheGame.Profiler.Start("FindMe.From");
            this.start = start;

            if (!goal.Equals(new TC(me.GobInfo.Loc)))
                updateWalls();

            range = 2 * (start - goal.Vector2).Length();
            range = Single.PositiveInfinity;

            TC startTC = new TC(start);

            List<TC> startTCs = new List<TC>();
            startTCs.Add(new TC((int)Math.Floor(start.X), (int)Math.Floor(start.Y)));
            startTCs.Add(new TC((int)Math.Ceiling(start.X), (int)Math.Floor(start.Y)));
            startTCs.Add(new TC((int)Math.Floor(start.X), (int)Math.Ceiling(start.Y)));
            startTCs.Add(new TC((int)Math.Ceiling(start.X), (int)Math.Ceiling(start.Y)));

            int pops = 0;


            CursedGame.TheGame.Profiler.Start("FindMe.From pops");

            while (fringe.Count > 0 && pops < Const.MaxPathPops)
            {

                bool visitedAll = true;
                foreach (TC tc in startTCs)
                    visitedAll &= Visited(tc);
                if (visitedAll)
                {
                    //System.Console.WriteLine("found path");
                    break;
                }

                CursedGame.TheGame.Profiler.Start("FindMe.From fringe sort");
                fringe.Sort(this);
                CursedGame.TheGame.Profiler.Stop("FindMe.From fringe sort");
                Pop();
                pops++;
            }
            CursedGame.TheGame.Profiler.Stop("FindMe.From pops");

            //if (pops > 0)
            //    System.Console.WriteLine(pops);

            Vector2 dir = Vector2.Zero;
            foreach (TC tc in startTCs)
            {
                Vector2 tcd = Direction(tc);
                if (tcd == Vector2.Zero)
                    continue;
                float d = 1 - Math.Min(1, (tc.Vector2 - start).Length());
                dir += tcd * d;
            }
            if (dir == Vector2.Zero)
                dir = me.GobInfo.Loc - start;
            if (dir != Vector2.Zero)
                dir.Normalize();
                

            CursedGame.TheGame.Profiler.Stop("FindMe.From");
            return dir;
        }



        #region IComparer<TC> Members

        float H(Vector2 loc)
        {
            return (loc - start).Length();
        }

        public int Compare(TC x, TC y)
        {
            float cx = Cost(x) + H(x.Vector2);
            float cy = Cost(y) + H(y.Vector2);
            if (cx < cy)
                return -1;
            if (cy < cx)
                return 1;
            return 0;
        }

        #endregion
    }
}
