﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob;
using Cursed.gob.mob.kob;
using Cursed.gob.mob;
using Microsoft.Xna.Framework;

namespace Cursed.input.ai
{
    public class GreedyFamiliarControl : Controller
    {
        Familiar familiar;
        //PathFinder pathFinder;

        public GreedyFamiliarControl(Familiar familiar)
        {
            this.familiar = familiar;
            //pathFinder = new PathFinder(familiar.CursedGame, 10);
            //pathFinder.Depth = 30;
            Push(familiar);
        }

        public override void act(Microsoft.Xna.Framework.GameTime time)
        {
            float bestScore = 0;
            Gob bestGob = null;
            int attackableCount = 0;
            Vector2 attackDirection = Vector2.Zero;
            foreach (Gob seenGob in familiar.SeenMobs)
            {
                if (!familiar.CanTouch(seenGob))
                    continue;
                Vector2 distance = familiar.GobInfo.Loc - seenGob.GobInfo.Loc;
                float score = 0;// (float)Math.Pow(distance.Length(), -.5f);
                if (seenGob is LifeBlob)
                {
                    LifeBlob seenBlob = (LifeBlob)seenGob;
                    if (familiar.TouchedGobs.Contains(seenBlob))
                        continue;
                    if (seenBlob.Source != familiar.Master)
                        continue;
                    score += seenBlob.LifeBlobInfo.Amount;
                    if (seenBlob.LifeBlobInfo.Amount < 0)
                        score = 0;
                }
                else if (seenGob is Kob)
                {
                    Kob seenKob = (Kob)seenGob;
                    if (seenKob.KobInfo.Energy <= Const.FamiliarEnergyVisibilityThreshold)
                        continue;
                    score += seenKob.KobInfo.Energy;
                    if (score > 0 && familiar.TouchedGobs.Contains(seenKob))
                    {
                        attackableCount++;
                        if (attackDirection != Vector2.Zero)
                        {
                            attackDirection = seenKob.GobInfo.Loc - familiar.GobInfo.Loc;
                        }
                    }
                }
                else
                {
                    score = 0;
                }


                if (score > bestScore)
                {
                    bestScore = score;
                    bestGob = seenGob;
                }
            }


            Vector2 direction = Vector2.Zero;
            bool touchesMaster = familiar.Master != null && familiar.Master.TouchedGobs.Contains(bestGob);

            if (familiar.Master != null && familiar.Master.LastAttacker != null)
            {
                if (familiar.Master.LastAttacker is Kob)
                {
                    if (((Kob)familiar.Master.LastAttacker).KobInfo.Energy > 0)
                    {
                        bestGob = familiar.Master.LastAttacker;
                    }
                }
            }

            if (bestGob != null)
            {
                FindMe findme = bestGob.FindMe(familiar);
                //pathFinder.Destination = bestGob.GobInfo.Loc;
                if (touchesMaster)
                    findme = familiar.Master.FindMe(familiar);
                direction = findme.From(familiar.GobInfo.Loc, familiar.GobInfo.ViewRange);
                //direction = bestGob.GobInfo.Loc - familiar.GobInfo.Loc;
                if (direction != Vector2.Zero)
                    direction.Normalize();
            }
            if ((bestGob is Kob && familiar.TouchedGobs.Contains(bestGob)))
            {
                attackDirection = bestGob.GobInfo.Loc - familiar.GobInfo.Loc;
            }

            float bufferToBest = 0;
            if (bestGob != null)
                bufferToBest = (bestGob.GobInfo.Loc - familiar.GobInfo.Loc).Length();
            if (bestGob is Mob)
            {
                bufferToBest -= ((Mob)bestGob).MobInfo.Scale * .5f;
            }
            else
            {
                bufferToBest -= .5f;
            }
            bufferToBest -= familiar.MobInfo.Scale * .5f;


            if (Target != null)
            {
                DirectionControlInfo directionInfo = new DirectionControlInfo();
                directionInfo.Direction = Vector2.Zero;


                if (!touchesMaster && bestGob is Kob)
                {
                    float d = (bestGob.GobInfo.Loc - familiar.GobInfo.Loc).Length();
                    if (d > familiar.DemonInfo.LungeRange - 1 && d < familiar.DemonInfo.LungeRange)
                    {
                        Target.control(new LungeInfo());
                        Vector2 dir = bestGob.GobInfo.Loc - familiar.GobInfo.Loc;
                        dir.Normalize();
                        directionInfo.Direction = dir;
                    }
                }
                if (touchesMaster && bufferToBest < 1)
                {
                    directionInfo.Direction = Vector2.Transform(direction, Matrix.CreateRotationZ(MathHelper.PiOver4 + .2f));
                }
                else if (touchesMaster && bufferToBest < 2)
                {
                    directionInfo.Direction = Vector2.Transform(direction, Matrix.CreateRotationZ(MathHelper.PiOver4));
                }
                else
                {
                    directionInfo.Direction = direction;
                }

                Target.control(directionInfo);
            }

            if (attackableCount > 0)
            {
                BasicAttackInfo attackInfo = new BasicAttackInfo();
                attackInfo.Direction = direction;
                if (familiar.DemonInfo.SpinAttack && attackableCount > 1)
                    attackInfo.Type = BasicAttackInfo.ATTACKTYPE.SPIN;
                if (Target != null)
                {
                    Target.control(attackInfo);
                }
            }
        }

        public override void cleanup()
        {

        }
    }
}
