﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.gob.mob.kob;
using Microsoft.Xna.Framework;
using Cursed.gob.mob;

namespace Cursed.input.ai
{
    public class Warren0 : Controller
    {
        protected CursedGame game;
        protected Warren warren;

        //PathFinder pathFinder;

        bool askedWood;
        bool seenWood;
        bool seenMonster;
        bool taughtCasting;
        bool summonedFamiliar;
        bool taughtLife;
        bool castLife;

        Sticker castWoodSticker;
        Sticker summonSticker;
        Sticker castLifeSticker;

        DialogMob lastDlog;

        public Warren0(Warren warren)
        {
            game = warren.CursedGame;
            this.warren = warren;
            //this.pathFinder = new PathFinder(warren.CursedGame, 10);
            Push(warren);

            askedWood = false;
            lastDlog = null;
            castWoodSticker = null;
            castLifeSticker = null;
            seenWood = false;
            seenMonster = false;
            taughtCasting = false;
            summonedFamiliar = false;
            taughtLife = false;
            castLife = false;
        }

        protected void say(string message, float timing)
        {
            game.RemoveGob(lastDlog);
            DialogMobInfo dmi = new DialogMobInfo();
            dmi.Label = "Warren";
            dmi.Line = new Line("Warren", 0, timing, message);
            game.AddGob(lastDlog = (DialogMob)dmi.Create(game));
        }

        protected void chaseDruid()
        {
            Vector2 dst = game.DruidPlayer.GobInfo.Loc;
            Vector2 src = warren.GobInfo.Loc;


            //pathFinder.Destination = dst;
            //Vector2 dir = pathFinder.GetDirection(src);
            Vector2 dir = game.DruidPlayer.FindMe(warren).From(warren.GobInfo.Loc, warren.GobInfo.ViewRange);

            double d = (dst - src).Length();

            if (d <= 3 && d > 2)
                dir *= 0.25f;
            if (d <= 2 && d > 1.5)
                dir *= 0;
            if (d <= 1.5)
                dir *= -0.25f;

            DirectionControlInfo dci = new DirectionControlInfo();
            dci.Direction = dir;
            warren.control(dci);
        }

        public override void act(Microsoft.Xna.Framework.GameTime time)
        {
            if (game.DruidPlayer == null)
                return;

            if (!askedWood)
            {
                warren.MobInfo.Stuck = true;
                string message = "Oh good you're alive!;";
                message += "I'm glad to see you're alright...;";
                message += "...Last thing I remember is getting this nasty bump on my head.;";
                message += "Some initiation ceremony, huh?;";
                message += "I'm not sure what happened, but it seems to have affected your initiation...;";
                message += "Can you cast your entangling spell?";
                say(message, 5);
                askedWood = true;
            }
            if (!seenWood)
            {
                if (askedWood && !lastDlog.Loaded && castWoodSticker == null)
                {
                    StickerInfo stickerInfo = new StickerInfo();
                    stickerInfo.Preset = "CastWood";
                    stickerInfo.Depth = Const.ExitDepth;
                    stickerInfo.Loc = warren.GobInfo.Loc - new Vector2(0, 1);
                    game.AddGob(castWoodSticker = (Sticker)stickerInfo.Create(game));
                }

                foreach (Mob mob in warren.SeenMobs)
                {
                    if (mob is Wood)
                    {
                        seenWood = true;

                        game.RemoveGob(castWoodSticker);

                        string msg = "Thats strange...I...";
                        msg += ";Wait, What was that?";
                        msg += ";Let's get out of here!";
                        msg += ";I'll follow you...";
                        say(msg, 2);
                        warren.MobInfo.Stuck = false;

                        return;
                    }
                }
            }

            if (seenWood && !seenMonster)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("seemonster");
                if (tz.TouchedGobs.Contains(warren))
                {
                    seenMonster = true;
                    string msg = "Oh spirits, what is that?!";
                    msg += ";Quick, Down the hallway to the left!";
                    msg += ";See if you can slow it down with your spell...";
                    say(msg, 2);
                }
            }

            if (seenMonster && !taughtCasting)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("hidecloset");
                if (tz.TouchedGobs.Contains(warren))
                {
                    taughtCasting = true;
                    string msg = "Those were evil spirits! Real evil spirits!";
                    msg += ";Why didn't you summon your familiar?";
                    msg += ";Don't look at me, you know I don't have one.";
                    msg += ";What do you mean you don't remember!?";
                    msg += ";You've been training your whole life for this!";
                    msg += ";That blast must have boggled your mind.";
                    msg += ";Here let me show you...";
                    say(msg, 3f);
                }
            }

            if (taughtCasting && !summonedFamiliar)
            {
                if (!lastDlog.Loaded && summonSticker == null)
                {
                    StickerInfo stickerInfo = new StickerInfo();
                    stickerInfo.Preset = "SummonInstructions";
                    stickerInfo.Depth = Const.ExitDepth;
                    stickerInfo.Loc = new Vector2(-15, 3);
                    game.AddGob(summonSticker = (Sticker)stickerInfo.Create(game));

                }
                foreach (Mob mob in warren.SeenMobs)
                {
                    if (mob is Familiar)
                    {
                        game.RemoveGob(summonSticker);
                        summonedFamiliar = true;
                    }
                }
            }
            if (summonedFamiliar && !taughtLife)
            {
                taughtLife = true;
                string msg = "There you go.";
                msg += ";Ok listen closely, you can't kill those spirits.";
                msg += "; They're already dead, so to speak.";
                msg += ";You need to first bring it to the living plane to kill it.";
                msg += ";Use your life spell to bring them this plane...";
                msg += ";And your familiar will take care of the rest.";
                msg += ";Now come on, lets get out of here.";
                say(msg, 3f);
            }

            if (taughtLife)
            {
                if (!lastDlog.Loaded && castLifeSticker == null)
                {
                    StickerInfo stickerInfo = new StickerInfo();
                    stickerInfo.Preset = "AttackInstructions";
                    stickerInfo.Depth = Const.ExitDepth;
                    stickerInfo.Loc = new Vector2(-15, 3);
                    game.AddGob(castLifeSticker = (Sticker)stickerInfo.Create(game));

                }
            }

            chaseDruid();

        }

        public override void cleanup()
        {

        }
    }
    public class Warren02 : Warren0
    {
        bool level2;
        bool doorRoom;
        bool mainEntrance;
        public Warren02(Warren warren)
            : base(warren)
        {
            level2 = false;
            doorRoom = false;
            mainEntrance = false;
        }

        public override void act(GameTime time)
        {
            if (game.DruidPlayer == null)
                return;
            if (!level2)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("level2");
                if (tz.TouchedGobs.Contains(warren))
                {
                    level2 = true;
                    string msg = "The main entrance is just east of here";
                    msg += ";Did you see that? I could have sworn I saw something";

                    say(msg, 3f);
                }
            }
            if (!doorRoom)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("doorRoom");
                if (tz.TouchedGobs.Contains(warren))
                {
                    doorRoom = true;
                    string msg = "The switch is on the other side";
                    msg += ";Send your familiar around to open it, we can let it once we are on the other side";
                    say(msg, 3f);
                }
            }
            if (!mainEntrance)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("mainEntrance");
                if (tz.TouchedGobs.Contains(warren))
                {
                    mainEntrance = true;
                    string msg = "The Door is blocked!";
                    msg += ";What is going on around here!?...";
                    msg += ":Evil spirits, all this blood...and now this!";
                    msg += ";It must have been one of the elders...";
                    msg += ";But why?";
                    msg += ";It could have been any of them...";
                    msg += ";Dear Spritis, it could have been all of them!";
                    msg += ";We need to get out of here and find them!";
                    msg += ";There are caves that run under the academy";
                    msg += ";We can use them to get out";
                    msg += ";Come on, there an entrance to the south";

                    say(msg, 3f);
                }
            }
            chaseDruid();
        }

    }
    public class Warren03 : Warren0
    {
        bool level3;
        bool level3exit;
        bool wrongway;
        public Warren03(Warren warren)
        :base(warren)
        {
            level3 = false;
            level3exit = false;
            wrongway = false;
        }

        public override void act(GameTime time)
        {
            if (game.DruidPlayer == null)
                return;
            if (!level3)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("level3");
                if (tz.TouchedGobs.Contains(warren))
                {
                    level3 = true;
                    string msg = "I haven't been down here in ages";
                    msg += ";I'm not even sure which way is out...";

                    say(msg, 3f);
                }
            }
            if (!level3exit)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("level3exit");
                if (tz.TouchedGobs.Contains(warren))
                {
                    level3exit = true;
                    string msg = "Theres the exit!";
                    msg += ";Come on lets go!";
                    say(msg, 3f);
                }
            }
            if (!wrongway)
            {
                TriggerZone tz = (TriggerZone)game.GetFirstLabeledGob("wrongway");
                if (tz.TouchedGobs.Contains(warren))
                {
                    wrongway = true;
                    string msg = "Not this way!";
                    msg += ";Definetly not this way!";
                    msg += ";RUN!";
                    say(msg, 1f);
                }
            }
            chaseDruid();
        }

    }
}
