﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Cursed.input.ai.rl
{
    public struct DruidHunterState : State
    {
        static Random rand = new Random();
        static Vector2 North = new Vector2(0, 1);
        static Vector2 South = new Vector2(0, -1);
        static Vector2 East = new Vector2(1, 0);
        static Vector2 West = new Vector2(-1, 0);
        static Vector2 NorthEast = new Vector2(1, 1);
        static Vector2 SouthWest = new Vector2(-1, -1);
        static Vector2 SouthEast = new Vector2(1, 1);
        static Vector2 NorthWest = new Vector2(-1, 1);

        int minX, minY;
        int spanX, spanY;
        int span;

        int druid_x, druid_y;
        int familiar_x, familiar_y;
        int demon_x, demon_y;

        int index;

        float[,] rewards;

        public DruidHunterState(int minX, int minY, float[,] rewards)
        {
            this.minX = minX;
            this.minY = minY;
            this.rewards = rewards;
            spanX = rewards.GetLength(0);
            spanY = rewards.GetLength(1);
            span = spanX * spanY;
            druid_x = minX;
            familiar_x = minX;
            demon_x = minX;
            druid_y = minY;
            familiar_y = minY;
            demon_y = minY;
            index = 0;
        }

        int druidIndexPart()
        {
            return (druid_x - minX) + spanX * (druid_y - minY);
        }
        int demonIndexPart()
        {
            return span * ((demon_x - minX) + spanX * (demon_y - minY));
        }
        int familiarIndexPart()
        {
            return 0;
            //return span * span * ((familiar_x - minX) + spanX * (familiar_y - minY));
        }

        public Vector2 Druid
        {
            set
            {
                int old_part = druidIndexPart();
                druid_x = (int)Math.Floor(value.X + 0.5f);
                druid_y = (int)Math.Floor(value.Y + 0.5f);
                index -= old_part;
                index += druidIndexPart();
            }
            get
            {
                return new Vector2(druid_x, druid_y);
            }
        }
        public Vector2 Familiar
        {
            set
            {
                int old_part = familiarIndexPart();
                familiar_x = (int)Math.Floor(value.X + 0.5f);
                familiar_y = (int)Math.Floor(value.Y + 0.5f);
                index -= old_part;
                index += familiarIndexPart();
            }
            get
            {
                return new Vector2(familiar_x, familiar_y);
            }
        }
        public Vector2 Demon
        {
            set
            {
                int old_part = demonIndexPart();
                demon_x = (int)Math.Floor(value.X + 0.5f);
                demon_y = (int)Math.Floor(value.Y + 0.5f);
                index -= old_part;
                index += demonIndexPart();
            }
            get
            {
                return new Vector2(demon_x, demon_y);
            }
        }

        #region IState Members

        public int Index
        {
            get { return index; }
        }

        float rewardAt(Vector2 loc)
        {
            int x = (int)Math.Floor(loc.X);
            int y = (int)Math.Floor(loc.Y);
            int ox = x - minX;
            int oy = y - minY;
            if (ox < 0 || ox >= spanX || oy < 0 || oy >= spanY)
                return Single.NegativeInfinity;
            return rewards[ox, oy];
        }

        public Vector2 updatePosition(Vector2 loc, Action action)
        {
            Vector2 dst = loc;
            switch (action)
            {
                case Action.N:
                    dst += North;
                    break;
                case Action.NE:
                    dst += NorthEast;
                    break;
                case Action.E:
                    dst += East;
                    break;
                case Action.SE:
                    dst += SouthEast;
                    break;
                case Action.S:
                    dst += South;
                    break;
                case Action.SW:
                    dst += SouthWest;
                    break;
                case Action.W:
                    dst += West;
                    break;
                case Action.NW:
                    dst += NorthWest;
                    break;
            }
            return dst;
        }

        Action RandomAction
        {
            get { return (Action)rand.Next((int)Action._MAX_); }
        }

        public override string ToString()
        {
            return "s{" + Druid + "," + Familiar + "," + Demon + "}";
        }

        public State Next(Action action)
        {
            DruidHunterState dhs = this;
            //dhs.Druid = updatePosition(Druid, RandomAction);
            //dhs.Familiar = updatePosition(Familiar, RandomAction);
            dhs.Demon = updatePosition(Demon, action);
            return dhs;
        }

        public float R(Action action)
        {
            float dll = (Druid - Demon).LengthSquared();
            float fll = (Familiar - Demon).LengthSquared();

            float r = rewardAt(Demon);
            if (dll < 1)
                return 0;
            return r;
        }

        public float H()
        {
            return rewardAt(Demon) - (Druid - Demon).Length();
        }

        #endregion
    }
}
