﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace Cursed.input.util
{
    public class WGesture
    {
        public enum Direction
        {
            Up, Right, Down, Left
        }

        List<Direction> directions;

        public WGesture(List<Vector2> trail)
        {
            directions = new List<Direction>();
            for (int i = 0; i < trail.Count - 1; i++)
            {
                directions.Add(getDir(trail[i + 1] - trail[i]));
            }
            //first, remove instances of a direction that lasted for only 2 steps
            for (int j = 0; j < 2 && directions.Count > 1; j++)
            {
                List<int> singles = new List<int>();
                for (int i = directions.Count - 1; i > 0; i--)
                {
                    if (directions[i] != directions[i - 1])
                        singles.Add(i);
                }
                singles.Add(0);
                foreach (int index in singles)
                {
                    directions.RemoveAt(index);
                }
            }
            
            //then, collapse all the like directions
            List<int> doubles = new List<int>();
            for (int i = directions.Count - 1; i > 0; i--)
            {
                if (directions[i] == directions[i - 1])
                    doubles.Add(i);
            }
            foreach (int index in doubles)
            {
                directions.RemoveAt(index);
            }
        }

        public Direction this[int index]
        {
            get { return directions[index]; }
        }
        public int Count
        {
            get { return directions.Count; }
        }

        public bool Matches(List<Direction> directions)
        {
            if (Count != directions.Count)
                return false;
            for (int i = 0; i < Count; i++)
                if (this[i] != directions[i])
                    return false;
            return true;
        }

        public override string ToString()
        {
            string s = "{";
            foreach (Direction d in directions)
            {
                if (d == Direction.Up)
                    s += "U";
                if (d == Direction.Down)
                    s += "D";
                if (d == Direction.Right)
                    s += "R";
                if (d == Direction.Left)
                    s += "L";
            }
            s += "}";
            return s;
        }

        Direction getDir(Vector2 d)
        {
            if (Math.Abs(d.Y) > Math.Abs(d.X))
            {
                if (d.Y > 0)
                    return Direction.Up;
                return Direction.Down;
            }
            if (d.X > 0)
                return Direction.Right;
            return Direction.Left;
        }
    }
}
