﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Windows;
using Cursed.geom;


namespace Cursed.view
{
    class LifePostProcessor
    {
        int h;
        int w;
        ICamera camera;
        VertexDeclaration vd;
        VertexPositionColorTexture[] vertices;
        Effect effect;
        public LifePostProcessor(int h, int w, ICamera camera, Effect effect)
        {
            this.h = h;
            this.w = w;
            this.camera = camera;
            this.effect = effect;

            updateVertices();
        }
        public void setSize(int h, int w)
        {
            this.h = h;
            this.w = w;
        }
        public void draw(GraphicsDevice device, LifeMap map, Texture2D text)
        {
            if (map != null)
            {
                if (map.Cols != w || map.Rows != h)
                    setGrid(map.Rows, map.Cols);
                applyMap(map);
            }
            effect.Parameters["View"].SetValue(camera.View);
            effect.Parameters["Projection"].SetValue(camera.Projection);
            effect.Parameters["EyePosition"].SetValue(camera.Position3);
            effect.Parameters["BasicTexture"].SetValue(text);

            Matrix world = Microsoft.Xna.Framework.Matrix.CreateScale(camera.ViewWidth, -camera.ViewHeight, 1)
                *Matrix.CreateTranslation(new Vector3(camera.Position,0)
                -new Vector3(camera.ViewWidth,-camera.ViewHeight,0)/2);
            
            effect.Parameters["World"].SetValue(world);
            effect.Begin();

            effect.Parameters["ShaderIndex"].SetValue(3);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                device.VertexDeclaration = vd;
                device.DrawUserPrimitives(PrimitiveType.TriangleList, vertices, 0, w*h*2);
                pass.End();
            }

            effect.End();
        }
        public void updateVertices()
        {
            vertices = new VertexPositionColorTexture[w*h*6];
            for(int i = 0; i<h-1;i++)
            {
                for (int j = 0; j < w-1; j++)
                {
                    //first triangel |\
                    
                    float m = i;
                    float n = j;
                    vertices[(j + i * w)*6] = new VertexPositionColorTexture();
                    vertices[(j + i * w)*6].Position = new Vector3(n / (w-1), m / (h-1), 0);
                    vertices[(j + i * w)*6].Color = Color.White;
                    vertices[(j + i * w) * 6].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                   
                    m = i+1;
                    n = j+1;
                    vertices[(j + i * w) * 6+1] = new VertexPositionColorTexture();
                    vertices[(j + i * w) * 6+1].Position = new Vector3(n / (w - 1), m / (h - 1), 0);
                    vertices[(j + i * w) * 6+1].Color = Color.White;
                    vertices[(j + i * w) * 6+1].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                    
                    m = i+1;
                    n = j;
                    vertices[(j + i * w) * 6+2] = new VertexPositionColorTexture();
                    vertices[(j + i * w) * 6+2].Position = new Vector3(n / (w - 1), m / (h - 1), 0);
                    vertices[(j + i * w) * 6+2].Color = Color.White;
                    vertices[(j + i * w) * 6+2].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                    
                    //second triangle \|
                    m = i;
                    n = j;
                    vertices[(j + i * w) * 6+3] = new VertexPositionColorTexture();
                    vertices[(j + i * w) * 6+3].Position = new Vector3(n / (w - 1), m / (h - 1), 0);
                    vertices[(j + i * w) * 6+3].Color = Color.White;
                    vertices[(j + i * w) * 6+3].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                   
                    m = i;
                    n = j+1;
                    vertices[(j + i * w) * 6+4] = new VertexPositionColorTexture();
                    vertices[(j + i * w) * 6+4].Position = new Vector3(n / (w - 1), m / (h - 1), 0);
                    vertices[(j + i * w) * 6+4].Color = Color.White;
                    vertices[(j + i * w) * 6+4].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                   
                    m = i+1;
                    n = j+1;
                    vertices[(j + i * w) * 6+5] = new VertexPositionColorTexture();
                    vertices[(j + i * w) * 6+5].Position = new Vector3(n / (w - 1), m / (h - 1), 0);
                    vertices[(j + i * w) * 6+5].Color = Color.White;
                    vertices[(j + i * w) * 6+5].TextureCoordinate = new Vector2(n / (w - 1), m / (h - 1));
                   
                    vd = new VertexDeclaration(CursedGame.TheGame.GraphicsDevice, 
                        VertexPositionColorTexture.VertexElements);
                }
            }
        }
        public void applyMap(LifeMap map)
        {
            for (int i = 0; i < h - 1; i++)
            {
                for (int j = 0; j < w - 1; j++)
                {
                    //first triangel |\

                    int m = i;
                    int n = w-(j+1);

                    vertices[(j + i * w) * 6].Color.A = (byte)(255 * map[m, n]);

                    m = i + 1;
                    n = w - (j + 1) - 1;

                    vertices[(j + i * w) * 6 + 1].Color.A = (byte)(255 * map[m, n]);
                   
                    m = i + 1;
                    n = w - (j + 1);

                    vertices[(j + i * w) * 6 + 2].Color.A = (byte)(255 * map[m, n]);
                    //second triangle \|
                    m = i;
                    n = w - (j + 1);

                    vertices[(j + i * w) * 6 + 3].Color.A = (byte)(255 * map[m, n]);
                    m = i;
                    n = w - (j + 1) - 1;

                    vertices[(j + i * w) * 6 + 4].Color.A = (byte)(255 * map[m, n]);
                    m = i + 1;
                    n = w - (j + 1) - 1;

                    vertices[(j + i * w) * 6 + 5].Color.A = (byte)(255 * map[m, n]);
                    
                }
            }
        }
        public int GridWidth
        {
            get { return w; }
            set
            {
                w = value;
                updateVertices();
            }
        }
        
        public int GridHeight
        {
            get { return h; }
            set
            {
                h = value;
                updateVertices();
            }
        }
        public void setGrid(int h, int w)
        {
            this.h = h;
            this.w = w;
            updateVertices();
        }
    }
}
