﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Cursed.view
{
    public enum MenuType{
        MAIN_MENU,
        PAUSE,
        LOSE_PROGRESS_CONFIRMATION,
        EXIT_CONFIRMATION
    }
    public class Menu
    {
        List<MenuButton> buttons = new List<MenuButton>();
        MenuType type;
        Texture2D background;

        public Menu(MenuType type, CursedGame game)
        {
            this.type = type;
            switch (type)
            {
                case MenuType.EXIT_CONFIRMATION:
                    {
                    } break;
                case MenuType.LOSE_PROGRESS_CONFIRMATION:
                    {
                    } break;
                case MenuType.PAUSE:
                    {
                        background = CursedGame.textureLib.GetTexture("square");
                        AddButton(new MenuButton(
                            "Resume",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1-Const.menuButtonWidth)/2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight*3)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.RESUME));
                        AddButton(new MenuButton(
                            "Restart Level",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 4)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.RESTART_LEVEL));
                        AddButton(new MenuButton(
                            "Main Menu",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 5)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.MAIN_MENU));
                        AddButton(new MenuButton(
                            "Quit Game",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 6)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.QUIT_GAME));
                    } break;
                case MenuType.MAIN_MENU:
                    {
                        background = CursedGame.textureLib.GetTexture("square");
                        AddButton(new MenuButton(
                            "New Game",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 4)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.NEW_GAME));
                        AddButton(new MenuButton(
                            "Tutorial",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 5)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.TUTORIAL));
                        AddButton(new MenuButton(
                            "Quit",
                            Color.White,
                            Color.Black,
                            game.Content.Load<SpriteFont>("fonts/FPS"),
                            new Rectangle(
                                (int)(game.GraphicsDevice.Viewport.Width * (1 - Const.menuButtonWidth) / 2f),
                                (int)(game.GraphicsDevice.Viewport.Height * (Const.menuButtonHeight * 6)),
                                (int)(game.GraphicsDevice.Viewport.Width * Const.menuButtonWidth),
                                (int)(game.GraphicsDevice.Viewport.Height * Const.menuButtonHeight)),
                            ButtonType.QUIT_GAME));
                    } break;
            }
        }

        private MenuButton pressedButton(Vector2 pos)
        {
            foreach (MenuButton button in buttons)
            {
                if(button.checkPressed(pos))
                    return button;
            }
            return null;
        }

        public void AddButton(MenuButton button)
        {
            buttons.Add(button);
        }

        public void Update(CursedGame game, MouseState mouse)
        {
            if (mouse.LeftButton == ButtonState.Pressed || GamePad.GetState(PlayerIndex.One).IsButtonDown(Buttons.A))
            {
                foreach (MenuButton button in buttons)
                {
                    if (button.checkPressed(new Vector2(mouse.X, mouse.Y)))
                        button.IsPressed = true;
                    else
                        button.IsPressed = false;
                }
            }
            else
            {
                foreach (MenuButton button in buttons)
                {
                    if (button.IsPressed && button.checkPressed(new Vector2(mouse.X, mouse.Y)))
                    {
                        button.OnPressed(game);
                    }
                    button.IsPressed = false;
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            if (background == null) return;
            if (type == MenuType.PAUSE)
            {
                spriteBatch.Draw(
                background,
                new Rectangle(
                    0,
                    0,
                    spriteBatch.GraphicsDevice.Viewport.Width,
                    spriteBatch.GraphicsDevice.Viewport.Height),
                new Color(0, 0, 0, .5f));
                spriteBatch.Draw(
                    background,
                    new Rectangle(
                        (int)(.35 * spriteBatch.GraphicsDevice.Viewport.Width + .5),
                        0,
                        (int)(.3f * spriteBatch.GraphicsDevice.Viewport.Width + .5),
                        spriteBatch.GraphicsDevice.Viewport.Height),
                    new Color(0, 0, 0, .5f));
            }
            else
            {
                spriteBatch.Draw(
                background,
                new Rectangle(
                    0,
                    0,
                    spriteBatch.GraphicsDevice.Viewport.Width,
                    spriteBatch.GraphicsDevice.Viewport.Height),
                new Color(0, 0, 0, 1f));
            }
            foreach (MenuButton button in buttons)
            {
                button.Draw(spriteBatch);
            }
        }
    }
}
