﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Cursed.gob.mob.kob;

namespace Cursed.view
{
    public enum ButtonType
    {
        RESUME,
        RESTART_LEVEL,
        MAIN_MENU,
        QUIT_GAME,
        NEW_GAME,
        TUTORIAL
    }

    public class MenuButton
    {
        string text;
        Color color;
        Color textColor;
        SpriteFont font;
        bool isPressed = false;
        ButtonType type;

        Rectangle bounds;

        public string Text{
            get{ return text;}
        }

        public bool IsPressed
        {
            get { return isPressed; }

            set { isPressed = value; }
        }

        public Color Color
        {
            get { return color; }
        }

        public Rectangle Bounds
        {
            get { return bounds; }
        }

        public Vector2 Center
        {
            get { return new Vector2(bounds.X + bounds.Width * .5f, bounds.Y + bounds.Height * .5f); }
        }

        public Rectangle TextBounds
        {
            get
            {
                return new Rectangle(
                    (int)(Center.X - font.MeasureString(text).X * .5f),
                    (int)(Center.Y - font.MeasureString(text).Y * .5f),
                    (int)(font.MeasureString(text).X),
                    (int)(font.MeasureString(text).Y));
            }
        }

        public MenuButton(string text, Color color, Color textColor, SpriteFont font, Rectangle bounds, ButtonType type)
        {
            this.text = text;
            this.color = color;
            this.textColor = textColor;
            this.font = font;
            this.bounds = bounds;
            this.type = type;
        }

        public void OnPressed(CursedGame game)
        {
            switch (type)
            {
                case ButtonType.MAIN_MENU:
                    {
                        game.MainMenu();
                        game.Paused = false;
                        game.currentMenu = null;
                    } break;
                case ButtonType.NEW_GAME:
                    {
                        DruidInfo druidInfo = new DruidInfo();
                        druidInfo.IsPlayer = true;
                        game.DruidPlayer = (Druid)druidInfo.Create(game);
                        game.SpawnLabel = "start";
                        game.LoadLevel("finallevel0");
                        game.currentMenu = null;
                    } break;
                case ButtonType.TUTORIAL:
                    {
                        DruidInfo druidInfo = new DruidInfo();
                        druidInfo.IsPlayer = true;
                        game.DruidPlayer = (Druid)druidInfo.Create(game);
                        game.SpawnLabel = "start";
                        game.LoadLevel("tutorial");
                        game.currentMenu = null;
                    } break;
                case ButtonType.QUIT_GAME:
                    {
                        game.currentMenu = new Menu(MenuType.EXIT_CONFIRMATION, game);
                        game.Exit();
                    } break;
                case ButtonType.RESTART_LEVEL:
                    {
                        game.ReloadLevel();
                        game.Paused = false;
                        game.currentMenu = null;
                    } break;
                case ButtonType.RESUME:
                    {
                        game.Paused = false;
                        game.currentMenu = null;
                    } break;
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(
                CursedGame.textureLib.GetTexture("button"),
                bounds,
                !isPressed ? color : new Color(color.R * .5f, color.G * .5f, color.B * .5f));
            spriteBatch.DrawString(
                font,
                text,
                new Vector2(
                    TextBounds.X,
                    TextBounds.Y),
                !isPressed ? textColor : new Color(textColor.R * .5f, textColor.G * .5f, textColor.B * .5f));
        }

        public bool checkPressed(Vector2 pos)
        {
            return pos.X > bounds.X && pos.X < bounds.X + bounds.Width && pos.Y > bounds.Y && pos.Y < bounds.Y + bounds.Height; 
        }
    }
}
