﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.view.renders;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.view
{
    class RenderPresets
    {
        static Bounds unitSquareBounds = new Bounds(Vector2.Zero, 1, 1);
        public static RenderInfo GetPreset(string name)
        {
            RenderInfo info;
            switch (name)
            {
                case "StoneFloor": case "WoodFloor":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("WoodFloor");
                    info.Mask = Const.PrimaryBrown;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "StoneWall":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("Wall");
                    info.Mask = new Color(.5f, .5f, .5f, 1f);
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "StoneDoor":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("gate");
                    info.Mask = new Color(.5f, .5f, .5f, 1f);
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Grass": case "DeadGround":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("ground");
                    info.Mask = Const.PrimaryGreen;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Familiar0":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("spirit");
                    info.Mask = Const.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "EvilFamiliar":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("spirit");
                    info.Mask = Const.Black;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Familiar1":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("spirit");
                    info.Mask = Const.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Demon":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("spirit");
                    info.Mask = Const.PrimaryPurple;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "DemonArm":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("circle");
                    info.Mask = Const.PrimaryPurple;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "BloodStain":
                    info = new PatchRenderInfo(unitSquareBounds, 3, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("BloodSheet");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Flowers":
                    info = new PatchRenderInfo(unitSquareBounds, 4, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("flowers");
                    int c = CursedGame.Random.Next(5);
                    switch (c)
                    {
                        case 0:
                            info.Mask = Const.LightPurple1; break;
                        case 1:
                            info.Mask = Const.LightPurple2; break;
                        case 2:
                            info.Mask = Const.Pink1; break;
                        case 3:
                            info.Mask = Const.LightBrown1; break;
                        case 4:
                            info.Mask = Const.Pink2; break;
                    }
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "FootSwitch":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Mask = Color.Gray;
                    info.Texture = CursedGame.textureLib.GetTexture("switch");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info; 
                case "Spark":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("crystal");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Druid":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Mask = Const.DarkGreen1;
                    info.Texture = CursedGame.textureLib.GetTexture("druid");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "EvilDruid":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Mask = new Color(.2f, .2f, .2f);
                    info.Texture = CursedGame.textureLib.GetTexture("druid");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Portal":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("Portal");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "Tree":
                    info = new SpriteRenderInfo(new Bounds(-1f, -1f, 1f, 1f));
                    info.Texture = CursedGame.textureLib.GetTexture("tree");
                    info.Mask = Const.DarkGreen1;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "Shrub":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("shrub");
                    info.Mask = Const.DarkGreen1;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "ShallowWater":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("water");
                    info.Mask = Const.TransparentBlue;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "DeepWater":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("water");
                    info.Mask = Const.Blue;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "BridgeNS":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("bridge_ns");
                    info.Mask = Const.LightBrown1;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "BridgeEW":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("bridge_ew");
                    info.Mask = Const.LightBrown1;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "Rock":
                    info = new SpriteRenderInfo(new Bounds(-.5f, -.5f, .5f, .5f));
                    info.Texture = CursedGame.textureLib.GetTexture("rock");
                    info.Mask = Const.Gray;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                //stickers
                case "ExperimentalLevels":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 7, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/experimental_levels");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TechDemo":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 7, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tech_demo");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TheGame":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 4, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/the_game");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "CastWood":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/cast_wood");
                    info.Mask = Const.PrimaryBrown;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "SummonInstructions":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/summon_instructions");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "AttackInstructions":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/attack_instructions");
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "SummonFrame":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 1.5f, 1.5f));
                    info.Texture = CursedGame.textureLib.GetTexture("circle");
                    info.Mask = new Color(.25f, .25f, .25f, 0.5f);
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "FamiliarDismiss":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("circle");
                    info.Mask = Microsoft.Xna.Framework.Graphics.Color.Red;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "FamiliarTeleport":
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("circle");
                    info.Mask = Microsoft.Xna.Framework.Graphics.Color.Blue;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "LifeCrystal":
                    info = new SpriteRenderInfo(unitSquareBounds);
                    info.Texture = CursedGame.textureLib.GetTexture("crystal");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                case "TutWASD":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/wasd");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutExplore":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/explore");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutSwamp":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/swamp");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutRiver":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/river");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutWoods":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 1));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/woods");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutAttack":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 8, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/attack");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutDoor":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/door");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutWood":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/wood");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutRecall":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/recall");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutPuzzle":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/puzzle");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
                case "TutAdventure":
                    info = new SpriteRenderInfo(new Bounds(Vector2.Zero, 5, 2));
                    info.Texture = CursedGame.textureLib.GetTexture("stickers/tutorial/adventure");
                    info.Mask = Color.White;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;

                default:
                    info = new PatchRenderInfo(unitSquareBounds, 1, 1);
                    info.Texture = CursedGame.textureLib.GetTexture("ground");
                    info.Mask = Const.PrimaryGreen;
                    info.Render = CursedGame.TheGame.getRender(Cursed.view.renders.Render.RenderType.SpriteRender);
                    return info;
            }
            return null;
        }
    }
}
