﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Windows;

namespace Cursed.view
{
    public class SpriteRender : Render
    {
        private VertexDeclaration vd;

        public VertexDeclaration Vd{
            get { return vd; }
        }


        public SpriteRender(GraphicsDevice gd, Effect effect) 
            : base(effect, gd)
        {

            vd = new VertexDeclaration(gd, VertexPositionColorTexture.VertexElements);
        }


        public override void Draw(GameTime time, RenderInfo info)
        {
            ((SpriteRenderInfo)info).SetVertices();
            effect.Parameters["ShaderIndex"].SetValue(3);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                base.gd.VertexDeclaration = vd;
                base.gd.DrawUserPrimitives(PrimitiveType.TriangleStrip, ((SpriteRenderInfo) info).Vertices, 0, 2);
                pass.End();
            }
        }
    }

    public class SpriteRenderInfo : RenderInfo
    {
        private VertexPositionColorTexture[] vertices;
        Rect bounds;

        public VertexPositionColorTexture[] Vertices
        {
            get { return vertices; }
        }
        
        public SpriteRenderInfo(Rect bounds)
        {
            this.bounds = bounds;

            SetVertices();
        }

        public Rect Bounds
        {
            get { return bounds; }
            set
            {
                bounds = value;
                SetVertices();
            }
        }

        public void SetVertices()
        {
            vertices = new VertexPositionColorTexture[4];
            vertices[0].Position = new Vector3((float)bounds.X, (float)(bounds.Y + bounds.Height), 0);
            vertices[1].Position = new Vector3((float)bounds.X, (float)bounds.Y, 0);
            vertices[2].Position = new Vector3((float)(bounds.X + bounds.Width), (float)(bounds.Y + bounds.Height), 0);
            vertices[3].Position = new Vector3((float)(bounds.X + bounds.Width), (float)bounds.Y, 0);

            vertices[0].TextureCoordinate = new Vector2(0, 1);
            vertices[1].TextureCoordinate = Vector2.Zero;
            vertices[2].TextureCoordinate = new Vector2(1, 1);
            vertices[3].TextureCoordinate = new Vector2(1, 0);

            vertices[0].Color = Mask;
            vertices[1].Color = Mask;
            vertices[2].Color = Mask;
            vertices[3].Color = Mask;
        }
    }
}
