﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Cursed.geom;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.view.renders
{
    public class AnimatedSpriteRenderInfo:SpriteRenderInfo
    {
        string state = "walk";
        float direction = 0;
        SpriteRenderInfo renderInfo = (SpriteRenderInfo)RenderPresets.GetPreset("DemonWalking");

        public AnimatedSpriteRenderInfo(Bounds bounds)
            : base(bounds)
        {
        }

        public SpriteRenderInfo CurrentRenderInfo
        {
            get { return renderInfo; }
        }

        public void Update(Vector2 position, Vector2 dir, GameTime gameTime)
        {
            this.direction = (float)Math.Atan2(dir.Y, dir.X);
            if (state == ("walk"))
                ((DistanceBasedRenderInfo)renderInfo).Update(position, dir);
            if (state==("attack"))
            {
                ((SimpleTimeAnimationInfo)renderInfo).Update(direction, gameTime);
                if (((SimpleTimeAnimationInfo)renderInfo).FinishedAnimating)
                {
                    ChangeState("walk");
                }
            }
            renderInfo.Mask = Mask;
            renderInfo.Bounds = Bounds;
            renderInfo.Texture = Texture;
            renderInfo.Transform = Transform;
        }

        public string CurrentState
        {
            get { return state; }
        }

        public void ChangeState(string newState)
        {
            switch (newState)
            {
                case "walk":
                    {
                        state = "walk";
                        renderInfo = (DistanceBasedRenderInfo)RenderPresets.GetPreset("DemonWalking");
                    } break;

                case "attack":
                    {
                        state = "attack";
                        renderInfo = (SimpleTimeAnimationInfo)RenderPresets.GetPreset("DemonAttack");
                    } break;
            }
        }

        public override void SetVertices()
        {
            switch (state)
            {
                case "walk":
                    {
                        ((DistanceBasedRenderInfo)renderInfo).SetVertices();
                    } break;

                case "attack":
                    {
                        ((SimpleTimeAnimationInfo)renderInfo).SetVertices();
                    } break;
            }
            renderInfo.SetVertices();
            vertices = renderInfo.Vertices;
        }
    }
}
