﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Cursed.geom;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Cursed.view.renders
{
    public class DistanceBasedRenderInfo : SpriteRenderInfo
    {
        float cycleDistance;
        float distanceTraveled;
        Vector2 lastPosition;
        public bool IsAnimating = false;
        float direction = 0;
        Vector2 lastSpot;

        int w, h;

        public DistanceBasedRenderInfo(Bounds bounds, float cycleDistance, int w, int h)
            : base(bounds)
        {
            this.cycleDistance = cycleDistance;
            this.w = w;
            this.h = h;
        }

        public static int DirectionOffset(float direction)
        {
                if (direction >= -5 * Math.PI / 8 && direction < -3 * Math.PI / 8)
                    return 0;
                else if (direction >= 3 * Math.PI / 8 && direction < 5 * Math.PI / 8)
                    return 28;
                else if (direction >= 7 * Math.PI / 8 || direction < -7 * Math.PI / 8)
                    return 14;
                else if (direction >= -1 * Math.PI / 8 && direction < 1 * Math.PI / 8)
                {
                    return 14;
                }
                else if (direction >= 5 * Math.PI / 8 && direction < 7 * Math.PI / 8)
                    return 21;
                else if (direction >= 1 * Math.PI / 8 && direction < 3 * Math.PI / 8)
                {
                    return 21;
                }
                else if (direction >= -7 * Math.PI / 8 && direction < -5 * Math.PI / 8)
                    return 7;
                else if (direction >= -3 * Math.PI / 8 && direction < -1 * Math.PI / 8)
                {
                    return 7;
                }
                return 0;
        }

        public static bool IsMirrored(float direction)
        {
            if (direction >= -1 * Math.PI / 8 && direction < 1 * Math.PI / 8)
                return true;
            if (direction >= 1 * Math.PI / 8 && direction < 3 * Math.PI / 8)
                return true;
            if (direction >= -3 * Math.PI / 8 && direction < -1 * Math.PI / 8)
                return true;
            return false;
        }

        public void Update(Vector2 position, Vector2 dir)
        {
            if (lastPosition == null)
                lastPosition = position;
            
            Vector2 motion = position - lastPosition;
            if (motion.Length() <= 0.01f)
            {
                IsAnimating = false;
            }
            else
            {
                IsAnimating = true;
               
                if ((lastSpot - position).Length() > .25f)
                {
                    direction = (float)Math.Atan2(dir.Y, dir.X);
                    lastSpot = position;
                }
            }

            //distanceTraveled += motion.Length();
            if (dir != Vector2.Zero)
                distanceTraveled += Vector2.Dot(motion, dir) / dir.Length();

            if (!IsAnimating)
            {
                SetDrawnFrame(DirectionOffset(direction), 1, w, h);
                imageMirrored = IsMirrored(direction);
            }
            else
            {
                distanceTraveled %= cycleDistance;
                int frameToDraw = (int)((distanceTraveled / cycleDistance) * 4);

                if(frameToDraw == 1)
                {
                    frameToDraw = 2;
                }
                else if (frameToDraw == 3)
                {
                }
                else
                {
                    frameToDraw = 1;
                }


                SetDrawnFrame(DirectionOffset(direction) + frameToDraw, 1, w, h);
                imageMirrored = IsMirrored(direction);
            }
            lastPosition = position;
        }
    }
}
