﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Cursed.geom;
using Microsoft.Xna.Framework;

namespace Cursed.view.renders
{
    public class SimpleTimeAnimationInfo : SpriteRenderInfo
    {
        private float elapsedSeconds = 0;
        private float duration;
        private int w, h;
        List<Point> frameProgression;

        public SimpleTimeAnimationInfo(Bounds bounds, float duration, int w, int h, List<Point> frameProgression)
            : base(bounds)
        {
            this.w = w;
            this.h = h;
            this.duration = duration;
            this.frameProgression = frameProgression;
        }

        public SimpleTimeAnimationInfo(Bounds bounds, float duration, int w, int h)
            : base(bounds)
        {
            this.w = w;
            this.h = h;
            this.duration = duration;
            this.frameProgression = new List<Point>();
            for (int i = 0; i < w; i++)
            {
                frameProgression.Add(new Point(i, 0));
            }

        }

        public bool FinishedAnimating
        {
            get { return elapsedSeconds > duration; }
        }

        public void Update(float direction, GameTime gameTime)
        {
            elapsedSeconds += (float)gameTime.ElapsedGameTime.TotalSeconds;
            this.SetDrawnFrame(
                frameProgression[Math.Min((int)(elapsedSeconds / duration * frameProgression.Count), frameProgression.Count - 1)].X + DistanceBasedRenderInfo.DirectionOffset(direction),
                frameProgression[Math.Min((int)(elapsedSeconds / duration * frameProgression.Count), frameProgression.Count - 1)].Y,
                w,
                h);
            this.imageMirrored = DistanceBasedRenderInfo.IsMirrored(direction);
        }

        public float ElapsedSeconds
        {
            get { return elapsedSeconds; }
            set
            {
                elapsedSeconds = Math.Max(0.0f, value);
                this.SetDrawnFrame(
                    frameProgression[(int)(elapsedSeconds / duration * frameProgression.Count)].X, 
                    frameProgression[(int)(elapsedSeconds / duration * frameProgression.Count)].Y, 
                    w, 
                    h);
                
            }
        }

    }
}
