﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using System.Windows;
using Cursed.geom;

namespace Cursed.view.renders
{
    public class SpriteRender : Render
    {
        private VertexDeclaration vd;
        private Texture2D lastTexture;

        public VertexDeclaration Vd{
            get { return vd; }
        }

        public SpriteRender(GraphicsDevice gd, Effect effect) 
            : base(effect, gd)
        {
            lastTexture = null;
            vd = new VertexDeclaration(gd, VertexPositionColorTexture.VertexElements);
        }

        public override void Begin(RenderInfo info)
        {
            base.Begin(info);
            lastTexture = null;
        }

        public override void End()
        {
            base.End();
            lastTexture = null;
        }

        public override void Draw(GameTime time, RenderInfo info)
        {
            if (lastTexture != info.Texture)
            {
                lastTexture = info.Texture;
                effect.Parameters["BasicTexture"].SetValue(info.Texture);
            }

            if (info.Transform != null)
            {
                effect.Parameters["World"].SetValue(info.Transform);
            }
            ((SpriteRenderInfo)info).SetVertices();
            effect.Parameters["ShaderIndex"].SetValue(3);
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();
                CursedGame.TheGame.GraphicsDevice.VertexDeclaration = vd;
                CursedGame.TheGame.GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleStrip, ((SpriteRenderInfo) info).Vertices, 0, 2);
                pass.End();
            }
        }
    }

    public class SpriteRenderInfo : RenderInfo, ICloneable
    {
        protected VertexPositionColorTexture[] vertices;
        protected Bounds bounds;
        protected bool imageMirrored = false;

        protected Bounds drawnFrame = new Bounds(0, 0, 1, 1);

        public float scaleX = 1, scaleY = 1;

        public SpriteRenderInfo(Bounds bounds)
        {
            this.bounds = bounds;

            SetVertices();
        }
        public void SetDrawnFrame(int x, int y, int w, int h)
        {
            drawnFrame = new Bounds((float)x / (float)w, (float)y / (float)h, (float)(x + 1) / (float)w, (float)(y + 1) / (float)h);
        }
        public virtual VertexPositionColorTexture[] Vertices
        {
            get 
            {
                SetVertices();
                return vertices; 
            }
        }
        

        public Bounds Bounds
        {
            get { return bounds; }
            set
            {
                bounds = value;
                SetVertices();
            }
        }

        public virtual void SetVertices()
        {

            vertices = new VertexPositionColorTexture[4];
            vertices[0].Position = new Vector3(bounds.MinX * scaleX, bounds.MaxY * scaleY, 0);
            vertices[1].Position = new Vector3(bounds.MinX * scaleX, bounds.MinY * scaleY, 0);
            vertices[2].Position = new Vector3(bounds.MaxX * scaleX, bounds.MaxY * scaleY, 0);
            vertices[3].Position = new Vector3(bounds.MaxX * scaleX, bounds.MinY * scaleY, 0);

            if (!imageMirrored)
            {
                vertices[0].TextureCoordinate = new Vector2(drawnFrame.MinX, drawnFrame.MinY);
                vertices[1].TextureCoordinate = new Vector2(drawnFrame.MinX, drawnFrame.MaxY);
                vertices[2].TextureCoordinate = new Vector2(drawnFrame.MaxX, drawnFrame.MinY);
                vertices[3].TextureCoordinate = new Vector2(drawnFrame.MaxX, drawnFrame.MaxY);
            }
            else
            {
                vertices[0].TextureCoordinate = new Vector2(drawnFrame.MaxX, drawnFrame.MinY);
                vertices[1].TextureCoordinate = new Vector2(drawnFrame.MaxX, drawnFrame.MaxY);
                vertices[2].TextureCoordinate = new Vector2(drawnFrame.MinX, drawnFrame.MinY);
                vertices[3].TextureCoordinate = new Vector2(drawnFrame.MinX, drawnFrame.MaxY);
            }
            vertices[0].Color = Mask;
            vertices[1].Color = Mask;
            vertices[2].Color = Mask;
            vertices[3].Color = Mask;
        }

        #region ICloneable Members

        public object Clone()
        {
            SpriteRenderInfo clone = new SpriteRenderInfo(this.bounds);
            clone.Mask = this.Mask;
            clone.Texture = this.Texture;
            clone.Transform = this.Transform;
            clone.drawnFrame = this.drawnFrame;
            clone.imageMirrored = this.imageMirrored;

            return clone;
        }

        #endregion
    }
}
