﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace Cursed.view.renders
{
    public class TrailRender : Render
    {
        VertexDeclaration vd;

        public TrailRender(Effect effect, GraphicsDevice gd) : base(effect, gd)
        {
            vd = new VertexDeclaration(gd, VertexPositionColorTexture.VertexElements);
        }

        public override void Begin(RenderInfo info){}

        public override void End(){}



        public override void Draw(Microsoft.Xna.Framework.GameTime gt, RenderInfo info)
        {
            effect.Parameters["ShaderIndex"].SetValue(3);
            foreach (SpriteRenderInfo frameRenderInfo in ((TrailRenderInfo)info).renderInfo)
            {
                effect.Parameters["BasicTexture"].SetValue(frameRenderInfo.Texture);
                frameRenderInfo.Transform = Matrix.CreateScale(.95f) * frameRenderInfo.Transform;
                frameRenderInfo.Mask = new Color(frameRenderInfo.Mask.R, frameRenderInfo.Mask.G, frameRenderInfo.Mask.B, (byte)(frameRenderInfo.Mask.A * .8f));
                frameRenderInfo.SetVertices();
                effect.Parameters["World"].SetValue(frameRenderInfo.Transform);
                effect.Begin();
                foreach (EffectPass pass in effect.CurrentTechnique.Passes)
                {
                    pass.Begin();
                    base.gd.VertexDeclaration = vd;
                    base.gd.DrawUserPrimitives(PrimitiveType.TriangleStrip, frameRenderInfo.Vertices, 0, 2);
                    pass.End();
                }
                effect.End();
            }
        }
    }

    public class TrailRenderInfo : RenderInfo
    {
        public List<SpriteRenderInfo> renderInfo;

        public TrailRenderInfo()
        {
            this.renderInfo = new List<SpriteRenderInfo>();
        }

        public void Update(SpriteRenderInfo renderInfo)
        {
            this.renderInfo.Add((SpriteRenderInfo)renderInfo.Clone());
            if (this.renderInfo.Count > 10)
            {
                this.renderInfo.Remove(this.renderInfo[0]);
            }
        }
    }
}
